#include "DrawNetwork.h"

//SDL_Color structures
SDL_Color black = {0x00,0x00,0x00,0};
SDL_Color red = {0xFF,0x00,0x00,0};
SDL_Color blue = {0x00,0x00,0xFF,0};
SDL_Color green = {0x00,0xFF,0x00,0};
SDL_Color light_blue = {0x00,0xFF,0xFF,0};
SDL_Color purple = {0xFF,0x00,0xFF,0};
SDL_Color yellow = {0xFF,0xFF,0x00,0};
SDL_Color grey = {0x99,0x99,0x99,0};
SDL_Color white = {0xFF,0xFF,0xFF,0};

//network graphic methods
void graph(const Organism* org) {

	//used surfaces
	SDL_Surface* screen;
	SDL_Surface* screen_backup;

	//the system video is initialized 
	if (SDL_Init(SDL_INIT_VIDEO) <0) {
		//SDL_GetError() returns a description of the error
		printf("SDL Init Error: %s\n", SDL_GetError());
		return;
	}

	//when exit, execute SDL_Quit to restore everything
	atexit(SDL_Quit);
	
	//a 32 bit integer used to store screen's flags 
	Uint32 flags = SDL_HWSURFACE|SDL_DOUBLEBUF;

	//SDL_SetVideoMode is used to have screen associated to the monitor 
	if (!(screen = SDL_SetVideoMode(DIM_H, DIM_V, 0,flags))) {
		printf("Screen Init Error : %s\n", SDL_GetError());
		return;
	}

	//white background
	fill_square(screen, NULL, white);

	//screen_backup is a surface of the same dimensions of screen used  to save the image when the user
	//changes its form (window to full screen and vice versa). 
	screen_backup = SDL_CreateRGBSurface(flags,screen->w,screen->h,screen->format->BitsPerPixel,0,0,0,0);
	
	//***********************************
	//here we call the drawing function
	//**********************************
	draw_net(org,screen);
	
	//SDL_Event struct used to manage events
	SDL_Event event;

	//variables used to control the game loop 
	bool done = false;
	bool paused = false;

	//game or main loop (as you prefer)
	while (!done) {
		//SDL_PollEvent manages user's events 
		while (SDL_PollEvent(&event)) {
			
			//the program quits if someone press the x symbol of the window with the mouse 
			if (event.type == SDL_QUIT)  
				done = true;  

			if (event.type == SDL_KEYDOWN) {
				//it quits also if someone press ESC 
				if (event.key.keysym.sym == SDLK_ESCAPE) 
					done = true; 
				else if (event.key.keysym.sym == SDLK_p) 
					paused = !paused ;

				//visualization form is changed if someone press SPACE 
				//(from window to full screen and vice versa)
				if (event.key.keysym.sym == SDLK_SPACE) {
					//the actual screen is saved in screen_backup
					SDL_BlitSurface(screen,NULL,screen_backup,NULL);
					//visualization form is changed
					flags^=SDL_FULLSCREEN;
					screen = SDL_SetVideoMode(DIM_H, DIM_V, 0,flags);
					//the original image is reBlitted
					SDL_BlitSurface(screen_backup,NULL,screen,NULL);
				}
			}
		}

		//the screen is refreshed
		SDL_Flip(screen);

		//a minimum delay 
		SDL_Delay(1);
	}
}

// this function does the real job inside the window just created
void draw_net(const Organism* org,SDL_Surface* surface) {
	list<Neuron*>::const_iterator curnode;
	list<IncomingLink*>::const_iterator curlink;
	// idea: neurons are sorted by layers, the outputs first;
	// we ignore loops which come back to previously marked nodes
	list< list<Neuron*> > layers;
	list< list<Neuron*> >::iterator layers_iter;
	// initialization: visited=false for each neuron, and give them a label (input, output or empty)
	for (curnode=org->inputs.begin(); curnode!=org->inputs.end(); curnode++) {
		(*curnode)->visited=false;
		(*curnode)->label="input";
	}
	for (curnode=org->hidden_nodes.begin(); curnode!=org->hidden_nodes.end(); curnode++) {
		(*curnode)->visited=false;
		(*curnode)->label="";
	}
	for (curnode=org->outputs.begin(); curnode!=org->outputs.end(); curnode++) {
		(*curnode)->visited=false;
		(*curnode)->label="output";
	}
	
	// initialization of the layers with the outputs:
	list<Neuron*> layer;
	list<int> widths;
	for (curnode=org->outputs.begin(); curnode!=org->outputs.end(); curnode++) {
		(*curnode)->visited=true;
		layer.push_back(*curnode);
	}
	widths.push_back(layer.size());
	layers.push_back(layer);
	double max_weight_link=0.0;
	// loop: while some neurons are missing, go on
	while (true) {
		layer.clear();
		for (curnode=layers.back().begin(); curnode!=layers.back().end(); curnode++) {
			if ((*curnode)->label=="input") continue; // inputs are treated at the end
			for (curlink=(*curnode)->incoming.begin(); curlink!=(*curnode)->incoming.end(); curlink++) {
				if (!(*curlink)->node->visited) {
					(*curlink)->node->visited=true;
					layer.push_back((*curlink)->node);
				}
				if (fabs((*curlink)->weight)>max_weight_link) max_weight_link=fabs((*curlink)->weight);
			}
		}
		if (layer.empty()) break;
		widths.push_back(layer.size());
		layers.push_back(layer);
	}

	list<Neuron*>::iterator curnode_,checknode,pushednode;
	list<IncomingLink*>::iterator curlink_;
	list<int>::iterator wid_it;
	bool rearranged=false,foundLink;
	// then rearrange the layers to get rid of the horizontal connections
	while (!rearranged) {
		rearranged=true;
		layers_iter=layers.begin();
		wid_it=widths.begin();
		while (layers_iter!=layers.end()) {
			layer.clear();
			curnode_=(*layers_iter).begin();
			while (curnode_!=(*layers_iter).end()) {
				foundLink=false;
				// check if this node connects one of the visited nodes from the layer in exploration
				// if yes, we create a new layer for him and his homologues
				for (curlink_=(*curnode_)->incoming.begin(); curlink_!=(*curnode_)->incoming.end(); curlink_++) {
					checknode=(*layers_iter).begin();
					while (checknode!=curnode_) {
						if ((*checknode)->id==(*curlink_)->node->id) {
							layer.push_back(*curnode_);
							// erase the node pushed back to the next layer:
							pushednode=curnode_;
							curnode_++;
							(*layers_iter).erase(pushednode);
							foundLink=true;
							break;
						}
						checknode++;
					}
					if (foundLink) break;
				}
				if (!foundLink) curnode_++;
			}
			if (!layer.empty()) {
				rearranged=false;
				layers_iter++;
				wid_it++;
				layers.insert(layers_iter,layer);
				widths.insert(wid_it,layer.size());
			}
			layers_iter++;
			wid_it++;
		}
	}
	// add the final layers: the inputs
	layer.clear();
	for (curnode=org->inputs.begin(); curnode!=org->inputs.end(); curnode++) layer.push_back(*curnode);
	layers.push_back(layer);
	widths.push_back(layer.size());

	// some constants initialisations:
	int height=layers.size(); // height (>=2, because inputs and outputs, at least)
	int cell_side=21; // size of a square representing a node 
	int gapH; // horizontal distance between two nodes (will depend on the layer)
	int gapV=(DIM_V/height)-cell_side; // vertical distance between two ndes
	SDL_Rect cell = {0,0,cell_side,cell_side}; // the basic cell to be drawn (a node)
	
	// then we calculate the coordinates before drawing
//	bool* usedAbs=(bool*)calloc(DIM_V,sizeof(bool)); // tells which abs' are available
													// can be used to gain drawing precision later
													// simple method: choose the abs inside the largest abs' gap,
													// taking care of the left/right offsets
	int curAbs,curOrd=max((gapV-cell_side)/2,0); // ord initialized with upper offset
	int counter=0;
	wid_it=widths.begin();
	for (layers_iter=layers.begin(); layers_iter!=layers.end(); layers_iter++) {
		gapH=(DIM_H/(*wid_it))-cell_side;
		int left_offset=max((gapH-cell_side)/2,0),shift_pixels=left_offset/height;
		curAbs=left_offset+(int)(0.5*counter*shift_pixels); // abs initialized with shifted left offset
		for (curnode_=(*layers_iter).begin(); curnode_!=(*layers_iter).end(); curnode_++) {
			(*curnode_)->x=curAbs;
			curAbs+=(cell_side+gapH);
			(*curnode_)->y=curOrd;
		}
		curOrd+=(cell_side+gapV);
		wid_it++;
		counter++;
	}

	// finally: the drawing
	point s,g;
	int shifts; // to draw wide lines for strong connections
	for (layers_iter=layers.begin(); layers_iter!=layers.end(); layers_iter++) {
		for (curnode_=(*layers_iter).begin(); curnode_!=(*layers_iter).end(); curnode_++) {
			cell.x=(*curnode_)->x;
			cell.y=(*curnode_)->y;
			s.y=cell.y+cell_side/2;

			if ((*curnode_)->label=="input") draw_square(surface, &cell, black, blue);
			else if ((*curnode_)->label=="output") draw_square(surface, &cell, black, red);
			else draw_square(surface, &cell, black, black);

			for (curlink_=(*curnode_)->incoming.begin(); curlink_!=(*curnode_)->incoming.end(); curlink_++) {
				g.y=(*curlink_)->node->y+cell_side/2;
				if (!(*curlink_)->enable) { // just a little light line for disable links
					s.x=cell.x+cell_side/2;
					g.x=(*curlink_)->node->x+cell_side/2;
					draw_line(surface,s,g,green);
					continue;
				}
				shifts=(int)(cell_side*(fabs((*curlink_)->weight))/(2.0*max_weight_link));
				while (shifts>=0) {
					s.x=cell.x+cell_side/3+shifts;
					g.x=(*curlink_)->node->x+cell_side/3+shifts;
					if ((*curlink_)->weight>0.0) draw_line(surface,s,g,black);
					else draw_line(surface,s,g,purple);
					shifts--;
				}
			}
		}
	}
}
