#include "Experiments.h"

// respectively the champion, who is a solution,
// and the max_fitness organism (maybe not a solution)
Organism *the_winner,*best_fit_org;

//the generic testing function
Organism** test(char* nameExp) {
	the_winner=0;
	best_fit_org=0;
	Timer tim; // to give length of experiment
	
	char *ifname=0,*startName=0;
	if (!strcmp(nameExp,"xor")) {
		ifname="xorstartgenes.ge";
		startName="START XOR TEST";
	}
	else if (!strcmp(nameExp,"pole1")) {
		ifname="pole1startgenes.ge";
		startName="START SINGLE POLE BALANCING EVOLUTION";
	}
	else if (!strcmp(nameExp,"fight")) {
		ifname="fighterstartgenes.ge";
		startName="START FIGHTERS EVOLUTION";
	}
	else {
		cerr << "unknown experiment" << endl;
		exit(1);
	}
	ifstream iFile(ifname);
	if (!iFile.is_open()) {
		cerr << "error can't find initialisation file for test" << endl;
		exit(EXIT_FAILURE);
	}

	cout << startName << endl;
	//Read in the start Genome
	Organism* first_organism;
	char curword[20];
	iFile >> curword;
	first_organism=new Organism(iFile);
	iFile.close();

	// for statistics:
	int totallinks=0,totalnodes=0,count_win=0;
	
	int winnerlinks,winnernodes,expcount,check_success;
	Population* pop;
	bool found_winner;
    for (expcount=1; expcount<=NEAT::num_runs; expcount++) {
		found_winner=false;
		//Spawn the Population
		Population::trace << "Spawning Population off initial genome" << endl;
		pop=new Population(first_organism,NEAT::pop_size);

		for (int gen=1; gen<=NEAT::generations; gen++) {
			cout << "Epoch " << gen << endl;
			Population::trace << endl << "Epoch " << gen << endl << endl;
			//Check for success
			check_success=epoch(pop,nameExp,winnerlinks,winnernodes);
			if (check_success==2) {
				count_win++;
				found_winner=true;
				//Collect Stats on end of experiment
				totallinks+=winnerlinks;
				totalnodes+=winnernodes;
			}
			else if (check_success==1) break;
			#ifdef FILE_OUT
			//Only print to file every print_every generations or when a winner appear
			if (found_winner || gen%(NEAT::print_every)==0) {
				// create the corresponding filename
				char* fname=(char*)malloc((8+(int)floor(log10((double)gen))+(int)floor(log10((double)expcount)))*sizeof(char));
				int cp_count=gen,index=(int)floor(log10((double)gen));
				while (cp_count>0) {
					fname[index--]='0'+cp_count%10;
					cp_count/=10;
				}
				index=(int)floor(log10((double)gen))+1;
				fname[index++]='_';
				index+=(int)floor(log10((double)expcount));
				cp_count=expcount;
				while (cp_count>0) {
					fname[index++]='0'+cp_count%10;
					cp_count/=10;
				}
				fname[index++]='.';
				fname[index++]='t';
				fname[index++]='x';
				fname[index++]='t';
				fname[index]=0;
				pop->print_to_file(fname);
				free(fname);
			}
			#endif
			if (found_winner) break;
		}
		delete pop;
		Population::trace << endl;
	}
	delete first_organism;

	//Average and print stats
	if (count_win==0) cout << endl << "Sorry, no solution found." << endl 
					<< "Try to change parameters, or increase time search, or disable fastest search." << endl;
	else {
		cout << endl << "Average solution's hidden nodes: " << ((double)totalnodes/count_win) << endl;
		cout << endl << "Average solution's links: " << ((double)totallinks/count_win) << endl;
	}
	cout << endl << "Time search: " << tim.elapsed() << " seconds" << endl;

	Organism** tabOrg=new Organism*[2];
	tabOrg[0]=the_winner;
	tabOrg[1]=best_fit_org;
	return tabOrg;
}

/***************************
 * the evaluation functions
****************************/

bool xor_evaluate(Organism* org) {
	// there we could train the organisms,
	// and later complexify the evaluation to fit video game
	
	double out[4]; //The four outputs (one on each example)
	//The four possible input combinations to xor
	double in[4][2]={{0.0,0.0},
					{0.0,1.0},
					{1.0,0.0},
					{1.0,1.0}};

	//Load and activate the network on each input
	for (int count=0; count<=3; count++) {
		org->load_sensors(in[count]);
		//get output by activating the net at least as much as needed
		for (int i=org->hidden_nodes.size(); i>=0; i--) org->activate();
		//we get the current output
		out[count]=(*(org->outputs.begin()))->activation;
		// useless, but more clean: we reinitialize the network's acivations
		org->flush();
	}

	// estimate the error and fitness
	double errorsum=(fabs(out[0])+fabs(1.0-out[1])+fabs(1.0-out[2])+fabs(out[3]));
	org->fitness=pow((4.0-errorsum),2);
	
	//Decide if its a winner
	org->winner=(out[0]<0.5 && out[1]>=0.5 && out[2]>=0.5 && out[3]<0.5);
	
	// keeps the best fit organism in memory (doesn't necessary solve the problem!)
	if (best_fit_org==0) best_fit_org=new Organism(org);
	else if (org->fitness > best_fit_org->fitness) {
		delete best_fit_org;
		best_fit_org=new Organism(org);
	}

	#ifdef DETAILS
	Population::trace << "Org " << org->id << " fitness: " << org->fitness << " error: "
	<< errorsum << " [" << out[0] << " " << out[1] << " " << out[2] << " " << out[3] << "]" << endl;
	#endif
	
	return org->winner;
}

bool pole1_evaluate(Organism *org) {
	// number of successive successes to get
	int MAX_STEPS=100000;

	//Try to balance a pole now
	org->fitness=go_cart(org,MAX_STEPS);

	//Decide if its a winner
	org->winner=(org->fitness>=MAX_STEPS);
	
	// keeps the best fit organism in memory (doesn't necessary solve the problem!)
	if (best_fit_org==0) best_fit_org=new Organism(org);
	else if (org->fitness > best_fit_org->fitness) {
		delete best_fit_org;
		best_fit_org=new Organism(org);
	}

	#ifdef DETAILS
	Population::trace << "Org " << org->id << " fitness: " << org->fitness << endl;
	#endif

	return org->winner;
}

// cart_and_pole() was taken directly from the pole simulator written
// by Richard Sutton and Charles Anderson.
int go_cart(Organism* org,int max_steps) {
	float x, // cart position, meters
		x_dot, // cart velocity
		theta, // pole angle, radians
		theta_dot; // pole angular velocity
	int steps=0,y;
	bool random_start=true;

	double in[4]; //Input loading array
	double out1;
	double out2;

//	double one_degree= 0.0174532; // 2pi/360
//	double six_degrees=0.1047192;
	double twelve_degrees=0.2094384;
//	double thirty_six_degrees= 0.628329;
//	double fifty_degrees=0.87266;

	if (random_start) {
		//set up random start state
		x=(rand()%4800)/1000.0-2.4;
		x_dot=(rand()%2000)/1000.0-1.0;
		theta=(rand()%400)/1000.0-0.2;
		theta_dot=(rand()%3000)/1000.0-1.5;
	}
	else x=x_dot=theta=theta_dot=0.0;

	//--- Iterate through the action-learn loop. ---
	list<Neuron*>::iterator out_iter;
	while (steps<max_steps) {
		in[0]=(x+2.4)/4.8;
		in[1]=(x_dot+0.75)/1.5;
		in[2]=(theta+twelve_degrees)/0.41;
		in[3]=(theta_dot+1.0)/2.0;
		org->load_sensors(in);

		//Activate the net
		for (int i=org->hidden_nodes.size(); i>=0; i--) org->activate();

		//-- decide which way to push via which output unit is greater --
		out_iter=org->outputs.begin();
		out1=(*out_iter)->activation;
		out_iter++;
		out2=(*out_iter)->activation;
		if (out1>out2) y=0;
		else y=1;

		//--- Apply action to the simulated cart-pole ---
		cart_pole(y, &x, &x_dot, &theta, &theta_dot);

		//--- Check for failure.  If so, return steps ---
		if (x<-2.4 || x>2.4 || theta<-twelve_degrees || theta>twelve_degrees) 
			return steps;

		steps++;
	}

	return steps;
}


// cart_and_pole() was take directly from the pole simulator written
// by Richard Sutton and Charles Anderson.
// This simulator uses normalized, continous inputs instead of 
// discretizing the input space.
//----------------------------------------------------------------------
// cart_pole:  Takes an action (0 or 1) and the current values of the
// four state variables and updates their values by estimating the state
// TAU seconds later.
//----------------------------------------------------------------------
void cart_pole(int action, float* x,float* x_dot, float* theta, float* theta_dot) {
	float xacc,thetaacc,force,costheta,sintheta,temp;

	static const float GRAVITY=9.8;
	static const float MASSCART=1.0;
	static const float MASSPOLE=0.1;
	static const float TOTAL_MASS=(MASSPOLE + MASSCART);
	static const float LENGTH=0.5; // actually half the pole's length
	static const float POLEMASS_LENGTH=(MASSPOLE * LENGTH);
	static const float FORCE_MAG=10.0;
	static const float TAU=0.02;	  // seconds between state updates
	static const float FOURTHIRDS=1.3333333333333;

	force = (action>0) ? FORCE_MAG : -FORCE_MAG;
	costheta=cos(*theta);
	sintheta=sin(*theta);

	temp=(force+POLEMASS_LENGTH*(*theta_dot)*(*theta_dot)*sintheta)/TOTAL_MASS;
	thetaacc=(GRAVITY*sintheta-costheta*temp)/(LENGTH*(FOURTHIRDS-MASSPOLE*costheta*costheta/TOTAL_MASS));
	xacc  = temp - POLEMASS_LENGTH * thetaacc* costheta / TOTAL_MASS;
  
	// Update the four state variables, using Euler's method.
	*x+=TAU*(*x_dot);
	*x_dot+=TAU*xacc;
	*theta+=TAU*(*theta_dot);
	*theta_dot+=TAU*thetaacc;
}

// fighter's evaluation and learning; it's a "solution" if it defeats human,
// or if it is above a fitness threshold
bool evaluate_learn_fighter(Organism* org) {
	//call an external function (too big to be there)
	launch_test_fight(org,best_fit_org);

	// keeps the best fit organism in memory
	if (best_fit_org==0) best_fit_org=new Organism(org);
	else if (org->fitness > best_fit_org->fitness) {
		delete best_fit_org;
		best_fit_org=new Organism(org);
	}

	#ifdef DETAILS
	Population::trace << "Org " << org->id << " fitness: " << org->fitness << endl;
	#endif

	return org->winner;
}

/******************************
 * end of evaluation functions
*******************************/

// generic epoch function
int epoch(Population* pop,char* nameExp,int& winnerlinks,int& winnernodes) {
	list<Organism*>::iterator curorg;
	list<Species*>::iterator curspec;

	//Evaluate each organism on a test
	// we keep the simplest winner (Occam's razor)
	bool win=false;
	for (curorg=pop->organisms.begin(); curorg!=pop->organisms.end(); curorg++) {
		if ((!strcmp(nameExp,"xor") && xor_evaluate(*curorg))
			|| (!strcmp(nameExp,"pole1") && pole1_evaluate(*curorg))
			|| (!strcmp(nameExp,"fight") && evaluate_learn_fighter(*curorg))) {
			win=true;
			if (winnernodes==0) {
				winnerlinks=(*curorg)->nb_links;
				winnernodes=(*curorg)->hidden_nodes.size();
				the_winner=new Organism(*curorg);
			}
			else if ((int)((*curorg)->hidden_nodes.size())<winnernodes 
					|| ((int)((*curorg)->hidden_nodes.size())==winnernodes && (*curorg)->nb_links<winnerlinks)) {
				winnerlinks=(*curorg)->nb_links;
				winnernodes=(*curorg)->hidden_nodes.size();
				delete the_winner;
				the_winner=new Organism(*curorg);
			}
			break;
		}
	}

	// go to the next generation, or exit if population extinction / time over:
	if (win) return 2;
	if (!pop->epoch(NEAT::search_time/NEAT::num_runs)) return 1;
	return 0;
}
