#include "Organism.h"

//Activates the net such (maybe all outputs will not be active if some reproduction has been really unlucky;
// in this case the output is simply SIGMOID(0))
void Organism::activate() {
	list<Neuron*>::iterator curnode;
	list<IncomingLink*>::iterator curlink;
	/* For each node, compute the sum of its incoming activations */
	for (curnode=hidden_nodes.begin(); curnode!=hidden_nodes.end(); curnode++) {
		(*curnode)->incoming_sum=0.0;
		/* For each incoming connection, add the activity from the connection to the activesum */
		for (curlink=(*curnode)->incoming.begin(); curlink!=(*curnode)->incoming.end(); curlink++) {
			if ((*curlink)->enable)
				(*curnode)->incoming_sum+=((*curlink)->weight)*((*curlink)->node->activation);
		}
	}
	for (curnode=outputs.begin(); curnode!=outputs.end(); curnode++) {
		(*curnode)->incoming_sum=0.0;
		/* For each incoming connection, add the activity from the connection to the activesum */
		for (curlink=(*curnode)->incoming.begin(); curlink!=(*curnode)->incoming.end(); curlink++) {
			if ((*curlink)->enable)
				(*curnode)->incoming_sum+=((*curlink)->weight)*((*curlink)->node->activation);
		}
	}
	/* Now activate all the non-sensor nodes off their incoming activation */
	for (curnode=hidden_nodes.begin(); curnode!=hidden_nodes.end(); curnode++) {
		if ((*curnode)->ftype==Neuron::SIGMOID)
			//Sigmoidal activation- see comments under fsigmoid
			(*curnode)->activation=fsigmoid((*curnode)->incoming_sum,4.924273,2.4621365);
	}
	for (curnode=outputs.begin(); curnode!=outputs.end(); curnode++) {
		if ((*curnode)->ftype==Neuron::SIGMOID)
			//Sigmoidal activation- see comments under fsigmoid
			(*curnode)->activation=fsigmoid((*curnode)->incoming_sum,4.924273,2.4621365);
	}
}

/* Add Gaussian noise to linkweights either GAUSSIAN or COLDGAUSSIAN (from zero) */
/* COLDGAUSSIAN means connection weight will be given completely new value */
void Organism::mutate_link_weights() {
	list<Neuron*>::iterator curnode;
	list<IncomingLink*>::iterator curlink;
	double randnum,gausspoint,coldgausspoint,age_factor;

	bool severe;  //Once in a while really shake things up
	if (randfloat()<0.5) severe=true;
	else severe=false;

	//Go through all the Genes and perturb their link's weights
	// Loop on all hidden neurons
	for(curnode=hidden_nodes.begin(); curnode!=hidden_nodes.end(); curnode++) {
		for (curlink=(*curnode)->incoming.begin(); curlink!=(*curnode)->incoming.end(); curlink++) {
			// age factor to mutate new links with increased probability
			if (link_innovations.empty()) age_factor=1.0;
			else age_factor=(double)((*curlink)->innovation_num+1)/((link_innovations.back())->innovation_num+1);

			//The following if determines the probabilities of doing cold gaussian
			//mutation, meaning the probability of replacing a link weight with
			//another, entirely random weight.  It is meant to bias such mutations
			//to the tail of a genome, because that is where less time-tested genes
			//reside. The gausspoint and coldgausspoint represent values below
			//which a random float will signify that kind of mutation.  
			if (severe) {
				gausspoint=age_factor;
				coldgausspoint=NEAT::gausspoint_thresh*age_factor;
			}
			else {
				gausspoint=NEAT::gausspoint_thresh*age_factor;
				coldgausspoint=NEAT::coldgausspoint_thresh*age_factor;
			}

			randnum=randfloat();
			if (randnum<coldgausspoint)
				((*curlink)->weight)=NEAT::weight_mut_power*randposneg()*randfloat()*(*curlink)->innovation_num;
			else if (randnum<gausspoint)
				((*curlink)->weight)+=NEAT::weight_mut_power*randposneg()*randfloat()*age_factor;
		}
	}
	// Loop on output neurons (same job)
	for(curnode=outputs.begin(); curnode!=outputs.end(); curnode++) {
		for (curlink=(*curnode)->incoming.begin(); curlink!=(*curnode)->incoming.end(); curlink++) {
			if (link_innovations.empty()) age_factor=1.0;
			else age_factor=(double)((*curlink)->innovation_num+1)/((link_innovations.back())->innovation_num+1);

			if (severe) {
				gausspoint=age_factor;
				coldgausspoint=NEAT::gausspoint_thresh*age_factor;
			}
			else {
				gausspoint=NEAT::gausspoint_thresh*age_factor;
				coldgausspoint=NEAT::coldgausspoint_thresh*age_factor;
			}

			randnum=randfloat();
			if (randnum<coldgausspoint)
				((*curlink)->weight)=NEAT::weight_mut_power*randposneg()*randfloat()*(*curlink)->innovation_num;
			else if (randnum<gausspoint)
				((*curlink)->weight)+=NEAT::weight_mut_power*randposneg()*randfloat()*age_factor;
		}
	}
}

/* Toggle genes from enable on to enable off or vice versa. */
void Organism::mutate_toggle_enable(int times) {
	list<Neuron*>::iterator curnode,checknode;
	list<IncomingLink*>::iterator curlink,toggle_link; //Gene to toggle
	int randnum,counter;
	bool foundLink;
	double age_factor;

	for (int i=times-1; i>=0; i--) {
		// first decide if toggle on hidden or output neurons
		if (randfloat()<(double)(hidden_nodes.size())/(hidden_nodes.size()+outputs.size())) {
			//Choose a random node
			randnum=randint(0,hidden_nodes.size()-1);
			curnode=hidden_nodes.begin();
			for (counter=randnum-1; counter>=0; counter--) curnode++;
			// choose a random link if not just one link enabled at this node (to avoid network cuts)
			counter=0;
			for (curlink=(*curnode)->incoming.begin(); curlink!=(*curnode)->incoming.end(); curlink++) {
				if ((*curlink)->enable) counter++;
			}
			if (counter<=1) continue;
			randnum=randint(0,(*curnode)->incoming.size()-1);
			toggle_link=(*curnode)->incoming.begin();
			for (counter=0; counter<randnum; counter++) toggle_link++;
			if (link_innovations.empty()) age_factor=1.0;
			else age_factor=(double)((*toggle_link)->innovation_num+1)/((link_innovations.back())->innovation_num+1);

			//Toggle the enable on this gene
			if ((*toggle_link)->enable && randfloat()<age_factor) {
				//We need to make sure that another gene connects out of the in-node
				//Because if not a section of network will break off and become isolated
				for (checknode=hidden_nodes.begin(); checknode!=hidden_nodes.end(); checknode++) {
					if (checknode==curnode) continue;
					foundLink=false;
					for (curlink=(*checknode)->incoming.begin(); curlink!=(*checknode)->incoming.end(); curlink++) {
						if ((*curlink)->enable && (*curlink)->node->id==(*toggle_link)->node->id) {
							foundLink=true;
							break;
						}
					}
					if (foundLink) break;
				}
				//Disable the gene if it's safe to do so
				if (checknode!=hidden_nodes.end()) (*toggle_link)->enable=false;
			}
			else (*toggle_link)->enable=true;
		}
		else {
			//Choose a random node
			randnum=randint(0,outputs.size()-1);
			curnode=outputs.begin();
			for (counter=randnum-1; counter>=0; counter--) curnode++;
			// choose a random link if not just one link enabled at this node (to avoid network cuts)
			counter=0;
			for (curlink=(*curnode)->incoming.begin(); curlink!=(*curnode)->incoming.end(); curlink++) {
				if ((*curlink)->enable) counter++;
			}
			if (counter<=1) continue;
			randnum=randint(0,(*curnode)->incoming.size()-1);
			toggle_link=(*curnode)->incoming.begin();
			for (counter=0; counter<randnum; counter++) toggle_link++;
			if (link_innovations.empty()) age_factor=1.0;
			else age_factor=(double)((*toggle_link)->innovation_num+1)/((link_innovations.back())->innovation_num+1);

			//Toggle the enable on this gene
			if ((*toggle_link)->enable && randfloat()<age_factor) {
				//We need to make sure that another gene connects out of the in-node
				//Because if not a section of network will break off and become isolated
				for (checknode=outputs.begin(); checknode!=outputs.end(); checknode++) {
					if (checknode==curnode) continue;
					foundLink=false;
					for (curlink=(*checknode)->incoming.begin(); curlink!=(*checknode)->incoming.end(); curlink++) {
						if ((*curlink)->enable && (*curlink)->node->id==(*toggle_link)->node->id) {
							foundLink=true;
							break;
						}
					}
					if (foundLink) break;
				}
				//Disable the gene if it's safe to do so
				if (checknode!=outputs.end()) (*toggle_link)->enable=false;
			}
			else (*toggle_link)->enable=true;
		}
	}
}

/* Find first disabled gene and enable it (if any is disabled) */
void Organism::mutate_gene_reenable() {
	list<Neuron*>::iterator curnode;
	list<IncomingLink*>::iterator curlink; //Gene to enable

	// decide which set of neurons to look first
	if (randfloat()<(double)(hidden_nodes.size())/(hidden_nodes.size()+outputs.size())) {
		for (curnode=hidden_nodes.begin(); curnode!=hidden_nodes.end(); curnode++) {
			for (curlink=(*curnode)->incoming.begin(); curlink!=(*curnode)->incoming.end(); curlink++) {
				if (!(*curlink)->enable) {
					(*curlink)->enable=true;
					return;
				}
			}
		}
		for (curnode=outputs.begin(); curnode!=outputs.end(); curnode++) {
			for (curlink=(*curnode)->incoming.begin(); curlink!=(*curnode)->incoming.end(); curlink++) {
				if (!(*curlink)->enable) {
					(*curlink)->enable=true;
					return;
				}
			}
		}
	}
	else {
		for (curnode=outputs.begin(); curnode!=outputs.end(); curnode++) {
			for (curlink=(*curnode)->incoming.begin(); curlink!=(*curnode)->incoming.end(); curlink++) {
				if (!(*curlink)->enable) {
					(*curlink)->enable=true;
					return;
				}
			}
		}
		for (curnode=hidden_nodes.begin(); curnode!=hidden_nodes.end(); curnode++) {
			for (curlink=(*curnode)->incoming.begin(); curlink!=(*curnode)->incoming.end(); curlink++) {
				if (!(*curlink)->enable) {
					(*curlink)->enable=true;
					return;
				}
			}
		}
	}
}

/* Mutate the genome by adding a new link between 2 random Neurons */
/* If the 2 Neurons are already connected, keep trying tries times */
bool Organism::mutate_add_link(int times) {
	list<Neuron*>::iterator it_node1,it_node2;
	list<IncomingLink*>::iterator curlink;
	int randnum; //random node number
	double randf;
	
	for (int i=times-1; i>=0; i--) {
		// randomly choose a pair of nodes (not allow connection inp-inp or out-out)
		randf=randfloat();
		bool firstIsInput=false,secondIsInput=false,firstIsOutput=false;
		if (randf<(double)hidden_nodes.size()/(inputs.size()+hidden_nodes.size()+outputs.size())) {
			randnum=randint(0,hidden_nodes.size()-1);
			it_node1=hidden_nodes.begin();
		}
		else if (randf<(double)(hidden_nodes.size()+outputs.size())
					/(inputs.size()+hidden_nodes.size()+outputs.size())) {
			randnum=randint(0,outputs.size()-1);
			it_node1=outputs.begin();
			firstIsOutput=true;
		}
		else {
			randnum=randint(0,inputs.size()-1);
			it_node1=inputs.begin();
			firstIsInput=true;
		}
		while (--randnum>=0) it_node1++;
		randf=randfloat();
		if (firstIsInput) {
			if (randf<(double)hidden_nodes.size()/(hidden_nodes.size()+outputs.size())) {
				randnum=randint(0,hidden_nodes.size()-1);
				it_node2=hidden_nodes.begin();
			}
			else {
				randnum=randint(0,outputs.size()-1);
				it_node2=outputs.begin();
			}
		}
		else if (firstIsOutput) {
			if (randf<(double)hidden_nodes.size()/(hidden_nodes.size()+inputs.size())) {
				randnum=randint(0,hidden_nodes.size()-1);
				it_node2=hidden_nodes.begin();
			}
			else {
				randnum=randint(0,inputs.size()-1);
				it_node2=inputs.begin();
			}
		}
		else {
			if (randf<(double)hidden_nodes.size()/(inputs.size()+hidden_nodes.size()+outputs.size())) {
				randnum=randint(0,hidden_nodes.size()-1);
				it_node2=hidden_nodes.begin();
			}
			else if (randf<(double)(hidden_nodes.size()+outputs.size())
						/(inputs.size()+hidden_nodes.size()+outputs.size())) {
				randnum=randint(0,outputs.size()-1);
				it_node2=outputs.begin();
			}
			else {
				randnum=randint(0,inputs.size()-1);
				it_node2=inputs.begin();
				secondIsInput=true;
			}
		}
		while (--randnum>=0) it_node2++;

		// order the iterators to have "it1<it2"
		list<Neuron*>::iterator aux_it;
		if (secondIsInput || firstIsOutput) {
			aux_it=it_node1;
			it_node1=it_node2;
			it_node2=aux_it;
		}
		
		// check if they are connected AND different (!)
		// NOTE: we don't allow double sense connections; perhaps could be useful,
		// but it seems we can do without (and final interpretation will be much simpler)
		if ((*it_node1)->id==(*it_node2)->id) continue;
		curlink=(*it_node2)->incoming.begin();
		while (curlink!=(*it_node2)->incoming.end() && (*curlink)->node->id!=(*it_node1)->id) curlink++;
		if (curlink!=(*it_node2)->incoming.end()) continue;
		curlink=(*it_node1)->incoming.begin();
		while (curlink!=(*it_node1)->incoming.end() && (*curlink)->node->id!=(*it_node2)->id) curlink++;
		if (curlink!=(*it_node1)->incoming.end()) continue;

		// they are not connected, so we create the link with weight 1.0
		// search if this mutation already exists
		int index=Innovation::global_link_innovation.size()-1;
		while (index>=0 && 
			(Innovation::global_link_innovation[index]->incnode->id!=(*it_node1)->id || Innovation::global_link_innovation[i]->outnode->id!=(*it_node2)->id))
			index--;
		if (index>=0) {
			// mutation ever met:
			link_innovations.push_back(Innovation::global_link_innovation[index]);
			(*it_node2)->add_incoming(*it_node1,1.0,Innovation::global_link_innovation[index]->innovation_num);
		}
		else {
			// new mutation
			(*it_node2)->add_incoming(*it_node1,1.0,Innovation::cur_innov);
			Link* lk=new Link(*it_node1,*it_node2,Innovation::cur_innov++);
			link_innovations.push_back(lk);
			Innovation::global_link_innovation.push_back(lk);
		}
		nb_links++;
		return true;
	}
	return false;
}

//This mutator adds a node to an organism by inserting it in the middle
//of an existing link between 2 nodes. This broken link will be disabled
//and now represented by 2 links with the new node between them.
//The global innovs list is used to compare the innovation
//with other innovations in the list and see whether they match.
//If they do, the same innovation number will be assigned to the new
//genes. If a disabled link is chosen after times trials, 
//then the method just exits with false
bool Organism::mutate_add_node(int times) {
	list<Neuron*>::iterator curnode;
	list<IncomingLink*>::iterator curlink; //random gene containing the original link
	int randnum; //The random gene number

	for (int i=times-1; i>=0; i--) {
		// pseudo-randomly choose a link
		if (randfloat()<(double)hidden_nodes.size()/(hidden_nodes.size()+outputs.size())) {
			randnum=randint(0,hidden_nodes.size()-1);
			curnode=hidden_nodes.begin();
		}
		else {
			randnum=randint(0,outputs.size()-1);
			curnode=outputs.begin();
		}
		while (--randnum>=0) curnode++;
		curlink=(*curnode)->incoming.begin();
		randnum=randint(0,(*curnode)->incoming.size()-1);
		while (--randnum>=0) curlink++;

		// NOTE: for such a mutation, the link can be disable;
		// in this case, the mutation will be active later
		if (randfloat()<(double)((*curlink)->innovation_num+2)/(Innovation::cur_innov+1)) {
			// search if this mutation already exists
			int index=Innovation::global_node_innovation.size()-1;
			while (Innovation::global_node_innovation[index]!=0 && 
				(Innovation::global_node_innovation[index]->incnode->id!=(*curlink)->node->id || Innovation::global_node_innovation[index]->outnode->id!=(*curnode)->id))
				index--;
			if (Innovation::global_node_innovation[index]!=0) {
				// mutation ever met: 
				//WARNING we have to check if this mutation didn't occur in this organism's history!
				list<LinkAndNode*>::iterator check_exists;
				for (check_exists=node_innovations.begin(); check_exists!=node_innovations.end(); check_exists++) {
					if ((*check_exists)->midnode->id==Innovation::global_node_innovation[index]->midnode->id) break;
				}
				if (check_exists!=node_innovations.end()) continue;
				node_innovations.push_back(Innovation::global_node_innovation[index]);
				Neuron* newnode=new Neuron(Neuron::SIGMOID,Innovation::global_node_innovation[index]->midnode->id);
				newnode->add_incoming((*curlink)->node,(*curlink)->weight,Innovation::global_node_innovation[index]->innovation_num);
				(*curnode)->add_incoming(newnode,(*curlink)->weight,Innovation::global_node_innovation[index]->innovation_num+1);
				(*curlink)->enable=false;
				add_neuron(newnode);
			}
			else {
				// new mutation
				Neuron* newnode=new Neuron(Neuron::SIGMOID,cur_node_id++); // create the new neuron
				newnode->add_incoming((*curlink)->node,(*curlink)->weight,Innovation::cur_innov);

				// add the first new link mutation:
				Link* lk=new Link((*curlink)->node,newnode,Innovation::cur_innov++);
				link_innovations.push_back(lk);
				Innovation::global_link_innovation.push_back(lk);
				(*curnode)->add_incoming(newnode,(*curlink)->weight,Innovation::cur_innov);

				// add the second new link mutation:
				lk=new Link(newnode,*curnode,Innovation::cur_innov++);
				link_innovations.push_back(lk);
				Innovation::global_link_innovation.push_back(lk);
				
				// finally, add the new node innovation:
				LinkAndNode* lkn=new LinkAndNode((*curlink)->node,newnode,*curnode,Innovation::cur_innov-2);
				node_innovations.push_back(lkn);
				Innovation::global_node_innovation.push_back(lkn);
				(*curlink)->enable=false;
				add_neuron(newnode);
			}
			nb_links+=2; // two new links appeared
			return true;
		}
	}
	return false;
}

/* This method mates this Organism with another Organism g.
     For every point in each Organism, where each Organism shares
     the innovation number, the Gene is chosen randomly from
     either parent.  If one parent has an innovation absent in
     the other, the baby may inherit the innovation
     if it is from the more fit parent.*/
Organism* Organism::mate_multipoint(const Organism* org) const {
	Organism* baby=new Organism(); // the new organism
				// we have to initialize his hidden neurons and innovations
				// fitness will be set elsewhere
	vector<int> innovLink,innovNode;
	list<Neuron*>::const_iterator curnode,curnode1,curnode2;
	list<IncomingLink*>::const_iterator curlink;
	// tells if genes from one organism should be chosen:
	bool thisIsBetter=false,orgIsBetter=false,thisIsSmaller=true,orgIsSmaller=true;
	
	//preprocessing: array of new neurons indexed by id's of the parents
	// highest ID's are typically find in the hidden layers;
	// if no hidden layer, we go through outputs neurons
	int sizeID=0;
	bool hdn=false;
	if (!hidden_nodes.empty()) {
		hdn=true;
		for (curnode=hidden_nodes.begin(); curnode!=hidden_nodes.end(); curnode++)
			if ((*curnode)->id>sizeID) sizeID=(*curnode)->id;
	}
	if (!org->hidden_nodes.empty()) {
		hdn=true;
		for (curnode=org->hidden_nodes.begin(); curnode!=org->hidden_nodes.end(); curnode++)
			if ((*curnode)->id>sizeID) sizeID=(*curnode)->id;
	}
	if (!hdn) sizeID=inputs.size()+outputs.size()-1;

	Neuron** tabNodes=new Neuron*[sizeID+1];
	bool* dupHidNodes=(bool*)calloc(sizeID+1,sizeof(bool)); // mark the nodes encountered twice in the hidden layers
	bool* nodes=(bool*)calloc(sizeID+1,sizeof(bool)); // mark all the nodes inside the organism;
														// to avoid "ghost" incoming nodes
	// duplication phase
	for (curnode=inputs.begin(); curnode!=inputs.end(); curnode++) {
		tabNodes[(*curnode)->id]=new Neuron(*curnode);
		baby->inputs.push_back(tabNodes[(*curnode)->id]);
		nodes[(*curnode)->id]=true;
	}
	for (curnode=outputs.begin(); curnode!=outputs.end(); curnode++) {
		tabNodes[(*curnode)->id]=new Neuron(*curnode);
		nodes[(*curnode)->id]=true;
	}
	for (curnode=hidden_nodes.begin(); curnode!=hidden_nodes.end(); curnode++) {
		tabNodes[(*curnode)->id]=new Neuron(*curnode);
		dupHidNodes[(*curnode)->id]=true;
	}
	for (curnode=org->hidden_nodes.begin(); curnode!=org->hidden_nodes.end(); curnode++) {
		if (!dupHidNodes[(*curnode)->id]) tabNodes[(*curnode)->id]=new Neuron(*curnode);
	}
	free(dupHidNodes);

	//Figure out which genome is better
	//The worse genome should not be allowed to add extra structural baggage
	//If they are the same, use the smaller one's disjoint and excess genes only
	// (principle of Occam's razor)
	if (orig_fitness>org->orig_fitness) thisIsBetter=true;
	else if (orig_fitness==org->orig_fitness) {
		if (hidden_nodes.size() < org->hidden_nodes.size()) orgIsSmaller=false;
		else if (hidden_nodes.size() > org->hidden_nodes.size()) thisIsSmaller=false;
	}
	else orgIsBetter=true;

	//Now move through the Genes of each parent until both genomes end;
	//neurons and incoming links are order by increasing id, so it's relatively easy
	curnode1=hidden_nodes.begin();
	curnode2=org->hidden_nodes.begin();
	while (curnode1!=hidden_nodes.end() || curnode2!=(org->hidden_nodes).end()) {
		if (curnode1==hidden_nodes.end()) {
			if (thisIsBetter) break;  //Skip excess from the worse genome
			if (orgIsBetter || orgIsSmaller) {
				innovNode.push_back((*curnode2)->id);
				for (curlink=(*curnode2)->incoming.begin(); curlink!=(*curnode2)->incoming.end(); curlink++) {
					// if a link is disabled, it has a little chance to be enabled for the child
					if (!(*curlink)->enable && randfloat()<0.25)
						tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,true);
					else tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
					innovLink.push_back((*curlink)->innovation_num);
				}
				baby->hidden_nodes.push_back(tabNodes[(*curnode2)->id]);
				baby->nb_links+=(*curnode2)->incoming.size();
				nodes[(*curnode2)->id]=true;
			}
			curnode2++;
		}
		else if (curnode2==org->hidden_nodes.end()) {
			if (orgIsBetter) break;  //Skip excess from the worse genome
			if (thisIsBetter || thisIsSmaller) {
				innovNode.push_back((*curnode1)->id);
				for (curlink=(*curnode1)->incoming.begin(); curlink!=(*curnode1)->incoming.end(); curlink++) {
					// if a link is disabled, it has a little chance to be enabled for the child
					if (!(*curlink)->enable && randfloat()<0.25)
						tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,true);
					else tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
					innovLink.push_back((*curlink)->innovation_num);
				}
				baby->hidden_nodes.push_back(tabNodes[(*curnode1)->id]);
				baby->nb_links+=(*curnode1)->incoming.size();
				nodes[(*curnode1)->id]=true;
			}
			curnode1++;
		}
		else {
			//two cases: features in both organisms, or just in one
			if ((*curnode1)->id==(*curnode2)->id) {
				// trial to add the best:
				if (thisIsBetter) {
					innovNode.push_back((*curnode1)->id);
					for (curlink=(*curnode1)->incoming.begin(); curlink!=(*curnode1)->incoming.end(); curlink++) {
						// if a link is disabled, it has a little chance to be enabled for the child
						if (!(*curlink)->enable && randfloat()<0.25)
							tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,true);
						else tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
						innovLink.push_back((*curlink)->innovation_num);
					}
					baby->hidden_nodes.push_back(tabNodes[(*curnode1)->id]);
					baby->nb_links+=(*curnode1)->incoming.size();
					nodes[(*curnode1)->id]=true;
				}
				else if (orgIsBetter) {
					innovNode.push_back((*curnode2)->id);
					for (curlink=(*curnode2)->incoming.begin(); curlink!=(*curnode2)->incoming.end(); curlink++) {
						// if a link is disabled, it has a little chance to be enabled for the child
						if (!(*curlink)->enable && randfloat()<0.25)
							tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,true);
						else tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
						innovLink.push_back((*curlink)->innovation_num);
					}
					baby->hidden_nodes.push_back(tabNodes[(*curnode2)->id]);
					baby->nb_links+=(*curnode2)->incoming.size();
					nodes[(*curnode2)->id]=true;
				}
				else {
					// choose at random
					if (randbit()) {
						innovNode.push_back((*curnode1)->id);
						for (curlink=(*curnode1)->incoming.begin(); curlink!=(*curnode1)->incoming.end(); curlink++) {
							// if a link is disabled, it has a little chance to be enabled for the child
							if (!(*curlink)->enable && randfloat()<0.25)
								tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,true);
							else tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
							innovLink.push_back((*curlink)->innovation_num);
						}
						baby->hidden_nodes.push_back(tabNodes[(*curnode1)->id]);
						baby->nb_links+=(*curnode1)->incoming.size();
						nodes[(*curnode1)->id]=true;
					}
					else {
						innovNode.push_back((*curnode2)->id);
						for (curlink=(*curnode2)->incoming.begin(); curlink!=(*curnode2)->incoming.end(); curlink++) {
							// if a link is disabled, it has a little chance to be enabled for the child
							if (!(*curlink)->enable && randfloat()<0.25)
								tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,true);
							else tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
							innovLink.push_back((*curlink)->innovation_num);
						}
						baby->hidden_nodes.push_back(tabNodes[(*curnode2)->id]);
						baby->nb_links+=(*curnode2)->incoming.size();
						nodes[(*curnode2)->id]=true;
					}
				}
				curnode1++;
				curnode2++;
			}
			else if ((*curnode1)->id<(*curnode2)->id) {
				if (thisIsBetter || thisIsSmaller) {
					innovNode.push_back((*curnode1)->id);
					for (curlink=(*curnode1)->incoming.begin(); curlink!=(*curnode1)->incoming.end(); curlink++) {
						// if a link is disabled, it has a little chance to be enabled for the child
						if (!(*curlink)->enable && randfloat()<0.25)
							tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,true);
						else tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
						innovLink.push_back((*curlink)->innovation_num);
					}
					baby->hidden_nodes.push_back(tabNodes[(*curnode1)->id]);
					baby->nb_links+=(*curnode1)->incoming.size();
					nodes[(*curnode1)->id]=true;
				}
				curnode1++;
			}
			else {
				if (orgIsBetter || orgIsSmaller) {
					innovNode.push_back((*curnode2)->id);
					for (curlink=(*curnode2)->incoming.begin(); curlink!=(*curnode2)->incoming.end(); curlink++) {
						// if a link is disabled, it has a little chance to be enabled for the child
						if (!(*curlink)->enable && randfloat()<0.25)
							tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,true);
						else tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
						innovLink.push_back((*curlink)->innovation_num);
					}
					baby->hidden_nodes.push_back(tabNodes[(*curnode2)->id]);
					baby->nb_links+=(*curnode2)->incoming.size();
					nodes[(*curnode2)->id]=true;
				}
				curnode2++;
			}
		}
	}
	// then look at the output nodes
	curnode1=outputs.begin();
	curnode2=(org->outputs).begin();
	while (curnode1!=outputs.end()) {
		// trial to add the best:
		if (thisIsBetter) {
			for (curlink=(*curnode1)->incoming.begin(); curlink!=(*curnode1)->incoming.end(); curlink++) {
				// if a link is disabled, it has a little chance to be enabled for the child
				if (!(*curlink)->enable && randfloat()<0.25)
					tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,true);
				else tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
				if ((*curlink)->innovation_num>=0) innovLink.push_back((*curlink)->innovation_num);
			}
			baby->outputs.push_back(tabNodes[(*curnode1)->id]);
			baby->nb_links+=(*curnode1)->incoming.size();
		}
		else if (orgIsBetter) {
			for (curlink=(*curnode2)->incoming.begin(); curlink!=(*curnode2)->incoming.end(); curlink++) {
				// if a link is disabled, it has a little chance to be enabled for the child
				if (!(*curlink)->enable && randfloat()<0.25)
					tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,true);
				else tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
				if ((*curlink)->innovation_num>=0) innovLink.push_back((*curlink)->innovation_num);
			}
			baby->outputs.push_back(tabNodes[(*curnode2)->id]);
			baby->nb_links+=(*curnode2)->incoming.size();
		}
		else {
			// choose at random
			if (randbit()) {
				for (curlink=(*curnode1)->incoming.begin(); curlink!=(*curnode1)->incoming.end(); curlink++) {
					// if a link is disabled, it has a little chance to be enabled for the child
					if (!(*curlink)->enable && randfloat()<0.25)
						tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,true);
					else tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
					if ((*curlink)->innovation_num>=0) innovLink.push_back((*curlink)->innovation_num);
				}
				baby->outputs.push_back(tabNodes[(*curnode1)->id]);
				baby->nb_links+=(*curnode1)->incoming.size();
			}
			else {
				for (curlink=(*curnode2)->incoming.begin(); curlink!=(*curnode2)->incoming.end(); curlink++) {
					// if a link is disabled, it has a little chance to be enabled for the child
					if (!(*curlink)->enable && randfloat()<0.25)
						tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,true);
					else tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
					if ((*curlink)->innovation_num>=0) innovLink.push_back((*curlink)->innovation_num);
				}
				baby->outputs.push_back(tabNodes[(*curnode2)->id]);
				baby->nb_links+=(*curnode2)->incoming.size();
			}
		}
		curnode1++;
		curnode2++;
	}

	list<Neuron*>::iterator curnode_;
	list<IncomingLink*>::iterator curlink_,deadlink;
	// we now have to erase the incomings to "ghost" nodes:
	for (curnode_=baby->hidden_nodes.begin(); curnode_!=baby->hidden_nodes.end(); curnode_++) {
		curlink_=(*curnode_)->incoming.begin();
		while (curlink_!=(*curnode_)->incoming.end()) {
			if (!nodes[(*curlink_)->node->id]) {
				deadlink=curlink_;
				curlink_++;
				(*curnode_)->incoming.erase(deadlink);
			}
			else curlink_++;
		}
	}
	for (curnode_=baby->outputs.begin(); curnode_!=baby->outputs.end(); curnode_++) {
		curlink_=(*curnode_)->incoming.begin();
		while (curlink_!=(*curnode_)->incoming.end()) {
			if (!nodes[(*curlink_)->node->id]) {
				deadlink=curlink_;
				curlink_++;
				(*curnode_)->incoming.erase(deadlink);
			}
			else curlink_++;
		}
	}

	// finally, look through all innovations,
	// and add those which have the corresponding features in baby's structure
	// it's faster than going through the lists each time we meet an innovation
	for (int i=innovLink.size()-1; i>=0; i--) {
		if (!nodes[Innovation::global_link_innovation[innovLink[i]]->incnode->id]) continue;
		baby->link_innovations.push_back(Innovation::global_link_innovation[innovLink[i]]);
	}
	free(nodes);
	for (int i=innovNode.size()-1; i>=0; i--)
		baby->node_innovations.push_back(Innovation::global_node_innovation[innovNode[i]]);

	return baby;
}

/* This is like mate_multipoint except it averages at each
   matching gene instead of choosing one or the other */
Organism* Organism::mate_multipoint_avg(const Organism* org) const {
	Organism* baby=new Organism(); // the new organism
				// we have to initialize his hidden neurons and innovations
				// fitness will be set elsewhere
	vector<int> innovLink,innovNode;
	double thisCoeff,orgCoeff;
	list<Neuron*>::const_iterator curnode,curnode1,curnode2;
	list<IncomingLink*>::const_iterator curlink,curlink1,curlink2;
	// tells if genes from one organism should be chosen
	bool thisIsBetter=false,orgIsBetter=false,thisIsSmaller=true,orgIsSmaller=true;
	
	//preprocessing: array of new neurons indexed by id's of the parents
	// highest ID's are typically find in the hidden layers;
	// if no hidden layer, we go through outputs neurons
	int sizeID=0;
	bool hdn=false;
	if (!hidden_nodes.empty()) {
		hdn=true;
		for (curnode=hidden_nodes.begin(); curnode!=hidden_nodes.end(); curnode++)
			if ((*curnode)->id>sizeID) sizeID=(*curnode)->id;
	}
	if (!org->hidden_nodes.empty()) {
		hdn=true;
		for (curnode=org->hidden_nodes.begin(); curnode!=org->hidden_nodes.end(); curnode++)
			if ((*curnode)->id>sizeID) sizeID=(*curnode)->id;
	}
	if (!hdn) sizeID=inputs.size()+outputs.size()-1;

	Neuron** tabNodes=new Neuron*[sizeID+1];
	bool* dupHidNodes=(bool*)calloc(sizeID+1,sizeof(bool)); // mark the nodes encountered twice in the hidden layers
	bool* nodes=(bool*)calloc(sizeID+1,sizeof(bool)); // mark all the nodes inside the organism;
														// to avoid "ghost" incoming nodes
	// duplication phase
	for (curnode=inputs.begin(); curnode!=inputs.end(); curnode++) {
		tabNodes[(*curnode)->id]=new Neuron(*curnode);
		baby->inputs.push_back(tabNodes[(*curnode)->id]);
		nodes[(*curnode)->id]=true;
	}
	for (curnode=outputs.begin(); curnode!=outputs.end(); curnode++) {
		tabNodes[(*curnode)->id]=new Neuron(*curnode);
		nodes[(*curnode)->id]=true;
	}
	for (curnode=hidden_nodes.begin(); curnode!=hidden_nodes.end(); curnode++) {
		tabNodes[(*curnode)->id]=new Neuron(*curnode);
		dupHidNodes[(*curnode)->id]=true;
	}
	for (curnode=org->hidden_nodes.begin(); curnode!=org->hidden_nodes.end(); curnode++) {
		if (!dupHidNodes[(*curnode)->id]) tabNodes[(*curnode)->id]=new Neuron(*curnode);
	}
	free(dupHidNodes);

	//Figure out which genome is better
	//The worse genome should not be allowed to add extra structural baggage
	//If they are the same, use the smaller one's disjoint and excess genes only
	// (principle of Occam's razor)
	if (orig_fitness>org->orig_fitness) thisIsBetter=true;
	else if (orig_fitness==org->orig_fitness) {
		if (hidden_nodes.size() < org->hidden_nodes.size()) orgIsSmaller=false;
		else if (hidden_nodes.size() > org->hidden_nodes.size()) thisIsSmaller=false;
	}
	else orgIsBetter=true;

	//Now move through the Genes of each parent until both genomes end;
	//neurons and incoming links are order by increasing id, so it's relatively easy
	curnode1=hidden_nodes.begin();
	curnode2=org->hidden_nodes.begin();
	while (curnode1!=hidden_nodes.end() || curnode2!=(org->hidden_nodes).end()) {
		if (curnode1==hidden_nodes.end()) {
			if (thisIsBetter) break; //Skip excess from the worse genome
			if (orgIsBetter || orgIsSmaller) {
				innovNode.push_back((*curnode2)->id);
				for (curlink=(*curnode2)->incoming.begin(); curlink!=(*curnode2)->incoming.end(); curlink++) {
					// if a link is disabled, it has a little chance to be enabled for the child
					if (!(*curlink)->enable && randfloat()<0.25)
						tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,true);
					else tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
					innovLink.push_back((*curlink)->innovation_num);
				}
				baby->hidden_nodes.push_back(tabNodes[(*curnode2)->id]);
				baby->nb_links+=(*curnode2)->incoming.size();
				nodes[(*curnode2)->id]=true;
			}
			curnode2++;
		}
		else if (curnode2==org->hidden_nodes.end()) {
			if (orgIsBetter) break; //Skip excess from the worse genome
			if (thisIsBetter || thisIsSmaller) {
				innovNode.push_back((*curnode1)->id);
				for (curlink=(*curnode1)->incoming.begin(); curlink!=(*curnode1)->incoming.end(); curlink++) {
					// if a link is disabled, it has a little chance to be enabled for the child
					if (!(*curlink)->enable && randfloat()<0.25)
						tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,true);
					else tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
					innovLink.push_back((*curlink)->innovation_num);
				}
				baby->hidden_nodes.push_back(tabNodes[(*curnode1)->id]);
				baby->nb_links+=(*curnode1)->incoming.size();
				nodes[(*curnode1)->id]=true;
			}
			curnode1++;
		}
		else {
			//two cases: features in both organisms, or just in one
			if ((*curnode1)->id==(*curnode2)->id) {
				// average the features, so it's more subtile (the only difference):
				innovNode.push_back((*curnode1)->id);
				curlink1=(*curnode1)->incoming.begin();
				curlink2=(*curnode2)->incoming.begin();
				while (curlink1!=(*curnode1)->incoming.end() || curlink2!=(*curnode2)->incoming.end()) {
					if (curlink1==(*curnode1)->incoming.end()) {
						if (thisIsBetter) break; // skip the worse organism (Occam's razor)
						if (orgIsBetter || orgIsSmaller) {
							// if a link is disabled, it has a little chance to be enabled for the child
							if (!(*curlink2)->enable && randfloat()<0.25)
								tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink2)->node->id],(*curlink2)->weight,(*curlink2)->innovation_num,true);
							else tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink2)->node->id],(*curlink2)->weight,(*curlink2)->innovation_num,(*curlink2)->enable);
							baby->nb_links++;
							innovLink.push_back((*curlink2)->innovation_num);
						}
						curlink2++;
					}
					else if (curlink2==(*curnode2)->incoming.end()) {
						if (orgIsBetter) break; // skip the worse organism (Occam's razor)
						if (thisIsBetter || thisIsSmaller) {
							// if a link is disabled, it has a little chance to be enabled for the child
							if (!(*curlink1)->enable && randfloat()<0.25)
								tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink1)->node->id],(*curlink1)->weight,(*curlink1)->innovation_num,true);
							else tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink1)->node->id],(*curlink1)->weight,(*curlink1)->innovation_num,(*curlink1)->enable);
							baby->nb_links++;
							innovLink.push_back((*curlink1)->innovation_num);
						}
						curlink1++;
					}
					else {
						// two cases: the links point the same node, or not
						if ((*curlink1)->node->id<(*curlink2)->node->id) {
							if (thisIsBetter || thisIsSmaller) {
								// if a link is disabled, it has a little chance to be enabled for the child
								if (!(*curlink1)->enable && randfloat()<0.25)
									tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink1)->node->id],(*curlink1)->weight,(*curlink1)->innovation_num,true);
								else tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink1)->node->id],(*curlink1)->weight,(*curlink1)->innovation_num,(*curlink1)->enable);
								baby->nb_links++;
								innovLink.push_back((*curlink1)->innovation_num);
							}
							curlink1++;
						}
						else if ((*curlink1)->node->id>(*curlink2)->node->id) {
							if (orgIsBetter || orgIsSmaller) {
								// if a link is disabled, it has a little chance to be enabled for the child
								if (!(*curlink2)->enable && randfloat()<0.25)
									tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink2)->node->id],(*curlink2)->weight,(*curlink2)->innovation_num,true);
								else tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink2)->node->id],(*curlink2)->weight,(*curlink2)->innovation_num,(*curlink2)->enable);
								baby->nb_links++;
								innovLink.push_back((*curlink2)->innovation_num);
							}
							curlink2++;
						}
						else {
							// we bias toward better organisms
							// NOTE: all those biases are not natural, and we could see what happens
							// if we let "natural biological process" happens..
							thisCoeff=1.0;
							orgCoeff=1.0;
							if (thisIsBetter) thisCoeff+=(orig_fitness-org->orig_fitness);
							else if (orgIsBetter) orgCoeff+=(org->orig_fitness-orig_fitness);
							
							// if a link is disabled for both parents, it has a little chance to be enabled for the child
							// if present in only one parent, more chances to get the gene ..etc
							if ((!(*curlink1)->enable && !(*curlink2)->enable && randfloat()<0.25)
								|| ((!(*curlink1)->enable || !(*curlink2)->enable) && randfloat()<0.6))
								tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink1)->node->id],
								(thisCoeff*(*curlink1)->weight+orgCoeff*(*curlink2)->weight)/(thisCoeff+orgCoeff),(*curlink1)->innovation_num);
							else tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink1)->node->id],
								(thisCoeff*(*curlink1)->weight+orgCoeff*(*curlink2)->weight)/(thisCoeff+orgCoeff),(*curlink1)->innovation_num,(*curlink1)->enable);
							baby->nb_links++;
							innovLink.push_back((*curlink1)->innovation_num);
							
							curlink1++;
							curlink2++;
						}
					}
				}
				baby->hidden_nodes.push_back(tabNodes[(*curnode1)->id]);
				nodes[(*curnode1)->id]=true;
				curnode1++;
				curnode2++;
			}
			else if ((*curnode1)->id<(*curnode2)->id) {
				if (thisIsBetter || thisIsSmaller) {
					innovNode.push_back((*curnode1)->id);
					for (curlink=(*curnode1)->incoming.begin(); curlink!=(*curnode1)->incoming.end(); curlink++) {
						// if a link is disabled, it has a little chance to be enabled for the child
						if (!(*curlink)->enable && randfloat()<0.25)
							tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,true);
						else tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
						innovLink.push_back((*curlink)->innovation_num);
					}
					baby->hidden_nodes.push_back(tabNodes[(*curnode1)->id]);
					baby->nb_links+=(*curnode1)->incoming.size();
					nodes[(*curnode1)->id]=true;
				}
				curnode1++;
			}
			else {
				if (orgIsBetter || orgIsSmaller) {
					innovNode.push_back((*curnode2)->id);
					for (curlink=(*curnode2)->incoming.begin(); curlink!=(*curnode2)->incoming.end(); curlink++) {
						// if a link is disabled, it has a little chance to be enabled for the child
						if (!(*curlink)->enable && randfloat()<0.25)
							tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,true);
						else tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
						innovLink.push_back((*curlink)->innovation_num);
					}
					baby->hidden_nodes.push_back(tabNodes[(*curnode2)->id]);
					baby->nb_links+=(*curnode2)->incoming.size();
					nodes[(*curnode2)->id]=true;
				}
				curnode2++;
			}
		}
	}
	// then look at the output nodes
	curnode1=outputs.begin();
	curnode2=(org->outputs).begin();
	while (curnode1!=outputs.end()) {
		// average the features, so it's more subtile (the only difference):
		curlink1=(*curnode1)->incoming.begin();
		curlink2=(*curnode2)->incoming.begin();
		while (curlink1!=(*curnode1)->incoming.end() || curlink2!=(*curnode2)->incoming.end()) {
			if (curlink1==(*curnode1)->incoming.end()) {
				if (thisIsBetter) break; // skip the worse organism (Occam's razor)
				if (orgIsBetter || orgIsSmaller) {
					// if a link is disabled, it has a little chance to be enabled for the child
					if (!(*curlink2)->enable && randfloat()<0.25)
						tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink2)->node->id],(*curlink2)->weight,(*curlink2)->innovation_num,true);
					else tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink2)->node->id],(*curlink2)->weight,(*curlink2)->innovation_num,(*curlink2)->enable);
					baby->nb_links++;
					if ((*curlink2)->innovation_num>=0) innovLink.push_back((*curlink2)->innovation_num);
				}
				curlink2++;
			}
			else if (curlink2==(*curnode2)->incoming.end()) {
				if (orgIsBetter) break; // skip the worse organism (Occam's razor)
				if (thisIsBetter || thisIsSmaller) {
					// if a link is disabled, it has a little chance to be enabled for the child
					if (!(*curlink1)->enable && randfloat()<0.25)
						tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink1)->node->id],(*curlink1)->weight,(*curlink1)->innovation_num,true);
					else tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink1)->node->id],(*curlink1)->weight,(*curlink1)->innovation_num,(*curlink1)->enable);
					baby->nb_links++;
					if ((*curlink1)->innovation_num>=0) innovLink.push_back((*curlink1)->innovation_num);
				}
				curlink1++;
			}
			else {
				// two cases: the links point the same node, or not
				if ((*curlink1)->node->id<(*curlink2)->node->id) {
					if (thisIsBetter || thisIsSmaller) {
						// if a link is disabled, it has a little chance to be enabled for the child
						if (!(*curlink1)->enable && randfloat()<0.25)
							tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink1)->node->id],(*curlink1)->weight,(*curlink1)->innovation_num,true);
						else tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink1)->node->id],(*curlink1)->weight,(*curlink1)->innovation_num,(*curlink1)->enable);
						baby->nb_links++;
						if ((*curlink1)->innovation_num>=0) innovLink.push_back((*curlink1)->innovation_num);
					}
					curlink1++;
				}
				else if ((*curlink1)->node->id>(*curlink2)->node->id) {
					if (orgIsBetter || orgIsSmaller) {
						// if a link is disabled, it has a little chance to be enabled for the child
						if (!(*curlink2)->enable && randfloat()<0.25)
							tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink2)->node->id],(*curlink2)->weight,(*curlink2)->innovation_num,true);
						else tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink2)->node->id],(*curlink2)->weight,(*curlink2)->innovation_num,(*curlink2)->enable);
						baby->nb_links++;
						if ((*curlink2)->innovation_num>=0) innovLink.push_back((*curlink2)->innovation_num);
					}
					curlink2++;
				}
				else {
					// we bias toward better organisms
					// NOTE: all those biases are not natural, and we could see what happens
					// if we let "natural biological process" happens..
					thisCoeff=1.0;
					orgCoeff=1.0;
					if (thisIsBetter) thisCoeff+=(orig_fitness-org->orig_fitness);
					else if (orgIsBetter) orgCoeff+=(org->orig_fitness-orig_fitness);

					// if a link is disabled for both parents, it has a little chance to be enabled for the child
					// if present in only one parent, more chances to get the gene ..etc
					if ((!(*curlink1)->enable && !(*curlink2)->enable && randfloat()<0.25)
						|| ((!(*curlink1)->enable || !(*curlink2)->enable) && randfloat()<0.6))
						tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink1)->node->id],
						(thisCoeff*(*curlink1)->weight+orgCoeff*(*curlink2)->weight)/(thisCoeff+orgCoeff),(*curlink1)->innovation_num,true);
					else tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink1)->node->id],
						(thisCoeff*(*curlink1)->weight+orgCoeff*(*curlink2)->weight)/(thisCoeff+orgCoeff),(*curlink1)->innovation_num,(*curlink1)->enable);
					baby->nb_links++;
					if ((*curlink1)->innovation_num>=0) innovLink.push_back((*curlink1)->innovation_num);

					curlink1++;
					curlink2++;
				}
			}
		}
		baby->outputs.push_back(tabNodes[(*curnode1)->id]);
		curnode1++;
		curnode2++;
	}

	list<Neuron*>::iterator curnode_;
	list<IncomingLink*>::iterator curlink_,deadlink;
	// we now have to erase the incomings to "ghost" nodes:
	for (curnode_=baby->hidden_nodes.begin(); curnode_!=baby->hidden_nodes.end(); curnode_++) {
		curlink_=(*curnode_)->incoming.begin();
		while (curlink_!=(*curnode_)->incoming.end()) {
			if (!nodes[(*curlink_)->node->id]) {
				deadlink=curlink_;
				curlink_++;
				(*curnode_)->incoming.erase(deadlink);
			}
			else curlink_++;
		}
	}
	for (curnode_=baby->outputs.begin(); curnode_!=baby->outputs.end(); curnode_++) {
		curlink_=(*curnode_)->incoming.begin();
		while (curlink_!=(*curnode_)->incoming.end()) {
			if (!nodes[(*curlink_)->node->id]) {
				deadlink=curlink_;
				curlink_++;
				(*curnode_)->incoming.erase(deadlink);
			}
			else curlink_++;
		}
	}

	// finally, look through all innovations,
	// and add those which have the corresponding features in baby's structure
	// it's faster than going through the lists each time we meet an innovation
	for (int i=innovLink.size()-1; i>=0; i--) {
		if (!nodes[Innovation::global_link_innovation[innovLink[i]]->incnode->id]) continue;
		baby->link_innovations.push_back(Innovation::global_link_innovation[innovLink[i]]);
	}
	free(nodes);
	for (int i=innovNode.size()-1; i>=0; i--)
		baby->node_innovations.push_back(Innovation::global_node_innovation[innovNode[i]]);

	return baby;
}

/* This method is similar to a standard single point crossover
     operator. Weights are averaged as in the previous mating
     method. A node id is chosen in the range of the organism's ids.
     We begin with the smaller organism, and when the node id is reached
	 we go on with the other organism */
Organism* Organism::mate_singlepoint(const Organism* org) const {
	Organism* baby=new Organism(); // the new organism
				// we have to initialize his hidden neurons and innovations;
				// fitness will be set elsewhere
	vector<int> innovLink,innovNode;
	list<Neuron*>::const_iterator curnode,curnode1,curnode2;
	list<IncomingLink*>::const_iterator curlink;
	// tells if genes from one organism should be chosen
	bool thisIsBetter=false,orgIsBetter=false,thisIsSmaller=true,orgIsSmaller=true;

	//preprocessing: array of new neurons indexed by id's of the parents
	// NOTE: maybe this could be improved with a global array..
	// highest ID's are typically find in the hidden layers;
	// if no hidden layer, we go through outputs neurons
	int sizeID1=0,sizeID2=0;
	if (!hidden_nodes.empty()) {
		for (curnode=hidden_nodes.begin(); curnode!=hidden_nodes.end(); curnode++)
			if ((*curnode)->id>sizeID1) sizeID1=(*curnode)->id;
	}
	else sizeID1=inputs.size()+outputs.size()-1;
	if (!org->hidden_nodes.empty()) {
		for (curnode=org->hidden_nodes.begin(); curnode!=org->hidden_nodes.end(); curnode++)
			if ((*curnode)->id>sizeID2) sizeID2=(*curnode)->id;
	}
	else sizeID2=org->inputs.size()+org->outputs.size()-1;
	
	int sizeID=max(sizeID1,sizeID2);
	Neuron** tabNodes=new Neuron*[sizeID+1];
	bool* dupHidNodes=(bool*)calloc(sizeID+1,sizeof(bool)); // mark the nodes encountered twice in the hidden layers
	bool* nodes=(bool*)calloc(sizeID+1,sizeof(bool)); // mark all the nodes inside the organism;
														// to avoid "ghost" incoming nodes
	// duplication phase
	for (curnode=inputs.begin(); curnode!=inputs.end(); curnode++) {
		tabNodes[(*curnode)->id]=new Neuron(*curnode);
		baby->inputs.push_back(tabNodes[(*curnode)->id]);
		nodes[(*curnode)->id]=true;
	}
	for (curnode=outputs.begin(); curnode!=outputs.end(); curnode++) {
		tabNodes[(*curnode)->id]=new Neuron(*curnode);
		nodes[(*curnode)->id]=true;
	}
	for (curnode=hidden_nodes.begin(); curnode!=hidden_nodes.end(); curnode++) {
		tabNodes[(*curnode)->id]=new Neuron(*curnode);
		dupHidNodes[(*curnode)->id]=true;
	}
	for (curnode=org->hidden_nodes.begin(); curnode!=org->hidden_nodes.end(); curnode++) {
		if (!dupHidNodes[(*curnode)->id]) tabNodes[(*curnode)->id]=new Neuron(*curnode);
	}
	free(dupHidNodes);

	//Figure out which genome is better
	//The worse genome should not be allowed to add extra structural baggage
	//If they are the same, use the smaller one's disjoint and excess genes only
	// (principle of Occam's razor)
	if (orig_fitness>org->orig_fitness) thisIsBetter=true;
	else if (orig_fitness==org->orig_fitness) {
		if (hidden_nodes.size() < org->hidden_nodes.size()) orgIsSmaller=false;
		else if (hidden_nodes.size() > org->hidden_nodes.size()) thisIsSmaller=false;
	}
	else orgIsBetter=true;

	// determine the crossover point
	int crossPoint;
	if (sizeID1<sizeID2) {
		if (thisIsBetter)
			crossPoint=randint(sizeID1/3,sizeID1);
		else if (orgIsBetter)
			crossPoint=randint(0,2*(sizeID1/3));
		else crossPoint=randint(0,sizeID1);
	}
	else {
		if (thisIsBetter)
			crossPoint=randint(0,2*(sizeID2/3));
		else if (orgIsBetter)
			crossPoint=randint(sizeID2/3,sizeID2);
		else crossPoint=randint(0,sizeID2);
	}

	//Now move through the Genes of each parent until both genomes end;
	//neurons and incoming links are order by increasing id, so it's relatively easy
	curnode1=hidden_nodes.begin();
	curnode2=org->hidden_nodes.begin();
	// prepare the beginning of the second half part (if needed):
	if (sizeID1<sizeID2) {
		while (curnode2!=org->hidden_nodes.end() && (*curnode2)->id<crossPoint) curnode2++;
	}
	else {
		while (curnode1!=hidden_nodes.end() && (*curnode1)->id<crossPoint) curnode1++;
	}
	while (curnode1!=hidden_nodes.end() || curnode2!=(org->hidden_nodes).end()) {
		if (curnode1==hidden_nodes.end()) {
			if (sizeID1<sizeID2 || (*curnode2)->id<crossPoint) {
				if (thisIsBetter) break; //Skip excess from the worse genome
				if (orgIsBetter || orgIsSmaller) {
					innovNode.push_back((*curnode2)->id);
					for (curlink=(*curnode2)->incoming.begin(); curlink!=(*curnode2)->incoming.end(); curlink++) {
						// if a link is disabled, it has a little chance to be enabled for the child
						if (!(*curlink)->enable && randfloat()<0.25)
							tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,true);
						else tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
						innovLink.push_back((*curlink)->innovation_num);
					}
					baby->hidden_nodes.push_back(tabNodes[(*curnode2)->id]);
					baby->nb_links+=(*curnode2)->incoming.size();
					nodes[(*curnode2)->id]=true;
				}
				curnode2++;
			}
			else break;
		}
		else if (curnode2==org->hidden_nodes.end()) {
			if (sizeID2<=sizeID1 || (*curnode1)->id<crossPoint) {
				if (orgIsBetter) break;  //Skip excess from the worse genome
				if (thisIsBetter || thisIsSmaller) {
					innovNode.push_back((*curnode1)->id);
					for (curlink=(*curnode1)->incoming.begin(); curlink!=(*curnode1)->incoming.end(); curlink++) {
						// if a link is disabled, it has a little chance to be enabled for the child
						if (!(*curlink)->enable && randfloat()<0.25)
							tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,true);
						else tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
						innovLink.push_back((*curlink)->innovation_num);
					}
					baby->hidden_nodes.push_back(tabNodes[(*curnode1)->id]);
					baby->nb_links+=(*curnode1)->incoming.size();
					nodes[(*curnode1)->id]=true;
				}
				curnode1++;
			}
			else break;
		}
		else {
			//we just look where we are relatively to the cutting point
			if (sizeID1<sizeID2) {
				if ((*curnode1)->id<crossPoint) {
					innovNode.push_back((*curnode1)->id);
					for (curlink=(*curnode1)->incoming.begin(); curlink!=(*curnode1)->incoming.end(); curlink++) {
						// if a link is disabled, it has a little chance to be enabled for the child
						if (!(*curlink)->enable && randfloat()<0.25)
							tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,true);
						else tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
						innovLink.push_back((*curlink)->innovation_num);
					}
					baby->hidden_nodes.push_back(tabNodes[(*curnode1)->id]);
					baby->nb_links+=(*curnode1)->incoming.size();
					nodes[(*curnode1)->id]=true;
					curnode1++;
				}
				else {
					innovNode.push_back((*curnode2)->id);
					for (curlink=(*curnode2)->incoming.begin(); curlink!=(*curnode2)->incoming.end(); curlink++) {
						// if a link is disabled, it has a little chance to be enabled for the child
						if (!(*curlink)->enable && randfloat()<0.25)
							tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,true);
						else tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
						innovLink.push_back((*curlink)->innovation_num);
					}
					baby->hidden_nodes.push_back(tabNodes[(*curnode2)->id]);
					baby->nb_links+=(*curnode2)->incoming.size();
					nodes[(*curnode2)->id]=true;
					curnode2++;
				}
			}
			else {
				if ((*curnode2)->id<crossPoint) {
					innovNode.push_back((*curnode2)->id);
					for (curlink=(*curnode2)->incoming.begin(); curlink!=(*curnode2)->incoming.end(); curlink++) {
						// if a link is disabled, it has a little chance to be enabled for the child
						if (!(*curlink)->enable && randfloat()<0.25)
							tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,true);
						else tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
						innovLink.push_back((*curlink)->innovation_num);
					}
					baby->hidden_nodes.push_back(tabNodes[(*curnode2)->id]);
					baby->nb_links+=(*curnode2)->incoming.size();
					nodes[(*curnode2)->id]=true;
					curnode2++;
				}
				else {
					innovNode.push_back((*curnode1)->id);
					for (curlink=(*curnode1)->incoming.begin(); curlink!=(*curnode1)->incoming.end(); curlink++) {
						// if a link is disabled, it has a little chance to be enabled for the child
						if (!(*curlink)->enable && randfloat()<0.25)
							tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,true);
						else tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
						innovLink.push_back((*curlink)->innovation_num);
					}
					baby->hidden_nodes.push_back(tabNodes[(*curnode1)->id]);
					baby->nb_links+=(*curnode1)->incoming.size();
					nodes[(*curnode1)->id]=true;
					curnode1++;
				}
			}
		}
	}
	// then look at the output nodes (of course we add all the outputs!)
	// outputs are always in the same order, for their final interpretation
	curnode1=outputs.begin();
	curnode2=(org->outputs).begin();
	while (curnode1!=outputs.end()) {
		if (sizeID1<sizeID2) {
			if ((*curnode1)->id<crossPoint) {
				for (curlink=(*curnode1)->incoming.begin(); curlink!=(*curnode1)->incoming.end(); curlink++) {
					// if a link is disabled, it has a little chance to be enabled for the child
					if (!(*curlink)->enable && randfloat()<0.25)
						tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,true);
					else tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
					if ((*curlink)->innovation_num>=0) innovLink.push_back((*curlink)->innovation_num);
				}
				baby->outputs.push_back(tabNodes[(*curnode1)->id]);
				baby->nb_links+=(*curnode1)->incoming.size();
			}
			else {
				for (curlink=(*curnode2)->incoming.begin(); curlink!=(*curnode2)->incoming.end(); curlink++) {
					// if a link is disabled, it has a little chance to be enabled for the child
					if (!(*curlink)->enable && randfloat()<0.25)
						tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,true);
					else tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
					if ((*curlink)->innovation_num>=0) innovLink.push_back((*curlink)->innovation_num);
				}
				baby->outputs.push_back(tabNodes[(*curnode2)->id]);
				baby->nb_links+=(*curnode2)->incoming.size();
			}
		}
		else {
			if ((*curnode2)->id<crossPoint) {
				for (curlink=(*curnode2)->incoming.begin(); curlink!=(*curnode2)->incoming.end(); curlink++) {
					// if a link is disabled, it has a little chance to be enabled for the child
					if (!(*curlink)->enable && randfloat()<0.25)
						tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,true);
					else tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
					if ((*curlink)->innovation_num>=0) innovLink.push_back((*curlink)->innovation_num);
				}
				baby->outputs.push_back(tabNodes[(*curnode2)->id]);
				baby->nb_links+=(*curnode2)->incoming.size();
			}
			else {
				for (curlink=(*curnode1)->incoming.begin(); curlink!=(*curnode1)->incoming.end(); curlink++) {
					// if a link is disabled, it has a little chance to be enabled for the child
					if (!(*curlink)->enable && randfloat()<0.25)
						tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,true);
					else tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
					if ((*curlink)->innovation_num>=0) innovLink.push_back((*curlink)->innovation_num);
				}
				baby->outputs.push_back(tabNodes[(*curnode1)->id]);
				baby->nb_links+=(*curnode1)->incoming.size();
			}
		}
		curnode1++;
		curnode2++;
	}

	list<Neuron*>::iterator curnode_;
	list<IncomingLink*>::iterator curlink_,deadlink; // for removal of ghost links
	// we now have to erase the incomings to "ghost" nodes:
	for (curnode_=baby->hidden_nodes.begin(); curnode_!=baby->hidden_nodes.end(); curnode_++) {
		curlink_=(*curnode_)->incoming.begin();
		while (curlink_!=(*curnode_)->incoming.end()) {
			if (!nodes[(*curlink_)->node->id]) {
				deadlink=curlink_;
				curlink_++;
				(*curnode_)->incoming.erase(deadlink);
			}
			else curlink_++;
		}
	}
	for (curnode_=baby->outputs.begin(); curnode_!=baby->outputs.end(); curnode_++) {
		curlink_=(*curnode_)->incoming.begin();
		while (curlink_!=(*curnode_)->incoming.end()) {
			if (!nodes[(*curlink_)->node->id]) {
				deadlink=curlink_;
				curlink_++;
				(*curnode_)->incoming.erase(deadlink);
			}
			else curlink_++;
		}
	}

	// finally, look through all innovations, and add them to the lists
	for (int i=innovLink.size()-1; i>=0; i--) {
		if (!nodes[Innovation::global_link_innovation[innovLink[i]]->incnode->id]) continue;
		baby->link_innovations.push_back(Innovation::global_link_innovation[innovLink[i]]);
	}
	free(nodes);
	for (int i=innovNode.size()-1; i>=0; i--)
		baby->node_innovations.push_back(Innovation::global_node_innovation[innovNode[i]]);

	return baby;
}

/* This function gives a measure of compatibility between
     two Organisms by computing a linear combination of 3
     characterizing variables of their compatibilty.
     The 3 variables represent PERCENT DISJOINT GENES,
     PERCENT EXCESS GENES, WEIGHT DIFFERENCE WITHIN
     MATCHING GENES.  So the formula for compatibility
     is inversly proportional to:
	 disjoint_coeff*pdg+excess_coeff*peg+weight_diff_coeff*mdmg.
     The 3 coefficients are global system parameters */
double Organism::compatibility(const Organism* org) const {
	list<Neuron*>::const_iterator curnode,curnode1,curnode2;
	list<IncomingLink*>::const_iterator curlink,curlink1,curlink2;

	//Set up the counters
	int num_disjoint=0;
	int num_excess=0;
	double weight_diff=0.0;
	int num_matching=0;  //Used to normalize mutation_num differences
	
	//Now move through the Genes of each parent until both genomes end;
	//neurons are order by increasing id and innovations by increasing innov_num,
	//so it's relatively easy
	curnode1=hidden_nodes.begin();
	curnode2=(org->hidden_nodes).begin();
	while (curnode1!=hidden_nodes.end() || curnode2!=(org->hidden_nodes).end()) {
		if (curnode1==hidden_nodes.end()) {
			num_excess+=(*curnode2)->incoming.size();
			curnode2++;
		}
		else if (curnode2==(org->hidden_nodes).end()) {
			num_excess+=(*curnode1)->incoming.size();
			curnode1++;
		}
		else {
			//two cases: features in both organisms, or just in one
			if ((*curnode1)->id==(*curnode2)->id) {
				curlink1=(*curnode1)->incoming.begin();
				curlink2=(*curnode2)->incoming.begin();
				while (curlink1!=(*curnode1)->incoming.end() || curlink2!=(*curnode2)->incoming.end()) {
					if (curlink1==(*curnode1)->incoming.end()) {
						num_disjoint++;
						curlink2++;
					}
					else if (curlink2==(*curnode2)->incoming.end()) {
						num_disjoint++;
						curlink1++;
					}
					else {
						// two cases: the links point the same node, or not
						if ((*curlink1)->node->id<(*curlink2)->node->id) {
							num_disjoint++;
							curlink1++;
						}
						else if ((*curlink1)->node->id>(*curlink2)->node->id) {
							num_disjoint++;
							curlink2++;
						}
						else {
							num_matching++;
							weight_diff+=fabs((*curlink1)->weight-(*curlink2)->weight);
							curlink1++;
							curlink2++;
						}
					}
				}
				curnode1++;
				curnode2++;
			}
			else if ((*curnode1)->id<(*curnode2)->id) {
				num_disjoint+=(*curnode1)->incoming.size();
				curnode1++;
			}
			else {
				num_disjoint+=(*curnode2)->incoming.size();
				curnode2++;
			}
		}
	}
	// then look at the output nodes
	curnode1=outputs.begin();
	curnode2=(org->outputs).begin();
	while (curnode1!=outputs.end()) {
		curlink1=(*curnode1)->incoming.begin();
		curlink2=(*curnode2)->incoming.begin();
		while (curlink1!=(*curnode1)->incoming.end() || curlink2!=(*curnode2)->incoming.end()) {
			if (curlink1==(*curnode1)->incoming.end()) {
				num_disjoint++;
				curlink2++;
			}
			else if (curlink2==(*curnode2)->incoming.end()) {
				num_disjoint++;
				curlink1++;
			}
			else {
				// two cases: the links point the same node, or not
				if ((*curlink1)->node->id<(*curlink2)->node->id) {
					num_disjoint++;
					curlink1++;
				}
				else if ((*curlink1)->node->id>(*curlink2)->node->id) {
					num_disjoint++;
					curlink2++;
				}
				else {
					num_matching++;
					weight_diff+=fabs((*curlink1)->weight-(*curlink2)->weight);
					curlink1++;
					curlink2++;
				}
			}
		}
		curnode1++;
		curnode2++;
	}

	// finally, calculate the formula
	return NEAT::disjoint_coeff*num_disjoint+NEAT::excess_coeff*num_excess+NEAT::weight_diff_coeff*((weight_diff+1.0)/(num_matching+1));
}

void Organism::print_to_file(char* filename) {
	ofstream outFile(filename);
	//Make sure it worked
	if (!outFile.is_open()) {
		cerr << "Can't open " << filename << " for output" << endl;
		return;
	}
	outFile << "genomestart " << 0 << endl;
	list<Neuron*>::iterator curnode;
	list<IncomingLink*>::iterator curlink;
	for (curnode=inputs.begin(); curnode!=inputs.end(); curnode++)
		outFile << "incnode " << (*curnode)->id << " " << (*curnode)->ftype << endl;
	for (curnode=hidden_nodes.begin(); curnode!=hidden_nodes.end(); curnode++) {
		outFile << "midnode " << (*curnode)->id << " " << (*curnode)->ftype << " ";
		for (curlink=(*curnode)->incoming.begin(); curlink!=(*curnode)->incoming.end(); curlink++)
			outFile << (*curlink)->node->id << " " << (*curlink)->innovation_num << " " << (*curlink)->weight << " ";
		outFile << ";" << endl;
	}
	for (curnode=outputs.begin(); curnode!=outputs.end(); curnode++) {
		outFile << "outnode " << (*curnode)->id << " " << (*curnode)->ftype << " ";
		for (curlink=(*curnode)->incoming.begin(); curlink!=(*curnode)->incoming.end(); curlink++)
			outFile << (*curlink)->node->id << " " << (*curlink)->innovation_num << " " << (*curlink)->weight << " ";
		outFile << ";" << endl;
	}
	outFile << "genomeend" << endl;
}
