#ifndef _Population
#define _Population

#include <iostream>
#include <fstream>
#include <list>
#include "Functions.h"
#include "IDCounters.h"
#include "Definitions.h"
#include "Innovation.h"
#include "Organism.h"
#include "Species.h"
#include "Neat.h"
#include "Timer.h"

using namespace std;

//#define SEARCH_TIME // if activated, epoch times are limited
//#define FASTEST_SEARCH // keep only a few species at a time; can miss good organisms..

/* POPULATION CLASS:
   A Population is a group of Organisms   
   including their species                        */
/* ---------------------------------------------  */

class Population {

public:
	
	// to look at what is happening:
	static ofstream trace;

	static constexpr double MAX_DOUBLE=1000.0; // unreachable compatibility measure; depend on the application
	
	list<Organism*> organisms; //The organisms in the Population
	list<Species*> species;  /* Species in the Population
							Note that the species should comprise all the genomes */
	/* When do we need to delta code? */
	double highest_fitness; //Stagnation detector
	int highest_last_changed; //If too high, leads to delta coding (==focus search on the two best species)

	Population () {
		highest_fitness=0.0;
		highest_last_changed=0;
	}
	
	/* Construct off of a single spawning Genome */
	Population(Organism* org,int size) {
		highest_fitness=0.0;
		highest_last_changed=0;
		spawn(org,size);
	}

	/* Construct off of a file of Genomes */
	Population(char* filename) {
		char curword[20]; //max word size of 20 characters
		ifstream iFile(filename);
		//Make sure it worked
		if (!iFile.is_open()) {
			cerr << "Can't open " << filename << " for input" << endl;
		}
		else {
			//Loop until file is finished, parsing each line
			while (iFile >> curword) {
				//Check for next
				if (strcmp(curword,"genomestart")==0) organisms.push_back(new Organism(iFile));
				//Ignore comments surrounded by /* */ - they get printed to screen
				else if (strcmp(curword,"/*")==0) {
					iFile >> curword;
					while (strcmp(curword,"*/")!=0) {
						cout << curword << " ";
						iFile >> curword;
					}
					cout << endl;
				}
			}
			iFile.close();
			speciate();
		}
		highest_fitness=0.0;
		highest_last_changed=0;
	}

	~Population() {
		for (list<Species*>::iterator curspec=species.begin(); curspec!=species.end(); curspec++)
			delete *curspec;
		for (list<Organism*>::iterator curorg=organisms.begin(); curorg!=organisms.end(); curorg++)
			delete *curorg;
	}
	
	/* A Population can be spawned off of a single genome/organism */
	void spawn(Organism* org,int size);
	void speciate(); //Separate the Organisms into species

	//Turnover the population to a new generation using fitness
	// return true if all happens OK in time
	bool epoch(double epoch_duration);
	//Perform mating and mutation to form next generation
	void reproduce(Species* spec);
	
	//Print Population to a file specified by a string 
	void print_to_file(char* filename);
};

#endif
