#include "Species.h"

// PRECONDITION: all the organisms have been tested and have fitness initialised
//Can change the fitness of the organisms in the Species to be higher 
//for very new species (to protect them).
//Divides the fitness by the size of the Species, so that fitness is "shared" by the species
void Species::adjust_fitness() {
	list<Organism*>::iterator curorg,deadorg;
	int age_debt=max(0,(age-age_of_last_improvement+1)-NEAT::dropoff_age);
	for (curorg=organisms.begin(); curorg!=organisms.end(); curorg++) {
		//Remember the original fitness before it gets modified
		(*curorg)->orig_fitness=(*curorg)->fitness;
		//Make fitness decrease after a stagnation point dropoff_age
		(*curorg)->fitness*=(1-(2*atan((double)age_debt)/M_PI));
		//Give a fitness boost up to some young age (niching)
		//The age_significance parameter is a system parameter
		//if it is 1, then young species get no fitness boost
		if (age<=NEAT::young_limit) ((*curorg)->fitness)*=NEAT::age_significance; 
		//Share fitness with the species
		(*curorg)->fitness=((*curorg)->fitness)/organisms.size();
	}

	//Sort the population
	sortList<Organism>(organisms);

	//Update age_of_last_improvement here
	if (((*(organisms.begin()))->orig_fitness)>max_fitness_ever) {
		age_of_last_improvement=age;
		max_fitness_ever=(*(organisms.begin()))->orig_fitness;
	}

	//Decide how many get to reproduce based on survival_thresh*pop_size
	//Adding 1.0 ensures that at least one will survive, unless all are too old
	// it's very useful for young species with few members
	int count=(int)floor(NEAT::survival_thresh*organisms.size()+1.0)-1;
	//delete those who are ranked too low, or too old to be parents
	//human analogy: if generation>=3, we delete (means only children/parents/grandparents);
	curorg=organisms.begin();
	while (count>=0) {
		if (curorg==organisms.end()) break;
		(*curorg)->generation++;
		if ((*curorg)->generation>=3) {
			(*curorg)->eliminate=true;
			//Remember where we are
			deadorg=curorg;
			curorg++;
			//Remove the organism from the master list
			organisms.erase(deadorg);
		}
		else {
			count--;
			curorg++;
		}
	}
	while (curorg!=organisms.end()) {
		(*curorg)->eliminate=true;
		//Remember where we are
		deadorg=curorg;
		curorg++;
		//Remove the organism from the master list
		organisms.erase(deadorg);
	}
}

//Compute the offspring each member of the species will get, based on
//the average population adjusted fitness or to get a fixed size (for DELTA-CODING)
void Species::compute_org_offspring(double ave_pop_fitness,int size) {
	list<Organism*>::iterator curorg;
	int expected_offspring;
	if (size==0) {
		double countFit=0.0;
		for (curorg=organisms.begin(); curorg!=organisms.end(); curorg++) countFit+=(*curorg)->fitness;
		if (randbit()) expected_offspring=(int)(ceil(countFit/ave_pop_fitness));
		else expected_offspring=(int)(floor(countFit/ave_pop_fitness));
	}
	else expected_offspring=size;
	
	// so the average number of children for each organism is :
	double ave_children=(double)expected_offspring/organisms.size(),sum=0.0;
	int counter=0;
	int floor_children=(int)(floor(ave_children)),ceil_children=(int)(ceil(ave_children));
	for (curorg=organisms.begin(); curorg!=organisms.end(); curorg++) {
		if (sum<counter*ave_children) {
			(*curorg)->expected_offspring=ceil_children;
			sum+=ceil_children;
		}
		else {
			(*curorg)->expected_offspring=floor_children;
			sum+=floor_children;
		}
		counter++;
	}
}
