#ifndef _Species
#define _Species

#include <iostream>
#include <fstream>
#include <list>
#include "Functions.h"
#include "IDCounters.h"
#include "Definitions.h"
#include "Innovation.h"
#include "Organism.h"
#include "Neat.h"

using namespace std;

/* SPECIES CLASS:
   A Species is a group of similar Organisms      
   Reproduction takes place mostly within a
   single species, so that compatible organisms
   can mate.                                      */
/* ---------------------------------------------  */

class Species {

public:

	int age,id; //The age and id of the Species
	int age_of_last_improvement; //If this is too long ago, the Species will goes extinct
	double max_fitness_ever; //The max fitness the species ever had
	list<Organism*> organisms; //The organisms in the Species
	
	Species () {
		id=cur_spec_id++;
		age=-1; // -1 to allow young species more chances to evolve (look at the function adjust_fitness())
		age_of_last_improvement=0;
		max_fitness_ever=0.0;
	}

	~Species() {}

	//Change the fitness of all the organisms in the species 
	//to possibly depend slightly on the age of the species
	//and then divide it by the size of the species so that the
	//organisms in the species "share" the fitness
	void adjust_fitness();

	//Counts the number of offspring expected from all its members
	void compute_org_offspring(double ave_pop_fitness,int size);
	
	//This is used for list sorting of Species by fitness of best organism
	//highest fitness first
	bool operator<(const Species& spec) const {
		// PRECONDITION: organisms have been sorted from highest fitness to lowest
		if (organisms.empty()) return false;
		if (spec.organisms.empty()) return true;
		return (*(organisms.begin()))->orig_fitness > (*(spec.organisms.begin()))->orig_fitness;
	}
};

#endif
