#include "GeneticGenome.h"

//Inserts a NNode into a given ordered list of NNodes in order
void Genome::node_insert(list<NNode*> &nlist,NNode *n) {
	list<NNode*>::iterator curnode;
	int id=n->node_id;
	curnode=nlist.begin();
	while ((curnode!=nlist.end())&&(((*curnode)->node_id)<id)) ++curnode;
	nlist.insert(curnode,n);
}

//Return id of final NNode in Genome
int Genome::get_last_node_id() {
	return ((*(nodes.rbegin()))->node_id)+1;
}  

//Return last innovation number in Genome
double Genome::get_last_gene_innovnum() {
	return ((*(genes.rbegin()))->innovation_num)+1;       
}

//Return a pointer to a new genome that is a copy of this one
Genome *Genome::duplicate(int new_id) {
	//Collections for the new Genome
	vector<Trait*> traits_dup;
	list<NNode*> nodes_dup;
	list<Gene*> genes_dup;
	//Iterators for the old Genome
	vector<Trait*>::iterator curtrait;
	list<NNode*>::iterator curnode;
	list<Gene*>::iterator curgene;
	//New item pointers
	Trait *newtrait;
	NNode *newnode;
	Gene *newgene;
	Trait *assoc_trait;  //Trait associated with current item
	NNode *inode; //For forming a gene 
	NNode *onode; //For forming a gene
	Trait *traitptr;
	Genome *newgenome;

	//Duplicate the traits
	for(curtrait=traits.begin();curtrait!=traits.end();++curtrait) {
		newtrait=new Trait(*curtrait);
		traits_dup.push_back(newtrait);
	}
	//Duplicate NNodes
	for(curnode=nodes.begin();curnode!=nodes.end();++curnode) {
		//First, find the trait that this node points to
		if (((*curnode)->nodetrait)==0) assoc_trait=0;
		else {
			curtrait=traits_dup.begin();
			while(((*curtrait)->trait_id)!=(((*curnode)->nodetrait)->trait_id)) ++curtrait;
			assoc_trait=(*curtrait);
		}
		newnode=new NNode(*curnode,assoc_trait);
		(*curnode)->dup=newnode;  //Remember this node's old copy
		//(*curnode)->activation_count=55;
		nodes_dup.push_back(newnode);    
	}
	//Duplicate Genes
	for(curgene=genes.begin();curgene!=genes.end();++curgene) {
		//First find the nodes connected by the gene's link
		inode=(((*curgene)->lnk)->in_node)->dup;
		onode=(((*curgene)->lnk)->out_node)->dup;
		//Get a pointer to the trait expressed by this gene
		traitptr=((*curgene)->lnk)->linktrait;
		if (traitptr==0) assoc_trait=0;
		else {
			curtrait=traits_dup.begin();
			while(((*curtrait)->trait_id)!=(traitptr->trait_id)) ++curtrait;
			assoc_trait=(*curtrait);
		}
		newgene=new Gene(*curgene,assoc_trait,inode,onode);
		genes_dup.push_back(newgene);
	}
	//Finally, return the genome
	newgenome=new Genome(new_id,traits_dup,nodes_dup,genes_dup);
	return newgenome;
}

//For debugging: A number of tests can be run on a genome to check its
// integrity
//Note: Some of these tests do not indicate a bug, but rather are meant
//  to be used to detect specific system states
bool Genome::verify() {
	list<NNode*>::iterator curnode;
	list<Gene*>::iterator curgene;
	list<Gene*>::iterator curgene2;
	NNode *inode;
	NNode *onode;
	bool disab;
	int last_id;
	int pause;
	//cout<<"Verifying Genome id: "<<this->genome_id<<endl;
	if (this==0) {
		cout<<"ERROR GENOME EMPTY"<<endl;
		cin>>pause;
	}

	//Check each gene's nodes
	for(curgene=genes.begin();curgene!=genes.end();++curgene) {
		inode=((*curgene)->lnk)->in_node;
		onode=((*curgene)->lnk)->out_node;
		//Look for inode
		curnode=nodes.begin();
		while((curnode!=nodes.end())&&((*curnode)!=inode)) ++curnode;
		if (curnode==nodes.end()) {
			cout<<"MISSING iNODE FROM GENE NOT IN NODES OF GENOME!!"<<endl;
			cin>>pause;
			return false;
		}
		//Look for onode
		curnode=nodes.begin();
		while((curnode!=nodes.end())&&((*curnode)!=onode)) ++curnode;
		if (curnode==nodes.end()) {
			cout<<"MISSING oNODE FROM GENE NOT IN NODES OF GENOME!!"<<endl;
			cin>>pause;
			return false;
		}
	}
	//Check for NNodes being out of order
	last_id=0;
	for(curnode=nodes.begin();curnode!=nodes.end();++curnode) {
		if ((*curnode)->node_id<last_id) {
			cout<<"ALERT: NODES OUT OF ORDER in "<<this<<endl;
			cin>>pause;
			return false;
		}
		last_id=(*curnode)->node_id;
	}

	//Make sure there are no duplicate genes
	for(curgene=genes.begin();curgene!=genes.end();++curgene) {
		for(curgene2=genes.begin();curgene2!=genes.end();++curgene2) {
			if (((*curgene)!=(*curgene2))&&
				((((*curgene)->lnk)->is_recurrent)==(((*curgene2)->lnk)->is_recurrent))&&
				((((((*curgene2)->lnk)->in_node)->node_id)==((((*curgene)->lnk)->in_node)->node_id))&&
				(((((*curgene2)->lnk)->out_node)->node_id)==((((*curgene)->lnk)->out_node)->node_id)))) {
				cout<<"ALERT: DUPLICATE GENES: "<<(*curgene)<<(*curgene2)<<endl;
				cout<<"INSIDE GENOME: "<<this<<endl;
				cin>>pause;
			}
		}
	}
	//Check for 2 disables in a row
	//Note:  Again, this is not necessarily a bad sign
	if (nodes.size()>=500) {
		disab=false;
		for(curgene=genes.begin();curgene!=genes.end();++curgene) {
			if ((((*curgene)->enable)==false)&&(disab==true)) {
				cout<<"ALERT: 2 DISABLES IN A ROW: "<<this<<endl;
			}
			if (((*curgene)->enable)==false) disab=true;
			else disab=false;
		}
	}
	//cout<<"GENOME OK!"<<endl;
	return true;
}

void Genome::mutate_random_trait() {
	vector<Trait*>::iterator thetrait; //Trait to be mutated
	int traitnum;
	//Choose a random traitnum
	traitnum=randint(0,(traits.size())-1);
	//Retrieve the trait and mutate it
	thetrait=traits.begin();
	(*(thetrait[traitnum])).mutate();
	//TRACK INNOVATION? (future possibility)
}

//This chooses a random gene, extracts the link from it,
//and repoints the link to a random trait
void Genome::mutate_link_trait(int times) {
	int traitnum;
	int genenum;
	list<Gene*>::iterator thegene;     //Link to be mutated
	vector<Trait*>::iterator thetrait; //Trait to be attached
	int count;
	int loop;
	for(loop=1;loop<=times;loop++) {
		//Choose a random traitnum
		traitnum=randint(0,(traits.size())-1);
		//Choose a random linknum
		genenum=randint(0,genes.size()-1);
		//set the link to point to the new trait
		thegene=genes.begin();
		for(count=0;count<genenum;count++) ++thegene;
		thetrait=traits.begin();
		((*thegene)->lnk)->linktrait=thetrait[traitnum];
		//TRACK INNOVATION- future use
		//(*thegene)->mutation_num+=randposneg()*randfloat()*linktrait_mut_sig;
	}
}

//This chooses a random node
//and repoints the node to a random trait
void Genome::mutate_node_trait(int times) {
	int traitnum;
	int nodenum;
	list<NNode*>::iterator thenode;     //Link to be mutated
	list<Gene*>::iterator thegene;  //Gene to record innovation
	vector<Trait*>::iterator thetrait; //Trait to be attached
	int count;
	int loop;

	for(loop=1;loop<=times;loop++) {
		//Choose a random traitnum
		traitnum=randint(0,(traits.size())-1);    
		//Choose a random nodenum
		nodenum=randint(0,nodes.size()-1);
		//set the link to point to the new trait
		thenode=nodes.begin();
		for(count=0;count<nodenum;count++) ++thenode;
		thetrait=traits.begin();
		(*thenode)->nodetrait=thetrait[traitnum];
		//TRACK INNOVATION! - possible future use
	}
}

/* Add Gaussian noise to linkweights either GAUSSIAN
   or COLDGAUSSIAN (from zero) */
/* COLDGAUSSIAN means ALL connection weights will be given completely 
   new values */
void Genome::mutate_link_weights(double power,double rate,mutator mut_type) {
	list<Gene*>::iterator curgene;
	double num;  //counts gene placement
	double gene_total;
	double powermod; //Modified power by gene number
	//The power of mutation will rise farther into the genome
	//on the theory that the older genes are more fit since
	//they have stood the test of time
	double randnum;
	double randchoice; //Decide what kind of mutation to do on a gene
	double endpart; //Signifies the last part of the genome
	double gausspoint;
	double coldgausspoint;
	bool severe;  //Once in a while really shake things up

  //Wright variables
  //double oldval;
  //double perturb;
  /* --------------- WRIGHT'S MUTATION METHOD -------------- 
  //Use the fact that we know ends are newest
  gene_total=(double) genes.size();
  endpart=gene_total*0.8;
  num=0.0;
  for(curgene=genes.begin();curgene!=genes.end();curgene++) {
    //Mutate rate 0.2 controls how many params mutate in the vector
    if ((randfloat()<rate)||
	((gene_total>=10.0)&&(num>endpart))) {
      oldval=((*curgene)->lnk)->weight;
      //The amount to perturb the value by
      perturb=randfloat()*power;
      //Once in a while leave the end part alone
      if (num>endpart)
	if (randfloat()<0.2) perturb=0;  
      //Decide positive or negative
      if (randbit()) {
	//Positive case
	//if it goes over the max, find something smaller
	if (oldval+perturb>100.0) {
	  perturb=(100.0-oldval)*randfloat();
	}
	((*curgene)->lnk)->weight+=perturb;
      }
      else {
	//Negative case
	//if it goes below the min, find something smaller
	if (oldval-perturb<100.0) {
	  perturb=(oldval+100.0)*randfloat();
	}
	((*curgene)->lnk)->weight-=perturb;
      }
    }
    num+=1.0;
  }
  */

	if (randfloat()>0.5) severe=true;
	else severe=false;
	//Go through all the Genes and perturb their link's weights
	num=0.0;
	gene_total=(double) genes.size();
	endpart=gene_total*0.8;
	//powermod=randposneg()*power*randfloat();  //Make power of mutation random
	//powermod=randfloat();
	powermod=1.0;

  //Possibility: Jiggle the newest gene randomly
  //if (gene_total>10.0) {
  //  lastgene=genes.end();
  //  lastgene--;
  //  if (randfloat()>0.4)
  //    ((*lastgene)->lnk)->weight+=0.5*randposneg()*randfloat();
  //}

	//Loop on all genes
	for(curgene=genes.begin();curgene!=genes.end();curgene++) {     
		//Possibility: Have newer genes mutate with higher probability
		//Only make mutation power vary along genome if it's big enough
		//if (gene_total>=10.0) {
		  //This causes the mutation power to go up towards the end up the genome
		  //powermod=((power-0.7)/gene_total)*num+0.7;
		//}
		//else powermod=power;

		//The following if determines the probabilities of doing cold gaussian
		//mutation, meaning the probability of replacing a link weight with
		//another, entirely random weight.  It is meant to bias such mutations
		//to the tail of a genome, because that is where less time-tested genes
		//reside.  The gausspoint and coldgausspoint represent values above
		//which a random float will signify that kind of mutation.  
		if (severe) {
			gausspoint=0.3;
			coldgausspoint=0.1;
		}
		else if ((gene_total>=10.0)&&(num>endpart)) {
			gausspoint=0.5;  //Mutate by modification % of connections
			coldgausspoint=0.3; //Mutate the rest by replacement % of the time
		}
		else {
			//Half the time don't do any cold mutations
			if (randfloat()>0.5) {
				gausspoint=1.0-rate;
				coldgausspoint=1.0-rate-0.1;
			}
			else {
				gausspoint=1.0-rate;
				coldgausspoint=1.0-rate;
			}
		}

		//Possible methods of setting the perturbation:
		//randnum=gaussrand()*powermod;
		//randnum=gaussrand();
		randnum=randposneg()*randfloat()*power*powermod;
		if (mut_type==GAUSSIAN) {
			randchoice=randfloat();
			if (randchoice>gausspoint) ((*curgene)->lnk)->weight+=randnum;
			else if (randchoice>coldgausspoint) ((*curgene)->lnk)->weight=randnum;
		}
		else if (mut_type==COLDGAUSSIAN) ((*curgene)->lnk)->weight=randnum;
		//Record the innovation
		//(*curgene)->mutation_num+=randnum;
		(*curgene)->mutation_num=((*curgene)->lnk)->weight;
		num+=1.0;
	}
}

/* Toggle genes from enable on to enable off or 
   vice versa.  Do it times times.  */
void Genome::mutate_toggle_enable(int times) {
	int genenum;
	int count;
	list<Gene*>::iterator thegene;  //Gene to toggle
	list<Gene*>::iterator checkgene;  //Gene to check
	int genecount;

	for (count=1;count<=times;count++) {     
		//Choose a random genenum
		genenum=randint(0,genes.size()-1);
		//find the gene
		thegene=genes.begin();
		for(genecount=0;genecount<genenum;genecount++) ++thegene;
		//Toggle the enable on this gene
		if (((*thegene)->enable)==true) {
			//We need to make sure that another gene connects out of the in-node
			//Because if not a section of network will break off and become isolated
			checkgene=genes.begin();
			while((checkgene!=genes.end())&&
				(((((*checkgene)->lnk)->in_node)!=(((*thegene)->lnk)->in_node))||
				(((*checkgene)->enable)==false)||
				((*checkgene)->innovation_num==(*thegene)->innovation_num)))
				++checkgene;
			//Disable the gene if it's safe to do so
			if (checkgene!=genes.end()) (*thegene)->enable=false;
		}
		else (*thegene)->enable=true;
	}
}

/* Find first disabled gene and enable it */
void Genome::mutate_gene_reenable() {
	list<Gene*>::iterator thegene;  //Gene to enable
	thegene=genes.begin();
	//Search for a disabled gene
	while((thegene!=genes.end())&&((*thegene)->enable==true)) ++thegene;
	//Reenable it
	if (thegene!=genes.end())
		if (((*thegene)->enable)==false) (*thegene)->enable=true;
}

//This mutator adds a node to a Genome by inserting it in the middle
//of an existing link between 2 nodes.  This broken link will be disabled
//and now represented by 2 links with the new node between them.
//The innovs list passed in is used to compare the innovation
//with other innovations in the list and see whether they match.
//If they do, the same innovation numbers will be assigned to the new
//genes. If a disabled link is chosen, then the method just exits with false
bool Genome::mutate_add_node(list<Innovation*> &innovs,int &curnode_id,double &curinnov) {
	list<Gene*>::iterator thegene;  //random gene containing the original link
	int genenum;  //The random gene number
	NNode *in_node; //Here are the nodes connected by the gene
	NNode *out_node; 
	Link *thelink;  //The link inside the random gene
	//double randmult;  //using a gaussian to find the random gene
	list<Innovation*>::iterator theinnov; //For finding a historical match
	bool done=false;
	Gene *newgene1;  //The new Genes
	Gene *newgene2;
	NNode *newnode;   //The new NNode
	Trait *traitptr; //The original link's trait
	//double splitweight;  //If used, Set to sqrt(oldweight of oldlink)
	double oldweight;  //The weight of the original link
	int trycount;  //Take a few tries to find an open node
	bool found;

	//First, find a random gene already in the genome  
	trycount=0;
	found=false;

	//For a very small genome, we need to bias splitting towards
	//older links to avoid a "chaining" effect which is likely
	//to occur when we keep splitting between the same two
	//nodes over and over again (on newer and newer connections)
	if (genes.size()<15) {
		thegene=genes.begin();
		while (((thegene!=genes.end())
			&&(!((*thegene)->enable)))||
			((thegene!=genes.end())
			&&(((*thegene)->lnk->in_node)->gen_node_label==BIAS)))
			++thegene;
		//Now randomize which node is chosen at this point
		//We bias the search towards older genes because 
		//this encourages splitting to distribute evenly
		while (((thegene!=genes.end())&&
			(randfloat()<0.3))||
			((thegene!=genes.end())
			&&(((*thegene)->lnk->in_node)->gen_node_label==BIAS)))
			++thegene;
		if ((!(thegene==genes.end()))&&
			((*thegene)->enable))
			found=true;
	}
	//In this else:
	//Alternative uniform random choice of genes
	//When the genome is not tiny, it is safe to choose randomly
	else {
		while ((trycount<20)&&(!found)) {
			//Choose a random genenum
			//Possible gaussian method
			//randmult=gaussrand()/4;
			//if (randmult>1.0) randmult=1.0;
			//This tends to select older genes for splitting
			//genenum=(int) floor((randmult*(genes.size()-1.0))+0.5);
      
			//Pure random splitting
			genenum=randint(0,genes.size()-1);
			//find the gene
			thegene=genes.begin();
			for(int genecount=0;genecount<genenum;genecount++) ++thegene;
			//If either the gene is disabled, or it has a bias input, try again
			if (!(((*thegene)->enable==false)||
				(((((*thegene)->lnk)->in_node)->gen_node_label)==BIAS)))
				found=true;
			++trycount;
		}
	}

	//If we couldn't find anything say goodbye
	if (!found) return false;
	//Disabled the gene
	(*thegene)->enable=false;
	//Extract the link
	thelink=(*thegene)->lnk;
	oldweight=(*thegene)->lnk->weight;
	//Extract the nodes
	in_node=thelink->in_node;
	out_node=thelink->out_node;

	//Check to see if this innovation has already been done   
	//in another genome
	//Innovations are used to make sure the same innovation in
	//two separate genomes in the same generation receives
	//the same innovation number.
	theinnov=innovs.begin();
	while(!done) {
		if (theinnov==innovs.end()) {
			//The innovation is totally novel
			//Get the old link's trait
			traitptr=thelink->linktrait;
			//Create the new NNode
			//By convention, it will point to the first trait
			newnode=new NNode(NEURON,curnode_id++,HIDDEN);
			newnode->nodetrait=(*(traits.begin()));
			//Create the new Genes
			if (thelink->is_recurrent) {
				newgene1=new Gene(traitptr,1.0,in_node,newnode,TRUE,curinnov,0);
				newgene2=new Gene(traitptr,oldweight,newnode,out_node,FALSE,curinnov+1,0);
				curinnov+=2.0;
			}
			else {
				newgene1=new Gene(traitptr,1.0,in_node,newnode,FALSE,curinnov,0);
				newgene2=new Gene(traitptr,oldweight,newnode,out_node,FALSE,curinnov+1,0);
				curinnov+=2.0;
			}
			//Add the innovations (remember what was done)
			innovs.push_back(new Innovation(in_node->node_id,out_node->node_id,curinnov-2.0,curinnov-1.0,newnode->node_id,(*thegene)->innovation_num));      
			done=true;
		}
		/* We check to see if an innovation already occured that was:
       -A new node
       -Stuck between the same nodes as were chosen for this mutation
       -Splitting the same gene as chosen for this mutation 
       If so, we know this mutation is not a novel innovation
       in this generation
       so we make it match the original, identical mutation which occured
       elsewhere in the population by coincidence */
		else if (((*theinnov)->innovation_type==NEWNODE)&&
				((*theinnov)->node_in_id==(in_node->node_id))&&
				((*theinnov)->node_out_id==(out_node->node_id))&&
				((*theinnov)->old_innov_num==(*thegene)->innovation_num)) {
			//Here, the innovation has been done before
			//Get the old link's trait
			traitptr=thelink->linktrait;
			//Create the new NNode
			newnode=new NNode(NEURON,(*theinnov)->newnode_id,HIDDEN);      
			//By convention, it will point to the first trait
			//Note: In future may want to change this
			newnode->nodetrait=(*(traits.begin()));
			//Create the new Genes
			if (thelink->is_recurrent) {
				newgene1=new Gene(traitptr,1.0,in_node,newnode,TRUE,(*theinnov)->innovation_num1,0);
				newgene2=new Gene(traitptr,oldweight,newnode,out_node,FALSE,(*theinnov)->innovation_num2,0);
			}
			else {
				newgene1=new Gene(traitptr,1.0,in_node,newnode,FALSE,(*theinnov)->innovation_num1,0);
				newgene2=new Gene(traitptr,oldweight,newnode,out_node,FALSE,(*theinnov)->innovation_num2,0);
			}
			done=true;
		}
		else ++theinnov;
	}

	//Now add the new NNode and new Genes to the Genome
	genes.push_back(newgene1);
	genes.push_back(newgene2);
	node_insert(nodes,newnode);
	return true;
} 

/* Mutate the genome by adding a new link between 2 random NNodes */
/* If the 2 NNodes are already connected, keep trying tries times */
bool Genome::mutate_add_link(list<Innovation*> &innovs,double &curinnov,int tries) {
	int nodenum1,nodenum2;  //Random node numbers
	list<NNode*>::iterator thenode1,thenode2;  //Random node iterators
	int nodecount;  //Counter for finding nodes
	int trycount; //Iterates over attempts to find an unconnected pair of nodes
	NNode *nodep1; //Pointers to the nodes
	NNode *nodep2; //Pointers to the nodes
	list<Gene*>::iterator thegene; //Searches for existing link
	bool found=false;  //Tells whether an open pair was found
	list<Innovation*>::iterator theinnov; //For finding a historical match
	int recurflag; //Indicates whether proposed link is recurrent
	Gene *newgene;  //The new Gene
	int traitnum;  //Random trait finder
	vector<Trait*>::iterator thetrait;
	double newweight;  //The new weight for the new link
	bool done;
	bool do_recur;
	bool loop_recur;
	int first_nonsensor;

	//These are used to avoid getting stuck in an infinite loop checking
	//for recursion
	//Note that we check for recursion to control the frequency of
	//adding recurrent links rather than to prevent any paricular
	//kind of error
	int thresh=(nodes.size())*(nodes.size());
	int count=0;

	//Make attempts to find an unconnected pair
	trycount=0;
	//Decide whether to make this recurrent
	if (randfloat()<recur_only_prob) do_recur=true;
	else do_recur=false;
	//Find the first non-sensor so that the to-node won't look at sensors as
	//possible destinations
	first_nonsensor=0;
	thenode1=nodes.begin();
	while(((*thenode1)->get_type())==SENSOR) {
		first_nonsensor++;
		++thenode1;
	}
	//Here is the recurrent finder loop- it is done separately
	if (do_recur) {
		while(trycount<tries) {
			//Some of the time try to make a recur loop
			if (randfloat()>0.5) {
				loop_recur=true;
			}
			else loop_recur=false;
			if (loop_recur) {
				nodenum1=randint(first_nonsensor,nodes.size()-1);
				nodenum2=nodenum1;
			}
			else {
				//Choose random nodenums
				nodenum1=randint(0,nodes.size()-1);
				nodenum2=randint(first_nonsensor,nodes.size()-1);
			}
			//Find the first node
			thenode1=nodes.begin();
			for(nodecount=0;nodecount<nodenum1;nodecount++) ++thenode1;
			//Find the second node
			thenode2=nodes.begin();
			for(nodecount=0;nodecount<nodenum2;nodecount++) ++thenode2;
			nodep1=(*thenode1);
			nodep2=(*thenode2);
			//See if a recur link already exists  ALSO STOP AT END OF GENES!!!!
			thegene=genes.begin();
			while ((thegene!=genes.end()) && 
				((nodep2->type)!=SENSOR) &&   //Don't allow SENSORS to get input
				(!((((*thegene)->lnk)->in_node==nodep1)&&
				(((*thegene)->lnk)->out_node==nodep2)&&
				((*thegene)->lnk)->is_recurrent))) 
				++thegene;
			if (thegene!=genes.end()) trycount++;
			else {
				count=0;
				recurflag=false;//phenotype->is_recur(nodep1->analogue,nodep2->analogue,count,thresh);
				//Exit if the network is faulty (contains an infinite loop)
				if (count>thresh) {
					cout<<"LOOP DETECTED DURING A RECURRENCY CHECK"<<endl;
					return false;
				}
				//Make sure it finds the right kind of link (recur)
				if (!(recurflag)) trycount++;
				else {
					trycount=tries;
					found=true;
				}
			}
		}
	}
	else {
		//Loop to find a nonrecurrent link
		while(trycount<tries) {
			//cout<<"TRY "<<trycount<<endl;
			//Choose random nodenums
			nodenum1=randint(0,nodes.size()-1);
			nodenum2=randint(first_nonsensor,nodes.size()-1);
			//Find the first node
			thenode1=nodes.begin();
			for(nodecount=0;nodecount<nodenum1;nodecount++) ++thenode1;
			//cout<<"RETRIEVED NODE# "<<(*thenode1)->node_id<<endl;
			//Find the second node
			thenode2=nodes.begin();
			for(nodecount=0;nodecount<nodenum2;nodecount++) ++thenode2;
			nodep1=(*thenode1);
			nodep2=(*thenode2);
			//See if a link already exists  ALSO STOP AT END OF GENES!!!!
			thegene=genes.begin();
			while ((thegene!=genes.end()) && 
				((nodep2->type)!=SENSOR) &&   //Don't allow SENSORS to get input
				(!((((*thegene)->lnk)->in_node==nodep1)&&
				(((*thegene)->lnk)->out_node==nodep2)&&
				(!(((*thegene)->lnk)->is_recurrent)))))
				++thegene;
			if (thegene!=genes.end()) trycount++;
			else {
				count=0;
				recurflag=false;//phenotype->is_recur(nodep1->analogue,nodep2->analogue,count,thresh);
				//Exit if the network is faulty (contains an infinite loop)
				if (count>thresh) {
					cout<<"LOOP DETECTED DURING A RECURRENCY CHECK"<<endl;
					return false;
				}
				//Make sure it finds the right kind of link (recur or not)
				if (recurflag) trycount++;
				else {
					trycount=tries;
					found=true;
				}
			}
		} //End of normal link finding loop
	}

	//Continue only if an open link was found
	if (found) {
		//Check to see if this innovation already occured in the population
		theinnov=innovs.begin();
		//If it was supposed to be recurrent, make sure it gets labeled that way
		if (do_recur) recurflag=1;
		done=false;
		while(!done) {
			//The innovation is totally novel
			if (theinnov==innovs.end()) {
				//Useful for debugging
				//cout<<"nodep1 id: "<<nodep1->node_id<<endl;
				//cout<<"nodep1: "<<nodep1<<endl;
				//cout<<"nodep1 analogue: "<<nodep1->analogue<<endl;
				//cout<<"nodep2 id: "<<nodep2->node_id<<endl;
				//cout<<"nodep2: "<<nodep2<<endl;
				//cout<<"nodep2 analogue: "<<nodep2->analogue<<endl;
				//cout<<"recurflag: "<<recurflag<<endl;

				//NOTE: Something like this could be used for time delays,
				// which are not yet supported.  However, this does not
				// have an application with recurrency.
				//If not recurrent, randomize recurrency
				//if (!recurflag) 
				//  if (randfloat()<recur_prob) recurflag=1;
	
				//Choose a random trait
				traitnum=randint(0,(traits.size())-1);
				thetrait=traits.begin();
				//Choose the new weight
				//newweight=(gaussrand())/1.5;  //Could use a gaussian
				newweight=randposneg()*randfloat()*10.0;
				//Create the new gene
				newgene=new Gene(((thetrait[traitnum])),newweight,nodep1,nodep2,recurflag,curinnov,newweight);
				//Add the innovation
				innovs.push_back(new Innovation(nodep1->node_id,nodep2->node_id,curinnov,newweight,traitnum));
				curinnov=curinnov+1.0;
				done=true;
			}
			//OTHERWISE, match the innovation in the innovs list
			else if (((*theinnov)->innovation_type==NEWLINK)&&
				((*theinnov)->node_in_id==(nodep1->node_id))&&
				((*theinnov)->node_out_id==(nodep2->node_id))&&
				((*theinnov)->recur_flag==recurflag)) {
				thetrait=traits.begin();
				//Create new gene
				newgene=new Gene(((thetrait[(*theinnov)->new_traitnum])),(*theinnov)->new_weight,nodep1,nodep2,recurflag,(*theinnov)->innovation_num1,0);
				done=true;
			}
			else {
				//Keep looking for a matching innovation from this generation
				++theinnov;
			}
		}
		//Now add the new Genes to the Genome
		genes.push_back(newgene);
		return true;
	}
	else {
		return false;
	}
}	 

/* This method mates this Genome with another Genome g.
     For every point in each Genome, where each Genome shares
     the innovation number, the Gene is chosen randomly from
     either parent.  If one parent has an innovation absent in
     the other, the baby may inherit the innovation
     if it is from the more fit parent.
     The new Genome is given the id in the genomeid argument.
 */
Genome *Genome::mate_multipoint(Genome *g,int genomeid,double fitness1,double fitness2) {
	//The baby Genome will contain these new Traits, NNodes, and Genes
	vector<Trait*> newtraits; 
	list<NNode*> newnodes;   
	list<Gene*> newgenes;    
	Genome *new_genome;
	list<Gene*>::iterator curgene2;  //Checks for link duplication
	//iterators for moving through the two parents' traits
	vector<Trait*>::iterator p1trait;
	vector<Trait*>::iterator p2trait;
	Trait *newtrait;
	//iterators for moving through the two parents' genes
	list<Gene*>::iterator p1gene;
	list<Gene*>::iterator p2gene;
	double p1innov;  //Innovation numbers for genes inside parents' Genomes
	double p2innov;
	Gene *chosengene;  //Gene chosen for baby to inherit
	int traitnum;  //Number of trait new gene points to
	NNode *inode;  //NNodes connected to the chosen Gene
	NNode *onode;
	NNode *new_inode;
	NNode *new_onode;
	list<NNode*>::iterator curnode;  //For checking if NNodes exist already 
	int nodetraitnum;  //Trait number for a NNode
	bool disable;  //Set to true if we want to disabled a chosen gene
	disable=false;
	Gene *newgene;
	bool p1better; //Tells if the first genome (this one) has better fitness or not
	bool skip;

	//First, average the Traits from the 2 parents to form the baby's Traits
	//It is assumed that trait vectors are the same length
	//In the future, may decide on a different method for trait mating
	p2trait=(g->traits).begin();
	for(p1trait=traits.begin();p1trait!=traits.end();++p1trait) {
		newtrait=new Trait(*p1trait,*p2trait);  //Construct by averaging
		newtraits.push_back(newtrait);
		++p2trait;
	}
	//Figure out which genome is better
	//The worse genome should not be allowed to add extra structural baggage
	//If they are the same, use the smaller one's disjoint and excess genes only
	if (fitness1>fitness2) p1better=true;
	else if (fitness1==fitness2) {
		if (genes.size()<(g->genes.size())) p1better=true;
		else p1better=false;
	}
	else p1better=false;

	//Now move through the Genes of each parent until both genomes end
	p1gene=genes.begin();
	p2gene=(g->genes).begin();
	while(!((p1gene==genes.end())&&(p2gene==(g->genes).end()))) {
		skip=false;  //Default to not skipping a chosen gene
		if (p1gene==genes.end()) {
			chosengene=*p2gene;
			++p2gene;
			if (p1better) skip=true;  //Skip excess from the worse genome
		}
		else if (p2gene==(g->genes).end()) {
			chosengene=*p1gene;
			++p1gene;
			if (!p1better) skip=true; //Skip excess from the worse genome
		}
		else {
			//Extract current innovation numbers
			p1innov=(*p1gene)->innovation_num;
			p2innov=(*p2gene)->innovation_num;
			if (p1innov==p2innov) {
				if (randfloat()<0.5) {
				chosengene=*p1gene;
			}
			else {
				chosengene=*p2gene;
			}
			//If one is disabled, the corresponding gene in the offspring
			//will likely be disabled
			if ((((*p1gene)->enable)==false)||
				(((*p2gene)->enable)==false)) 
				if (randfloat()<0.75) disable=true;
			++p1gene;
			++p2gene;
		}
		else if (p1innov<p2innov) {
			chosengene=*p1gene;
			++p1gene;
			if (!p1better) skip=true;
		}
		else if (p2innov<p1innov) {
			chosengene=*p2gene;
			++p2gene;
			if (p1better) skip=true;
		}
	}
    
	//Uncomment this line to let growth go faster (from both parents excesses)
	//skip=false;
	//Check to see if the chosengene conflicts with an already chosen gene
	//i.e. do they represent the same link    
	curgene2=newgenes.begin();
	while ((curgene2!=newgenes.end())&&
			(!((((((*curgene2)->lnk)->in_node)->node_id)==((((chosengene)->lnk)->in_node)->node_id))&&
			(((((*curgene2)->lnk)->out_node)->node_id)==((((chosengene)->lnk)->out_node)->node_id))&&((((*curgene2)->lnk)->is_recurrent)== (((chosengene)->lnk)->is_recurrent)) ))&&
			(!((((((*curgene2)->lnk)->in_node)->node_id)==((((chosengene)->lnk)->out_node)->node_id))&&
			(((((*curgene2)->lnk)->out_node)->node_id)==((((chosengene)->lnk)->in_node)->node_id))&&
			(!((((*curgene2)->lnk)->is_recurrent)))&&
			(!((((chosengene)->lnk)->is_recurrent))))))	
		++curgene2;
	if (curgene2!=newgenes.end()) skip=true;  //Links conflicts, abort adding
	if (!skip) {
		//Now add the chosengene to the baby
		//First, get the trait pointer
		if ((((chosengene->lnk)->linktrait))==0) traitnum=(*(traits.begin()))->trait_id;
		else traitnum=(((chosengene->lnk)->linktrait)->trait_id)-(*(traits.begin()))->trait_id;  //The subtracted number normalizes depending on whether traits start counting at 1 or 0
		//Next check for the nodes, add them if not in the baby Genome already
		inode=(chosengene->lnk)->in_node;
		onode=(chosengene->lnk)->out_node;
		//Check for inode in the newnodes list
		if (inode->node_id<onode->node_id) {
			//inode before onode
			//Checking for inode's existence
			curnode=newnodes.begin();
			while((curnode!=newnodes.end())&&((*curnode)->node_id!=inode->node_id)) ++curnode;
			if (curnode==newnodes.end()) {
				//Here we know the node doesn't exist so we have to add it
				//(normalized trait number for new NNode)
				if (!(inode->nodetrait)) nodetraitnum=0;
				else nodetraitnum=((inode->nodetrait)->trait_id)-((*(traits.begin()))->trait_id);			       
				new_inode=new NNode(inode,newtraits[nodetraitnum]);
				node_insert(newnodes,new_inode);
			}
			else {
				new_inode=(*curnode);
			}
			//Checking for onode's existence
			curnode=newnodes.begin();
			while((curnode!=newnodes.end())&&((*curnode)->node_id!=onode->node_id)) ++curnode;
			if (curnode==newnodes.end()) {
				//Here we know the node doesn't exist so we have to add it
				//normalized trait number for new NNode
				if (!(onode->nodetrait)) nodetraitnum=0;
				else nodetraitnum=((onode->nodetrait)->trait_id)-(*(traits.begin()))->trait_id;			       
				new_onode=new NNode(onode,newtraits[nodetraitnum]);
				node_insert(newnodes,new_onode);
			}
			else {
				new_onode=(*curnode);
			}
		}
		//If the onode has a higher id than the inode we want to add it first
		else {
			//Checking for onode's existence
			curnode=newnodes.begin();
			while((curnode!=newnodes.end())&&((*curnode)->node_id!=onode->node_id)) ++curnode;
			if (curnode==newnodes.end()) {
				//Here we know the node doesn't exist so we have to add it
				//normalized trait number for new NNode
				if (!(onode->nodetrait)) nodetraitnum=0;
				else nodetraitnum=((onode->nodetrait)->trait_id)-(*(traits.begin()))->trait_id;			       
				new_onode=new NNode(onode,newtraits[nodetraitnum]);
				//newnodes.push_back(new_onode);
				node_insert(newnodes,new_onode);
			}
			else {
				new_onode=(*curnode);
			}

			//Checking for inode's existence
			curnode=newnodes.begin();
			while((curnode!=newnodes.end())&&((*curnode)->node_id!=inode->node_id)) ++curnode;
			if (curnode==newnodes.end()) {
				//Here we know the node doesn't exist so we have to add it
				//normalized trait number for new NNode
				if (!(inode->nodetrait)) nodetraitnum=0;  //fix
				else nodetraitnum=((inode->nodetrait)->trait_id)-(*(traits.begin()))->trait_id;		                 
				new_inode=new NNode(inode,newtraits[nodetraitnum]);
				node_insert(newnodes,new_inode);
			}
			else {
				new_inode=(*curnode);
			}
		} //End NNode checking section- NNodes are now in new Genome
		//cout<<"new_inode trait: "<<new_inode->nodetrait<<endl;
		//cout<<"new_onode trait: "<<new_onode->nodetrait<<endl;

		//Add the Gene
		newgene=new Gene(chosengene,newtraits[traitnum],new_inode,new_onode);
		if (disable) {
			newgene->enable=false;
			disable=false;
		}
		newgenes.push_back(newgene);
	}
	}
	new_genome=new Genome(genomeid,newtraits,newnodes,newgenes);
	//Return the baby Genome
	return (new_genome);
}

/* This is like mate_multipoint except it averages at each
   matching gene instead of choosing one or the other */
Genome *Genome::mate_multipoint_avg(Genome *g,int genomeid,double fitness1,double fitness2) {
	//The baby Genome will contain these new Traits, NNodes, and Genes
	vector<Trait*> newtraits; 
	list<NNode*> newnodes;   
	list<Gene*> newgenes;    
	//iterators for moving through the two parents' traits
	vector<Trait*>::iterator p1trait;
	vector<Trait*>::iterator p2trait;
	Trait *newtrait;
	list<Gene*>::iterator curgene2; //Checking for link duplication
	//iterators for moving through the two parents' genes
	list<Gene*>::iterator p1gene;
	list<Gene*>::iterator p2gene;
	double p1innov;  //Innovation numbers for genes inside parents' Genomes
	double p2innov;
	Gene *chosengene;  //Gene chosen for baby to inherit
	int traitnum;  //Number of trait new gene points to
	NNode *inode;  //NNodes connected to the chosen Gene
	NNode *onode;
	NNode *new_inode;
	NNode *new_onode;
	list<NNode*>::iterator curnode;  //For checking if NNodes exist already 
	int nodetraitnum;  //Trait number for a NNode
	//This Gene is used to hold the average of the two genes to be averaged
	Gene *avgene;
	Gene *newgene;
	bool skip;
	bool p1better;  //Designate the better genome

	/* BLX-alpha variables - for assigning weights within a good space */
	/* This is for BLX-style mating, which isn't used in this implementation,
		but can easily be made from multipoint_avg */
	//double blx_alpha;
	//double w1,w2;
	//double blx_min, blx_max;
	//double blx_range;   //The space range
	//double blx_explore;  //Exploration space on left or right
	//double blx_pos;  //Decide where to put gnes distancewise
	//blx_pos=randfloat();

	//First, average the Traits from the 2 parents to form the baby's Traits
	//It is assumed that trait vectors are the same length
	//In future, could be done differently
	p2trait=(g->traits).begin();
	for(p1trait=traits.begin();p1trait!=traits.end();++p1trait) {
		newtrait=new Trait(*p1trait,*p2trait);  //Construct by averaging
		newtraits.push_back(newtrait);
		++p2trait;
	}
	//Set up the avgene
	avgene=new Gene(0,0,0,0,0,0,0);
	//Figure out which genome is better
	//The worse genome should not be allowed to add extra structural baggage
	//If they are the same, use the smaller one's disjoint and excess genes only
	if (fitness1>fitness2) p1better=true;
	else if (fitness1==fitness2) {
		if (genes.size()<(g->genes.size())) p1better=true;
		else p1better=false;
	}
	else p1better=false;

	//Now move through the Genes of each parent until both genomes end
	p1gene=genes.begin();
	p2gene=(g->genes).begin();
	while(!((p1gene==genes.end())&&
		(p2gene==(g->genes).end()))) {
		avgene->enable=true;  //Default to enabled
		skip=false;
		if (p1gene==genes.end()) {
			chosengene=*p2gene;
			++p2gene;
			if (p1better) skip=true;
		}
		else if (p2gene==(g->genes).end()) {
			chosengene=*p1gene;
			++p1gene;
			if (!p1better) skip=true;
		}
		else {
			//Extract current innovation numbers
			p1innov=(*p1gene)->innovation_num;
			p2innov=(*p2gene)->innovation_num;
			if (p1innov==p2innov) {
				//Average them into the avgene
				if (randfloat()>0.5) (avgene->lnk)->linktrait=((*p1gene)->lnk)->linktrait;
				else (avgene->lnk)->linktrait=((*p2gene)->lnk)->linktrait;
				//WEIGHTS AVERAGED HERE
				(avgene->lnk)->weight=(((*p1gene)->lnk)->weight+((*p2gene)->lnk)->weight)/2.0;

				/*
				//BLX-alpha method (Eschelman et al 1993)
				//Not used in this implementation, but the commented code works
				//with alpha=0.5, this will produce babies evenly in exploitation and exploration space
				//and uniformly distributed throughout
				blx_alpha=-0.4;
				w1=(((*p1gene)->lnk)->weight);
				w2=(((*p2gene)->lnk)->weight);
				if (w1>w2) {
					blx_max=w1; blx_min=w2;
				}
				else {
					blx_max=w2; blx_min=w1;
				}
				blx_range=blx_max-blx_min;
				blx_explore=blx_alpha*blx_range;
				//Now extend the range into the exploraton space
				blx_min-=blx_explore;
				blx_max+=blx_explore;
				blx_range=blx_max-blx_min;
				//Set the weight in the new range
				(avgene->lnk)->weight=blx_min+blx_pos*blx_range;
				*/

				if (randfloat()>0.5) (avgene->lnk)->in_node=((*p1gene)->lnk)->in_node;
				else (avgene->lnk)->in_node=((*p2gene)->lnk)->in_node;
				if (randfloat()>0.5) (avgene->lnk)->out_node=((*p1gene)->lnk)->out_node;
				else (avgene->lnk)->out_node=((*p2gene)->lnk)->out_node;
				if (randfloat()>0.5) (avgene->lnk)->is_recurrent=((*p1gene)->lnk)->is_recurrent;
				else (avgene->lnk)->is_recurrent=((*p2gene)->lnk)->is_recurrent;
				avgene->innovation_num=(*p1gene)->innovation_num;
				avgene->mutation_num=((*p1gene)->mutation_num+(*p2gene)->mutation_num)/2.0;
				if ((((*p1gene)->enable)==false)||
					(((*p2gene)->enable)==false)) 
					if (randfloat()<0.75) avgene->enable=false;
				chosengene=avgene;
				++p1gene;
				++p2gene;
			}
			else if (p1innov<p2innov) {
				chosengene=*p1gene;
				++p1gene;
				if (!p1better) skip=true;
			}
			else if (p2innov<p1innov) {
				chosengene=*p2gene;
				++p2gene;
				if (p1better) skip=true;
			}
		}

		//THIS LINE MUST BE DELETED TO SLOW GROWTH
		//skip=false;
		//Check to see if the chosengene conflicts with an already chosen gene
		//i.e. do they represent the same link    
		curgene2=newgenes.begin();
		while ((curgene2!=newgenes.end())) {
			if (((((((*curgene2)->lnk)->in_node)->node_id)==((((chosengene)->lnk)->in_node)->node_id))&&
				(((((*curgene2)->lnk)->out_node)->node_id)==((((chosengene)->lnk)->out_node)->node_id))&&
				((((*curgene2)->lnk)->is_recurrent)== (((chosengene)->lnk)->is_recurrent)))||
				((((((*curgene2)->lnk)->out_node)->node_id)==((((chosengene)->lnk)->in_node)->node_id))&&
				(((((*curgene2)->lnk)->in_node)->node_id)==((((chosengene)->lnk)->out_node)->node_id))&&
				(!((((*curgene2)->lnk)->is_recurrent)))&&
				(!((((chosengene)->lnk)->is_recurrent))))) { 
				skip=true;
			}
			++curgene2;
		}
		if (!skip) {
			//Now add the chosengene to the baby
			//First, get the trait pointer
			if ((((chosengene->lnk)->linktrait))==0) traitnum=(*(traits.begin()))->trait_id;
			else traitnum=(((chosengene->lnk)->linktrait)->trait_id)-(*(traits.begin()))->trait_id;  //The subtracted number normalizes depending on whether traits start counting at 1 or 0
			//Next check for the nodes, add them if not in the baby Genome already
			inode=(chosengene->lnk)->in_node;
			onode=(chosengene->lnk)->out_node;
			//Check for inode in the newnodes list
			if (inode->node_id<onode->node_id) {
				//Checking for inode's existence
				curnode=newnodes.begin();
				while((curnode!=newnodes.end())&&
					((*curnode)->node_id!=inode->node_id)) 
					++curnode;
				if (curnode==newnodes.end()) {
				//Here we know the node doesn't exist so we have to add it
				//normalized trait number for new NNode
				if (!(inode->nodetrait)) nodetraitnum=0;
				else nodetraitnum=((inode->nodetrait)->trait_id)-((*(traits.begin()))->trait_id);			       
				new_inode=new NNode(inode,newtraits[nodetraitnum]);
				node_insert(newnodes,new_inode);
			}
			else {
				new_inode=(*curnode);
			}

			//Checking for onode's existence
			curnode=newnodes.begin();
			while((curnode!=newnodes.end())&&
				((*curnode)->node_id!=onode->node_id)) 
				++curnode;
			if (curnode==newnodes.end()) {
				//Here we know the node doesn't exist so we have to add it
				//normalized trait number for new NNode
				if (!(onode->nodetrait)) nodetraitnum=0;
				else nodetraitnum=((onode->nodetrait)->trait_id)-(*(traits.begin()))->trait_id;			
				new_onode=new NNode(onode,newtraits[nodetraitnum]);
				node_insert(newnodes,new_onode);
			}
			else {
				new_onode=(*curnode);
			}
		}
		//If the onode has a higher id than the inode we want to add it first
		else {
			//Checking for onode's existence
			curnode=newnodes.begin();
			while((curnode!=newnodes.end())&&
				((*curnode)->node_id!=onode->node_id)) 
				++curnode;
			if (curnode==newnodes.end()) {
				//Here we know the node doesn't exist so we have to add it
				//normalized trait number for new NNode
				if (!(onode->nodetrait)) nodetraitnum=0;
				else nodetraitnum=((onode->nodetrait)->trait_id)-(*(traits.begin()))->trait_id;			       
				new_onode=new NNode(onode,newtraits[nodetraitnum]);
				node_insert(newnodes,new_onode);
			}
			else {
				new_onode=(*curnode);
			}
			//Checking for inode's existence
			curnode=newnodes.begin();
			while((curnode!=newnodes.end())&&((*curnode)->node_id!=inode->node_id)) ++curnode;
			if (curnode==newnodes.end()) {
				//Here we know the node doesn't exist so we have to add it
				//normalized trait number for new NNode
				if (!(inode->nodetrait)) nodetraitnum=0;
				else nodetraitnum=((inode->nodetrait)->trait_id)-(*(traits.begin()))->trait_id;			       
				new_inode=new NNode(inode,newtraits[nodetraitnum]);
				node_insert(newnodes,new_inode);
			}
			else {
				new_inode=(*curnode);
			}
		} //End NNode checking section- NNodes are now in new Genome
    
		//Add the Gene
		newgene=new Gene(chosengene,newtraits[traitnum],new_inode,new_onode);
		newgenes.push_back(newgene);
		}  //End if which checked for link duplicationb
	}
	delete avgene;  //Clean up used object
	//Return the baby Genome
	return (new Genome(genomeid,newtraits,newnodes,newgenes));
}

/* This method is similar to a standard single point crossover
     operator.  Traits are averaged as in the previous 2 mating
     methods.  A Gene is chosen in the small Genome for splitting.
     When the Gene is reached, it is averaged with the matching
     Gene from the larger Genome, if one exists.  Then every other
     Gene is taken from the larger Genome  */
Genome *Genome::mate_singlepoint(Genome *g,int genomeid) {
	//The baby Genome will contain these new Traits, NNodes, and Genes
	vector<Trait*> newtraits; 
	list<NNode*> newnodes;   
	list<Gene*> newgenes;    
	//iterators for moving through the two parents' traits
	vector<Trait*>::iterator p1trait;
	vector<Trait*>::iterator p2trait;
	Trait *newtrait;
	list<Gene*>::iterator curgene2;  //Check for link duplication
	//iterators for moving through the two parents' genes
	list<Gene*>::iterator p1gene;
	list<Gene*>::iterator p2gene;
	list<Gene*>::iterator stopper;  //To tell when finished
	list<Gene*>::iterator p2stop;
	list<Gene*>::iterator p1stop;
	double p1innov;  //Innovation numbers for genes inside parents' Genomes
	double p2innov;
	Gene *chosengene;  //Gene chosen for baby to inherit
	int traitnum;  //Number of trait new gene points to
	NNode *inode;  //NNodes connected to the chosen Gene
	NNode *onode;
	NNode *new_inode;
	NNode *new_onode;
	list<NNode*>::iterator curnode;  //For checking if NNodes exist already 
	int nodetraitnum;  //Trait number for a NNode
	//This Gene is used to hold the average of the two genes to be averaged
	Gene *avgene;
	int crosspoint; //The point in the Genome to cross at
	int genecounter; //Counts up to the crosspoint
	bool skip; //Used for skipping unwanted genes

	//First, average the Traits from the 2 parents to form the baby's Traits
	//It is assumed that trait vectors are the same length
	p2trait=(g->traits).begin();
	for(p1trait=traits.begin();p1trait!=traits.end();++p1trait) {
		newtrait=new Trait(*p1trait,*p2trait);  //Construct by averaging
		newtraits.push_back(newtrait);
		++p2trait;
	}
	//Set up the avgene
	avgene=new Gene(0,0,0,0,0,0,0);
	//Decide where to cross  (p1gene will always be in smaller Genome)
	if (genes.size()<(g->genes).size()) {
		crosspoint=randint(0,(genes.size())-1);
		p1gene=genes.begin();
		p2gene=(g->genes).begin();
		stopper=(g->genes).end();
		p1stop=genes.end();
		p2stop=(g->genes).end();
	}
	else {
		crosspoint=randint(0,((g->genes).size())-1);
		p2gene=genes.begin();
		p1gene=(g->genes).begin();
		stopper=genes.end();
		p1stop=(g->genes.end());
		p2stop=genes.end();
	}
	genecounter=0;  //Ready to count to crosspoint
	skip=false;  //Default to not skip a Gene
	//Note that we skip when we are on the wrong Genome before
	//crossing

	//Now move through the Genes of each parent until both genomes end
	while(p2gene!=stopper) {
		avgene->enable=true;  //Default to true
		if (p1gene==p1stop) {
			chosengene=*p2gene;
			++p2gene;
		}
		else if (p2gene==p2stop) {
			chosengene=*p1gene;
			++p1gene;
		}
		else {
			//Extract current innovation numbers
			if (p1gene==g->genes.end()) cout<<"WARNING p1"<<endl;
			if (p2gene==g->genes.end()) cout<<"WARNING p2"<<endl;
			p1innov=(*p1gene)->innovation_num;
			p2innov=(*p2gene)->innovation_num;
			if (p1innov==p2innov) {
				//Pick the chosengene depending on whether we've crossed yet
				if (genecounter<crosspoint) {
					chosengene=*p1gene;
				}
				else if (genecounter>crosspoint) {
					chosengene=*p2gene;
				}
				//We are at the crosspoint here
				else {
					//Average them into the avgene
					if (randfloat()>0.5) (avgene->lnk)->linktrait=((*p1gene)->lnk)->linktrait;
					else (avgene->lnk)->linktrait=((*p2gene)->lnk)->linktrait;
					//WEIGHTS AVERAGED HERE
					(avgene->lnk)->weight=(((*p1gene)->lnk)->weight+((*p2gene)->lnk)->weight)/2.0;
					if (randfloat()>0.5) (avgene->lnk)->in_node=((*p1gene)->lnk)->in_node;
					else (avgene->lnk)->in_node=((*p2gene)->lnk)->in_node;
					if (randfloat()>0.5) (avgene->lnk)->out_node=((*p1gene)->lnk)->out_node;
					else (avgene->lnk)->out_node=((*p2gene)->lnk)->out_node;
					if (randfloat()>0.5) (avgene->lnk)->is_recurrent=((*p1gene)->lnk)->is_recurrent;
					else (avgene->lnk)->is_recurrent=((*p2gene)->lnk)->is_recurrent;
					avgene->innovation_num=(*p1gene)->innovation_num;
					avgene->mutation_num=((*p1gene)->mutation_num+(*p2gene)->mutation_num)/2.0;
					if ((((*p1gene)->enable)==false)||
						(((*p2gene)->enable)==false)) 
						avgene->enable=false;
					chosengene=avgene;
				}
				++p1gene;
				++p2gene;
				++genecounter;
			}
			else if (p1innov<p2innov) {
				if (genecounter<crosspoint) {
					chosengene=*p1gene;
					++p1gene;
					++genecounter;
				}
				else {
					chosengene=*p2gene;
					++p2gene;
				}
			}
			else if (p2innov<p1innov) {
				++p2gene;
				skip=true;  //Special case: we need to skip to the next iteration
				//because this Gene is before the crosspoint on the wrong Genome
			}
		}
    
		//Check to see if the chosengene conflicts with an already chosen gene
		//i.e. do they represent the same link    
		curgene2=newgenes.begin();
		while ((curgene2!=newgenes.end())&&
			(!((((((*curgene2)->lnk)->in_node)->node_id)==((((chosengene)->lnk)->in_node)->node_id))&&
			(((((*curgene2)->lnk)->out_node)->node_id)==((((chosengene)->lnk)->out_node)->node_id))&&((((*curgene2)->lnk)->is_recurrent)== (((chosengene)->lnk)->is_recurrent)) ))&&
			(!((((((*curgene2)->lnk)->in_node)->node_id)==((((chosengene)->lnk)->out_node)->node_id))&&
			(((((*curgene2)->lnk)->out_node)->node_id)==((((chosengene)->lnk)->in_node)->node_id))&&
			(!((((*curgene2)->lnk)->is_recurrent)))&&
			(!((((chosengene)->lnk)->is_recurrent)))))) {
			++curgene2;
		}
		if (curgene2!=newgenes.end()) skip=true;  //Link is a duplicate
		if (!skip) {
			//Now add the chosengene to the baby
			//First, get the trait pointer
			if ((((chosengene->lnk)->linktrait))==0) traitnum=(*(traits.begin()))->trait_id;
			else traitnum=(((chosengene->lnk)->linktrait)->trait_id)-(*(traits.begin()))->trait_id;  //The subtracted number normalizes depending on whether traits start counting at 1 or 0
			//Next check for the nodes, add them if not in the baby Genome already
			inode=(chosengene->lnk)->in_node;
			onode=(chosengene->lnk)->out_node;
			//Check for inode in the newnodes list
			if (inode->node_id<onode->node_id) {
				//cout<<"inode before onode"<<endl;
				//Checking for inode's existence
				curnode=newnodes.begin();
				while((curnode!=newnodes.end())&&((*curnode)->node_id!=inode->node_id)) ++curnode;
				if (curnode==newnodes.end()) {
					//Here we know the node doesn't exist so we have to add it
					//normalized trait number for new NNode
					if (!(inode->nodetrait)) nodetraitnum=0;
					else nodetraitnum=((inode->nodetrait)->trait_id)-((*(traits.begin()))->trait_id);			       
					new_inode=new NNode(inode,newtraits[nodetraitnum]);
					node_insert(newnodes,new_inode);
				}
				else {
					new_inode=(*curnode);
				}
				//Checking for onode's existence
				curnode=newnodes.begin();
				while((curnode!=newnodes.end())&&((*curnode)->node_id!=onode->node_id)) ++curnode;
				if (curnode==newnodes.end()) {
					//Here we know the node doesn't exist so we have to add it
					//normalized trait number for new NNode
					if (!(onode->nodetrait)) nodetraitnum=0;
					else nodetraitnum=((onode->nodetrait)->trait_id)-(*(traits.begin()))->trait_id;			     
					new_onode=new NNode(onode,newtraits[nodetraitnum]);
					node_insert(newnodes,new_onode);
				}
				else {
					new_onode=(*curnode);
				}
			}
			//If the onode has a higher id than the inode we want to add it first
			else {
				//Checking for onode's existence
				curnode=newnodes.begin();
				while((curnode!=newnodes.end())&&((*curnode)->node_id!=onode->node_id)) ++curnode;
				if (curnode==newnodes.end()) {
					//Here we know the node doesn't exist so we have to add it
					//normalized trait number for new NNode
					if (!(onode->nodetrait)) nodetraitnum=0;
					else nodetraitnum=((onode->nodetrait)->trait_id)-(*(traits.begin()))->trait_id;			       
					new_onode=new NNode(onode,newtraits[nodetraitnum]);
					node_insert(newnodes,new_onode);
				}
				else {
					new_onode=(*curnode);
				}

				//Checking for inode's existence
				curnode=newnodes.begin();
				while((curnode!=newnodes.end())&&((*curnode)->node_id!=inode->node_id)) ++curnode;
				if (curnode==newnodes.end()) {
				//Here we know the node doesn't exist so we have to add it
				//normalized trait number for new NNode
				if (!(inode->nodetrait)) nodetraitnum=0;
				else nodetraitnum=((inode->nodetrait)->trait_id)-(*(traits.begin()))->trait_id;			       
				new_inode=new NNode(inode,newtraits[nodetraitnum]);
				//newnodes.push_back(new_inode);
				node_insert(newnodes,new_inode);
			}
			else {
				new_inode=(*curnode);
			}
		} //End NNode checking section- NNodes are now in new Genome
		//Add the Gene
		newgenes.push_back(new Gene(chosengene,newtraits[traitnum],new_inode,new_onode));
	}  //End of if (!skip)
	skip=false;
	}
	delete avgene;  //Clean up used object
	//Return the baby Genome
	return (new Genome(genomeid,newtraits,newnodes,newgenes));
}

/* This function gives a measure of compatibility between
     two Genomes by computing a linear combination of 3
     characterizing variables of their compatibilty.
     The 3 variables represent PERCENT DISJOINT GENES,
     PERCENT EXCESS GENES, MUTATIONAL DIFFERENCE WITHIN
     MATCHING GENES.  So the formula for compatibility
     is:  disjoint_coeff*pdg+excess_coeff*peg+mutdiff_coeff*mdmg.
     The 3 coefficients are global system parameters */
double Genome::compatibility(Genome *g) {
	//iterators for moving through the two potential parents' Genes
	list<Gene*>::iterator p1gene;
	list<Gene*>::iterator p2gene;  
	//Innovation numbers
	double p1innov;
	double p2innov;
	//Intermediate value
	double mut_diff;
	//Set up the counters
	double num_disjoint=0.0;
	double num_excess=0.0;
	double mut_diff_total=0.0;
	double num_matching=0.0;  //Used to normalize mutation_num differences
	double max_genome_size; //Size of larger Genome
	//Get the length of the longest Genome for percentage computations
	if (genes.size()<(g->genes).size()) max_genome_size=(g->genes).size();
	else max_genome_size=genes.size();

	//Now move through the Genes of each potential parent 
	//until both Genomes end
	p1gene=genes.begin();
	p2gene=(g->genes).begin();
	while(!((p1gene==genes.end())&&
		(p2gene==(g->genes).end()))) {
		if (p1gene==genes.end()) {
			++p2gene;
			num_excess+=1.0;
		}
		else if (p2gene==(g->genes).end()) {
			++p1gene;
			num_excess+=1.0;
		}
		else {
			//Extract current innovation numbers
			p1innov=(*p1gene)->innovation_num;
			p2innov=(*p2gene)->innovation_num;
			if (p1innov==p2innov) {
				num_matching+=1.0;
				mut_diff=((*p1gene)->mutation_num)-((*p2gene)->mutation_num);
				if (mut_diff<0.0) mut_diff=0.0-mut_diff;
				mut_diff_total+=mut_diff;
				++p1gene;
				++p2gene;
			}
			else if (p1innov<p2innov) {
				++p1gene;
				num_disjoint+=1.0;
			}
			else if (p2innov<p1innov) {
				++p2gene;
				num_disjoint+=1.0;
			}
		}
	} //End while

	//Return the compatibility number using compatibility formula
	//Note that mut_diff_total/num_matching gives the AVERAGE
	//difference between mutation_nums for any two matching Genes
	//in the Genome
	/*
	//Normalizing for genome size
	return (disjoint_coeff*(num_disjoint/max_genome_size)+
		excess_coeff*(num_excess/max_genome_size)+
		mutdiff_coeff*(mut_diff_total/num_matching));
	*/

	//Look at disjointedness and excess in the absolute (ignoring size)
	return (disjoint_coeff*(num_disjoint/1.0)+
		excess_coeff*(num_excess/1.0)+
		mutdiff_coeff*(mut_diff_total/num_matching));
}

/* Return # of non-disabled genes */
int Genome::extrons() {
	list<Gene*>::iterator curgene;
	int total=0;
	for(curgene=genes.begin();curgene!=genes.end();curgene++) {
		if ((*curgene)->enable) ++total;
	}
	return total;
}

//Print the genome to a file
void Genome::print_to_file(ofstream &outFile) {
	vector<Trait*>::iterator curtrait;
	list<NNode*>::iterator curnode;
	list<Gene*>::iterator curgene;
	outFile<<"genomestart "<<genome_id<<endl;
	//Output the traits
	for(curtrait=traits.begin();curtrait!=traits.end();++curtrait) {
		(*curtrait)->trait_id=curtrait-traits.begin()+1;
		(*curtrait)->print_to_file(outFile);
	}
	//Output the nodes
	for(curnode=nodes.begin();curnode!=nodes.end();++curnode) {
		(*curnode)->print_to_file(outFile);
	}
	//Output the genes
	for(curgene=genes.begin();curgene!=genes.end();++curgene) {
		(*curgene)->print_to_file(outFile);
	}
	outFile<<"genomeend "<<genome_id<<endl;
}

void Genome::print_to_filename(char *filename) {
	ofstream oFile(filename,ios::out);
	print_to_file(oFile);
	oFile.close();
}

//Non-operator printing of genome to screen
void Genome::print_genome() {
	list<NNode*>::iterator curnode;
	list<Gene*>::iterator curgene;
	vector<Trait*>::iterator curtrait;
	Genome *thegenome=this;
	list<NNode*> nodes=thegenome->nodes;
	list<Gene*> genes=thegenome->genes;
	vector<Trait*> traits=thegenome->traits;
	cout<<"GENOME START"<<endl;
	for(curnode=nodes.begin();curnode!=nodes.end();++curnode) {
		if (((*curnode)->gen_node_label)==INPUT) cout<<"I";
		else if (((*curnode)->gen_node_label)==OUTPUT) cout<<"O";
		else if (((*curnode)->gen_node_label)==HIDDEN) cout<<"H";
		else if (((*curnode)->gen_node_label)==BIAS) cout<<"B";
		cout<<(*curnode)<<" ";
	}
	cout<<endl;
	for(curgene=genes.begin();curgene!=genes.end();++curgene) {
		cout<<(*curgene)<<" ";
	}
	cout<<endl;
	cout<<"Traits: ";
	for(curtrait=traits.begin();curtrait!=traits.end();++curtrait) {
		cout<<(*curtrait)<<" ";
	}
	cout<<endl;
	cout<<"GENOME END"<<endl;
}

//Genome printing method
ostream& operator<< (ostream& os, const Genome *thegenome) {
	list<NNode*>::iterator curnode;
	list<Gene*>::iterator curgene;
	vector<Trait*>::iterator curtrait;
	list<NNode*> nodes=thegenome->nodes;
	list<Gene*> genes=thegenome->genes;
	vector<Trait*> traits=thegenome->traits;
	cout<<"GENOME START"<<endl;
	for(curnode=nodes.begin();curnode!=nodes.end();++curnode) {
		if (((*curnode)->gen_node_label)==INPUT) cout<<"I";
		else if (((*curnode)->gen_node_label)==OUTPUT) cout<<"O";
		else if (((*curnode)->gen_node_label)==HIDDEN) cout<<"H";
		else if (((*curnode)->gen_node_label)==BIAS) cout<<"B";
		cout<<(*curnode)<<" ";
	}
	cout<<endl;
	for(curgene=genes.begin();curgene!=genes.end();++curgene) {
		cout<<(*curgene)<<" ";
	}
	cout<<endl;
	cout<<"Traits: ";
	for(curtrait=traits.begin();curtrait!=traits.end();++curtrait) {
		cout<<(*curtrait)<<" ";
	}
	cout<<endl;
	cout<<"GENOME END"<<endl;
	return os;
}
