#ifndef _GeneticOrganism
#define _GeneticOrganism

#include "NetworksNetwork.h"
#include "GeneticGenome.h"
#include "Neat.h"

/* ---------------------------------------------  */
/* ORGANISM CLASS:
   Organisms are Genomes and Networks with fitness
   information 
   i.e. The genotype and phenotype together */
/* ---------------------------------------------  */

class Organism : public NEAT {

public:
  
	double fitness;  //A measure of fitness for the Organism
	double orig_fitness;  //A fitness measure that won't change during adjustments
	double error;  //Used just for reporting purposes
	bool winner;  //Win marker (if needed for a particular task)
	Network *net;  //The Organism's phenotype
	Genome *gnome; //The Organism's genotype 
	double expected_offspring; //Number of children this Organism may have
	int generation;  //Tells which generation this Organism is from
	bool eliminate;  //Marker for destruction of inferior Organisms
	bool champion; //Marks the species champ
	int super_champ_offspring;  //Number of reserved offspring for a population leader
	bool pop_champ;  //Marks the best in population
	bool pop_champ_child; //Marks the duplicate child of a champion (for tracking purposes)
	double high_fit; //DEBUG variable- high fitness of champ

	void update_phenotype(); /* Regenerate the network based on a change in the genotype */

	//Track its origin- for debugging or analysis- we can tell how the organism was born
	bool mut_struct_baby;
	bool mate_baby;

	Organism(double fit, Genome *g,int gen) {
		fitness=fit;
		orig_fitness=fitness;
		gnome=g;
		net=net->genesis(gnome->genome_id,gnome);
		expected_offspring=0;
		generation=gen;
		eliminate=false;
		error=0;
		winner=false;
		champion=false;
		super_champ_offspring=0;
		//DEBUG vars
		pop_champ=false;
		pop_champ_child=false;
		high_fit=0;
		mut_struct_baby=0;
		mate_baby=0;
	}

	~Organism() {
		delete net;
		delete gnome;
	}

	bool operator<(Organism *x);
	
};

#endif
