#include "GeneticPopulation.h" 

//Create a population of size size off of Genome g
//The new Population will have the same topology as g
//with linkweights slightly perturbed from g's
bool Population::spawn(Genome *g,int size) {
	int count;
	Genome *new_genome;
	Organism *new_organism;
	//Create size copies of the Genome
	//Start with perturbed linkweights
	for(count=1;count<=size;count++) {
		//cout<<"CREATING ORGANISM "<<count<<endl;
		new_genome=g->duplicate(count); 
		//new_genome->mutate_link_weights(10.0,1.0,GAUSSIAN);
		new_genome->mutate_link_weights(1.0,1.0,GAUSSIAN);
		new_organism=new Organism(0.0,new_genome,1);
		organisms.push_back(new_organism);
	}
	//Keep a record of the innovation and node number we are on
	cur_node_id=new_genome->get_last_node_id();
	cur_innov_num=new_genome->get_last_gene_innovnum();
	//Separate the new Population into species
	speciate();
	return true;
}

//speciate separates the organisms into species by
//checking compatibilities against a threshold
//Any organism that is not compatible with the first
//organism in any existing species becomes a new species
bool Population::speciate() {
	list<Organism*>::iterator curorg;  //For stepping through Population
	list<Species*>::iterator curspecies; //Steps through species
	Organism *comporg=0;  //Organism for comparison 
	Species *newspecies; //For adding a new species
	int counter=0; //Species counter
	//Step through all existing organisms
	for(curorg=organisms.begin();curorg!=organisms.end();++curorg) {
		//For each organism, search for a species it is compatible to
		curspecies=species.begin();
		if (curspecies==species.end()){
			//Create the first species
			newspecies=new Species(++counter);
			species.push_back(newspecies);
			newspecies->add_Organism(*curorg);  //Add the current organism
			// (*curorg)->species=newspecies;  //Point organism to its species
			/* --> faire plutot un tableau associatif organism/species ***/
		}
		else {
			comporg=(*curspecies)->first();
			while((comporg!=0)&&
				(curspecies!=species.end())) {
				if ((((*curorg)->gnome)->compatibility(comporg->gnome))<compat_threshold) {
					//Found compatible species, so add this organism to it
					(*curspecies)->add_Organism(*curorg);
					//(*curorg)->species=(*curspecies);  //Point organism to its species
					comporg=0;  //Note the search is over
				}
				else {
					//Keep searching for a matching species
					++curspecies;
					if (curspecies!=species.end()) 
						comporg=(*curspecies)->first();
				}
			}
			//If we didn't find a match, create a new species
			if (comporg!=0) {
				newspecies=new Species(++counter);
				species.push_back(newspecies);
				newspecies->add_Organism(*curorg);  //Add the current organism
				// (*curorg)->species=newspecies;  //Point organism to its species
			}
		} //end else 
	} //end for
	last_species=counter;  //Keep track of highest species
	return true;
}

//epoch turns over a Population to the next generation based on fitness
bool Population::epoch(int generation) {
	list<Species*>::iterator curspecies;
	list<Species*>::iterator deadspecies;  //For removing empty Species
	list<Organism*>::iterator curorg;
	list<Organism*>::iterator deadorg;
	list<Innovation*>::iterator curinnov;  
	list<Innovation*>::iterator deadinnov;  //For removing old Innovs
	double total=0.0; //Used to compute average fitness over all Organisms
	double overall_average;  //The average modified fitness among ALL organisms
	int orgcount;
	//The fractional parts of expected offspring that can be 
	//Used only when they accumulate above 1 for the purposes of counting
	//Offspring
	double skim; 
	int total_expected;  //precision checking
	int total_organisms=organisms.size();
	int max_expected;
	Species *best_species;
	int final_expected;
	//Rights to make babies can be stolen from inferior species
	//and given to their superiors, in order to concentrate exploration on
	//the best species
	int NUM_STOLEN=babies_stolen; //Number of babies to steal
	int one_fifth_stolen;
	int one_tenth_stolen;
	list<Species*> sorted_species;  //Species sorted by max fit org in Species
	int stolen_babies; //Babies taken from the bad species and given to the champs
	int half_pop;
	int best_species_num;  //Used in debugging to see why (if) best species dies
	bool best_ok;
	//We can try to keep the number of species constant at this number
	//  int num_species_target=100;
	int num_species=species.size();
	//  double compat_mod=0.3;  //Modify compat thresh to control speciation
	//Record what generation we are on
	final_gen=generation;

	/*
	//Keeping species diverse
	//This commented out code forces the system to aim for 
	// num_species species at all times, enforcing diversity
	//This tinkers with the compatibility threshold, which
	// normally would be held constant
	if (generation>1) {
		if (num_species<num_species_target)
			compat_threshold-=compat_mod;
		else if (num_species>num_species_target)
			compat_threshold+=compat_mod;
		if (compat_threshold<0.3) compat_threshold=0.3;
	}
	*/

	cout<<"Number of Species: "<<num_species<<endl;
	cout<<"compat_thresh: "<<compat_threshold<<endl;
	//Use Species' ages to modify the objective fitness of organisms
	// in other words, make it more fair for younger species
	// so they have a chance to take hold
	//Also penalize stagnant species
	//Then adjust the fitness using the species size to "share" fitness
	//within a species.
	//Then, within each Species, mark for death 
	//those below survival_thresh*average
	for(curspecies=species.begin();curspecies!=species.end();++curspecies) {
		(*curspecies)->adjust_fitness();
	}
	//Go through the organisms and add up their fitnesses to compute the
	//overall average
	for(curorg=organisms.begin();curorg!=organisms.end();++curorg) {
		total+=(*curorg)->fitness;
	}
	overall_average=total/total_organisms;
	cout<<"Generation "<<generation<<": "<<"overall_average = "<<overall_average<<endl;
	//Now compute expected number of offspring for each individual organism
	for(curorg=organisms.begin();curorg!=organisms.end();++curorg) {
		(*curorg)->expected_offspring=(((*curorg)->fitness)/overall_average);
	}
	//Now add those offspring up within each Species to get the number of
	//offspring per Species
	skim=0.0;
	total_expected=0;
	for(curspecies=species.begin();curspecies!=species.end();++curspecies) {
		skim=(*curspecies)->count_offspring(skim);
		total_expected+=(*curspecies)->expected_offspring;
	}    

	//Need to make up for lost foating point precision in offspring assignment
	//If we lost precision, give an extra baby to the best Species
	if (total_expected<total_organisms) {
		//Find the Species expecting the most
		max_expected=0;
		final_expected=0;
		for(curspecies=species.begin();curspecies!=species.end();++curspecies) {
			if ((*curspecies)->expected_offspring>=max_expected) {
				max_expected=(*curspecies)->expected_offspring;
				best_species=(*curspecies);
			}
			final_expected+=(*curspecies)->expected_offspring;
		}
		//Give the extra offspring to the best species
		++(best_species->expected_offspring);
		final_expected++;
		//If we still arent at total, there is a problem
		//Note that this can happen if a stagnant Species
		//dominates the population and then gets killed off by its age
		//Then the whole population plummets in fitness
		//If the average fitness is allowed to hit 0, then we no longer have 
		//an average we can use to assign offspring.
		if (final_expected<total_organisms) {
			cout<<"Population died!"<<endl;
			//cin>>pause;
			for(curspecies=species.begin();curspecies!=species.end();++curspecies) {
				(*curspecies)->expected_offspring=0;
			}
			best_species->expected_offspring=total_organisms;
		}
	}
  
	//Stick the Species pointers into a new Species list for sorting
	for(curspecies=species.begin();curspecies!=species.end();++curspecies) {
		sorted_species.push_back(*curspecies);
	}
	//Sort the Species by max fitness (Use an extra list to do this)
	//These need to use ORIGINAL fitness
	sorted_species.sort();
	best_species_num=(*(sorted_species.begin()))->id;
	for(curspecies=sorted_species.begin();curspecies!=sorted_species.end();++curspecies) {
		//Print out for Debugging/viewing what's going on 
		cout<<"orig fitness of Species"<<(*curspecies)->id<<"(Size "<<(*curspecies)->organisms.size()<<"): "<<(*((*curspecies)->organisms).begin())->orig_fitness<<" last improved "<<((*curspecies)->age-(*curspecies)->age_of_last_improvement)<<endl;
	}
	//Check for Population-level stagnation
	curspecies=sorted_species.begin();
	(*(((*curspecies)->organisms).begin()))->pop_champ=true; //DEBUG marker of the best of pop
	if (((*(((*curspecies)->organisms).begin()))->orig_fitness)>highest_fitness) {
		highest_fitness=((*(((*curspecies)->organisms).begin()))->orig_fitness);
		highest_last_changed=0;
		cout<<"NEW POPULATION RECORD FITNESS: "<<highest_fitness<<endl;
	}
	else {
		++highest_last_changed;
		cout<<highest_last_changed<<" generations since last population fitness record: "<<highest_fitness<<endl;
	}
	//Check for stagnation- if there is stagnation, perform delta-coding
	if (highest_last_changed>=dropoff_age+5) {
		cout<<"PERFORMING DELTA CODING"<<endl;
		highest_last_changed=0;
		half_pop=pop_size/2;
		cout<<"half_pop"<<half_pop<<" pop_size-halfpop: "<<pop_size-half_pop<<endl;
		curspecies=sorted_species.begin();
		(*(((*curspecies)->organisms).begin()))->super_champ_offspring=half_pop;
		(*curspecies)->expected_offspring=half_pop;
		(*curspecies)->age_of_last_improvement=(*curspecies)->age;
		++curspecies;
		if (curspecies!=sorted_species.end()) {
			(*(((*curspecies)->organisms).begin()))->super_champ_offspring=pop_size-half_pop;
			(*curspecies)->expected_offspring=pop_size-half_pop;
			(*curspecies)->age_of_last_improvement=(*curspecies)->age;
			++curspecies;
			//Get rid of all species under the first 2
			while(curspecies!=sorted_species.end()) {
				(*curspecies)->expected_offspring=0;
				++curspecies;
			}
		}
		else {
			curspecies=sorted_species.begin();
			(*(((*curspecies)->organisms).begin()))->super_champ_offspring+=pop_size-half_pop;
			(*curspecies)->expected_offspring=pop_size-half_pop;
		}
	}
	//STOLEN BABIES:  The system can take expected offspring away from
	//  worse species and give them to superior species depending on
	//  the system parameter babies_stolen (when babies_stolen > 0)
	else if (babies_stolen>0) {
		//Take away a constant number of expected offspring from the worst few species
		stolen_babies=0;
		curspecies=sorted_species.end();
		curspecies--;
		while ((stolen_babies<NUM_STOLEN)&&
			(curspecies!=sorted_species.begin())) {
			//cout<<"Considering Species "<<(*curspecies)->id<<": age "<<(((*curspecies)->age))<<" expected offspring "<<(((*curspecies)->expected_offspring))<<endl;
			if ((((*curspecies)->age)>5)&&
				(((*curspecies)->expected_offspring)>2)) {
				//cout<<"STEALING!"<<endl;
				//This species has enough to finish off the stolen pool
				if (((*curspecies)->expected_offspring-1)>=(NUM_STOLEN-stolen_babies)) {
					(*curspecies)->expected_offspring-=(NUM_STOLEN-stolen_babies);
					stolen_babies=NUM_STOLEN;
				}
				//Not enough here to complete the pool of stolen
				else {
					stolen_babies+=(*curspecies)->expected_offspring-1;
					(*curspecies)->expected_offspring=1;
				}
			}
			curspecies--;
			//if (stolen_babies>0)
			//cout<<"stolen babies so far: "<<stolen_babies<<endl;
		}
		//cout<<"STOLEN BABIES: "<<stolen_babies<<endl;

		//Mark the best champions of the top species to be the super champs
		//who will take on the extra offspring for cloning or mutant cloning
		curspecies=sorted_species.begin();
		//Determine the exact number that will be given to the top three
		//They get , in order, 1/5 1/5 and 1/10 of the stolen babies
		one_fifth_stolen=babies_stolen/5;
		one_tenth_stolen=babies_stolen/10;
		//Don't give to dying species even if they are champs
		while(((*curspecies)->last_improved()>dropoff_age)&&(curspecies!=sorted_species.end()))
			++curspecies;
		//Concentrate A LOT on the number one species
		if ((stolen_babies>=one_fifth_stolen)&&(curspecies!=sorted_species.end())) {
			(*(((*curspecies)->organisms).begin()))->super_champ_offspring=one_fifth_stolen;
			(*curspecies)->expected_offspring+=one_fifth_stolen;
			stolen_babies-=one_fifth_stolen;
			//cout<<"Gave "<<one_fifth_stolen<<" babies to Species "<<(*curspecies)->id<<endl;
			cout<<"The best superchamp is "<<(*(((*curspecies)->organisms).begin()))->gnome->genome_id<<endl;
			//Print this champ to file "champ" for observation if desired
			//IMPORTANT:  This causes generational file output 
			//print_Genome_tofile((*(((*curspecies)->organisms).begin()))->gnome,"champ");
			curspecies++;
		}
		//Don't give to dying species even if they are champs
		while(((*curspecies)->last_improved()>dropoff_age)&&(curspecies!=sorted_species.end()))
			++curspecies;
		if ((curspecies!=sorted_species.end())) {
			if (stolen_babies>=one_fifth_stolen) {
				(*(((*curspecies)->organisms).begin()))->super_champ_offspring=one_fifth_stolen;
				(*curspecies)->expected_offspring+=one_fifth_stolen;
				stolen_babies-=one_fifth_stolen;
				//cout<<"Gave "<<one_fifth_stolen<<" babies to Species "<<(*curspecies)->id<<endl;
				curspecies++;
			}
		}
		//Don't give to dying species even if they are champs
		while(((*curspecies)->last_improved()>dropoff_age)&&(curspecies!=sorted_species.end()))
			++curspecies;
		if (curspecies!=sorted_species.end())
			if (stolen_babies>=one_tenth_stolen) {
				(*(((*curspecies)->organisms).begin()))->super_champ_offspring=one_tenth_stolen;
				(*curspecies)->expected_offspring+=one_tenth_stolen;
				stolen_babies-=one_tenth_stolen;
				//cout<<"Gave "<<one_tenth_stolen<<" babies to Species "<<(*curspecies)->id<<endl;
				curspecies++;
			}
		//Don't give to dying species even if they are champs
		while(((*curspecies)->last_improved()>dropoff_age)&&(curspecies!=sorted_species.end()))
			++curspecies;
		while((stolen_babies>0)&&
			(curspecies!=sorted_species.end())) {
			//Randomize a little which species get boosted by a super champ
			if (randfloat()>0.1)
				if (stolen_babies>3) {
					(*(((*curspecies)->organisms).begin()))->super_champ_offspring=3;
					(*curspecies)->expected_offspring+=3;
					stolen_babies-=3;
					//cout<<"Gave 3 babies to Species "<<(*curspecies)->id<<endl;
				}
				else {
					//cout<<"3 or less babies available"<<endl;
					(*(((*curspecies)->organisms).begin()))->super_champ_offspring=stolen_babies;
					(*curspecies)->expected_offspring+=stolen_babies;
					//cout<<"Gave "<<stolen_babies<<" babies to Species "<<(*curspecies)->id<<endl;
					stolen_babies=0;
				}
			curspecies++;
			//Don't give to dying species even if they are champs
			while((curspecies!=sorted_species.end())&&((*curspecies)->last_improved()>dropoff_age))
				++curspecies;
		}
		//cout<<"Done giving back babies"<<endl;
		//If any stolen babies aren't taken, give them to species #1's champ
		if (stolen_babies>0) {
			//cout<<"Not all given back, giving to best Species"<<endl;
			curspecies=sorted_species.begin();
			(*(((*curspecies)->organisms).begin()))->super_champ_offspring+=stolen_babies;
			(*curspecies)->expected_offspring+=stolen_babies;
			stolen_babies=0;
		}
	}
	//Kill off all Organisms marked for death.  The remainder
	//will be allowed to reproduce.
	curorg=organisms.begin();
	while(curorg!=organisms.end()) {
		if (((*curorg)->eliminate)) {
			//Remove the organism from its Species
			// ((*curorg)->species)->remove_org(*curorg);
			//Delete the organism from memory
			delete (*curorg);
			//Remember where we are
			deadorg=curorg;
			++curorg;
			//Remove the organism from the master list
			organisms.erase(deadorg);
		}
		else {
			++curorg;
		}
	}
	cout<<"Reproducing"<<endl;
	//Perform reproduction.  Reproduction is done on a per-Species
	//basis.  (So this could be paralellized potentially.)
	for(curspecies=species.begin();curspecies!=species.end();++curspecies) {
		reproduce(generation,*curspecies,sorted_species);
	}    
	cout<<"Reproduction Complete"<<endl;

	int i=1;
	for(curspecies=species.begin();curspecies!=species.end();++curspecies) cerr << "espece " << i++ << endl;
	
	//Destroy and remove the old generation from the organisms and species  
	curorg=organisms.begin();
	while(curorg!=organisms.end()) {
		//Remove the organism from its Species (first find the species..)
		for (curspecies=species.begin(); curspecies!=species.end();curspecies++)
			if ((*curspecies)->remove_org(*curorg)) break;
		//Delete the organism from memory
		delete (*curorg);
		//Remember where we are
		deadorg=curorg;
		++curorg;
		//Remove the organism from the master list
		organisms.erase(deadorg);
	}
	
	i=1;
	for(curspecies=species.begin();curspecies!=species.end();++curspecies) cerr << "espece " << i++ << endl;
	
	//Remove all empty Species and age ones that survive
	//As this happens, create master organism list for the new generation
	curspecies=species.begin();
	orgcount=0;	
	while(curspecies!=species.end()) {
		if (((*curspecies)->organisms.size())==0) {
			delete (*curspecies);
			deadspecies=curspecies;
			++curspecies;
			species.erase(deadspecies);
		}
		//Age surviving Species and 
		//Rebuild master Organism list: NUMBER THEM as they are added to the list
		else {
			//Age any Species that is not newly created in this generation
			if ((*curspecies)->novel) {
				(*curspecies)->novel=false;
			}
			else ++((*curspecies)->age);

			//Go through the organisms of the curspecies and add them to 
			//the master list
			for(curorg=((*curspecies)->organisms).begin();curorg!=((*curspecies)->organisms).end();++curorg) {
				((*curorg)->gnome)->genome_id=orgcount++;
				organisms.push_back(*curorg);
			}
			++curspecies;
		}
	}
	//Remove the innovations of the current generation
	curinnov=innovations.begin();
	while(curinnov!=innovations.end()) {
		delete (*curinnov);
		deadinnov=curinnov;
		++curinnov;
		innovations.erase(deadinnov);
	}

	//DEBUG: Check to see if the best species died somehow
	// We don't want this to happen
	curspecies=species.begin();
	best_ok=false;
	while(curspecies!=species.end()) {
		if (((*curspecies)->id)==best_species_num) best_ok=true;
		++curspecies;
	}
	if (!best_ok) {
		cout<<"ERROR: THE BEST SPECIES DIED!"<<endl;
	}
	else {
		cout<<"The best survived: "<<best_species_num<<endl;
	}
	//DEBUG: Checking the top organism's duplicate in the next gen
	//This prints the champ's child to the screen
	for(curorg=organisms.begin();curorg!=organisms.end();++curorg) {
		if ((*curorg)->pop_champ_child) {
			cout<<"At end of reproduction cycle, the child of the pop champ is: "<<(*curorg)->gnome<<endl;
		}
	}
	cout<<"babies_stolen at end: "<<babies_stolen<<endl;
	cout<<"Epoch complete"<<endl; 
	return true;
}

//Perform mating and mutation to form next generation
bool Population::reproduce(int generation,Species* spec,list<Species*> &sorted_species) {
	int count;
	list<Organism*>::iterator curorg;
	int poolsize;  //The number of Organisms in the old generation
	list<Species*>::iterator curspecies;  //For adding baby
	Species *newspecies; //For babies in new Species
	Organism *comporg;  //For Species determination through comparison
	int orgnum;  //Random variable
	int orgcount;
	Organism *mom; //Parent Organisms
	Organism *dad;

	if (spec->size()==0) {
		bool found=false;
		// look for mother
		while (!found) {
			int i=0,stop=rand()%sorted_species.size(); // espece d'arret
			curspecies=sorted_species.begin(); 
			while (i++<stop) curspecies++;
			if ((*curspecies)->size()>0) {
				mom=*((*curspecies)->organisms.begin());
				found=true;
			}
		}
		found=false;
		// look for father
		while (!found) {
			int i=0,stop=rand()%sorted_species.size(); // espece d'arret
			curspecies=sorted_species.begin(); 
			while (i++<stop) curspecies++;
			if ((*curspecies)->size()>0) {
				dad=*((*curspecies)->organisms.rbegin());
				found=true;
			}
		}
	}
	else if (spec->size()==1) {
		mom=*(spec->organisms.begin());
		bool found=false;
		// look for father
		while (!found) {
			int i=0,stop=rand()%sorted_species.size(); // espece d'arret
			curspecies=sorted_species.begin(); 
			while (i++<stop) curspecies++;
			if ((*curspecies)->size()>0) {
				dad=*((*curspecies)->organisms.rbegin());
				found=true;
			}
		}
	}
	else {
		mom=*(spec->organisms.begin());
		dad=*(spec->organisms.rbegin());
	}
	
	Organism *baby;  //The new Organism
	Genome *new_genome;  //For holding baby's genes
	Species *randspecies;  //For mating outside the Species
	double randmult;
	int randspeciesnum;
	int spcount;  
	list<Species*>::iterator cursp;
	Network *net_analogue;  //For adding link to test for recurrency
	int pause;
	bool outside;
	bool found;  //When a Species is found
	bool champ_done=false; //Flag the preservation of the champion  
	Organism *thechamp;
	int giveup; //For giving up finding a mate outside the species
	bool mut_struct_baby;
	bool mate_baby;
	//The weight mutation power is species specific depending on its age
	double mut_power=weight_mut_power;

	//Roulette wheel variables
	//double total_fitness=0.0;
	//double marble;  //The marble will have a number between 0 and total_fitness
	//double spin;  //Fitness total while the wheel is spinning
	//Compute total fitness of species for a roulette wheel
	//Note: You don't get much advantage from a roulette here
	// because the size of a species is relatively small.
	// But you can use it by using the roulette code here
	//for(curorg=organisms.begin();curorg!=organisms.end();++curorg) {
	//	total_fitness+=(*curorg)->fitness;
	//}

	//Check for a mistake
	if ((mom->expected_offspring>0)&&
		(organisms.size()==0)) {
		cout<<"ERROR:  ATTEMPT TO REPRODUCE OUT OF EMPTY SPECIES"<<endl;
		return false;
	}
	poolsize=organisms.size()-1;
	thechamp=(*(organisms.begin()));

	//Create the designated number of offspring for the Species
	//one at a time
	for (count=0;count<mom->expected_offspring;count++) {
		mut_struct_baby=false;
		mate_baby=false;
		outside=false;
		//Debug Trap
		if (mom->expected_offspring>pop_size) {
			cout<<"ALERT: EXPECTED OFFSPRING = "<<mom->expected_offspring<<endl;
			cin>>pause;
		}
		//If we have a super_champ (Population champion), finish off some special clones
		if ((thechamp->super_champ_offspring) > 0) {
			mom=thechamp;
			new_genome=(mom->gnome)->duplicate(count);
			if ((thechamp->super_champ_offspring) == 1) {
			}
			//Most superchamp offspring will have their connection weights mutated only
			//The last offspring will be an exact duplicate of this super_champ
			//Note: Superchamp offspring only occur with stolen babies!
			//      Settings used for published experiments did not use this
			if ((thechamp->super_champ_offspring) > 1) {
				if ((randfloat()<0.8)||
					(mutate_add_link_prob==0.0)) 
					//ABOVE LINE IS FOR:
					//Make sure no links get added when the system has link adding disabled
					new_genome->mutate_link_weights(mut_power,1.0,GAUSSIAN);
				else {
					//Sometimes we add a link to a superchamp
					net_analogue=net_analogue->genesis(generation,new_genome);
					new_genome->mutate_add_link(innovations,cur_innov_num,newlink_tries);
					delete net_analogue;
					mut_struct_baby=true;
				}
			}
			baby=new Organism(0.0,new_genome,generation);
			if ((thechamp->super_champ_offspring) == 1) {
				if (thechamp->pop_champ) {
					//cout<<"The new org baby's genome is "<<baby->gnome<<endl;
					baby->pop_champ_child=true;
					baby->high_fit=mom->orig_fitness;
				}
			}
			thechamp->super_champ_offspring--;
		}
		//If we have a Species champion, just clone it 
		else if ((!champ_done)&&(mom->expected_offspring>5)) {
			mom=thechamp; //Mom is the champ
			new_genome=(mom->gnome)->duplicate(count);
			baby=new Organism(0.0,new_genome,generation);  //Baby is just like mommy
			champ_done=true;
		}
		//First, decide whether to mate or mutate
		//If there is only one organism in the pool, then always mutate
		else if ((randfloat()<mutate_only_prob)||poolsize== 0) {
			//Choose the random parent
			//RANDOM PARENT CHOOSER
			orgnum=randint(0,poolsize);
			curorg=organisms.begin();
			for(orgcount=0;orgcount<orgnum;orgcount++) ++curorg;                       

			/*
			//Roulette Wheel
			marble=randfloat()*total_fitness;
			curorg=organisms.begin();
			spin=(*curorg)->fitness;
			while(spin<marble) {
				++curorg;
				//Keep the wheel spinning
				spin+=(*curorg)->fitness;
			}
			//Finished roulette
			*/

			mom=(*curorg);
			new_genome=(mom->gnome)->duplicate(count);
			//Do the mutation depending on probabilities of 
			//various mutations
			if (randfloat()<mutate_add_node_prob) {
				//cout<<"mutate add node"<<endl;
				new_genome->mutate_add_node(innovations,cur_node_id,cur_innov_num);
				mut_struct_baby=true;
			}
			else if (randfloat()<mutate_add_link_prob) {
				//cout<<"mutate add link"<<endl;
				net_analogue=net_analogue->genesis(generation,new_genome);
				new_genome->mutate_add_link(innovations,cur_innov_num,newlink_tries);
				delete net_analogue;
				mut_struct_baby=true;
			}
			//NOTE:  A link CANNOT be added directly after a node was added because the phenotype
			//       will not be appropriately altered to reflect the change
			else {
				//If we didn't do a structural mutation, we do the other kinds
				if (randfloat()<mutate_random_trait_prob) {
					//cout<<"mutate random trait"<<endl;
					new_genome->mutate_random_trait();
				}
				if (randfloat()<mutate_link_trait_prob) {
					//cout<<"mutate_link_trait"<<endl;
					new_genome->mutate_link_trait(1);
				}
				if (randfloat()<mutate_node_trait_prob) {
					//cout<<"mutate_node_trait"<<endl;
					new_genome->mutate_node_trait(1);
				}
				if (randfloat()<mutate_link_weights_prob) {
					//cout<<"mutate_link_weights"<<endl;
					new_genome->mutate_link_weights(mut_power,1.0,GAUSSIAN);
				}
				if (randfloat()<mutate_toggle_enable_prob) {
					//cout<<"mutate toggle enable"<<endl;
					new_genome->mutate_toggle_enable(1);
				}
				if (randfloat()<mutate_gene_reenable_prob) {
					//cout<<"mutate gene reenable"<<endl;
					new_genome->mutate_gene_reenable();
				}
			}
			baby=new Organism(0.0,new_genome,generation);
		}
		//Otherwise we should mate 
		else {
			//Choose the random mom
			orgnum=randint(0,poolsize);
			curorg=organisms.begin();
			for(orgcount=0;orgcount<orgnum;orgcount++) ++curorg;

			/*
			//Roulette Wheel
			marble=randfloat()*total_fitness;
			curorg=organisms.begin();
			spin=(*curorg)->fitness;
			while(spin<marble) {
				++curorg;
				//Keep the wheel spinning
				spin+=(*curorg)->fitness;
			}
			//Finished roulette
			*/

			mom=(*curorg);         
			//Choose random dad
			if ((randfloat()>interspecies_mate_rate)) {
				//Mate within Species
				orgnum=randint(0,poolsize);
				curorg=organisms.begin();
				for(orgcount=0;orgcount<orgnum;orgcount++) ++curorg;
	
				/*
				//Use a roulette wheel
				marble=randfloat()*total_fitness;
				curorg=organisms.begin();
				spin=(*curorg)->fitness;
				while(spin<marble) {
					++curorg;
					//Keep the wheel spinning
					spin+=(*curorg)->fitness;
				}
				//Finished roulette
				*/	

				dad=(*curorg);
			}
			else {
				//Mate outside Species  
				//Select a random species
				giveup=0;  //Give up if you cant find a different Species
				do {
					//This old way just chose any old species
					//randspeciesnum=randint(0,(pop->species).size()-1);
					//Choose a random species tending towards better species
					randmult=gaussrand()/4;
					if (randmult>1.0) randmult=1.0;
					//This tends to select better species
					randspeciesnum=(int) floor((randmult*(sorted_species.size()-1.0))+0.5);
					cursp=(sorted_species.begin());
					for(spcount=0;spcount<randspeciesnum;spcount++) ++cursp;
					randspecies=(*cursp);
					++giveup;
				}
				while((randspecies==this->species.front())&&(giveup<5));
				//New way: Make dad be a champ from the random species
				dad=(*((randspecies->organisms).begin()));
				outside=true;
				//cout<<"outside species"<<endl;
			}

			//Perform mating based on probabilities of different mating types
			if (randfloat()<mate_multipoint_prob) { 
				new_genome=(mom->gnome)->mate_multipoint(dad->gnome,count,mom->orig_fitness,dad->orig_fitness);
				//cout<<"mate multipoint baby: "<<new_genome<<endl;
			}
			else if (randfloat()<(mate_multipoint_avg_prob/(mate_multipoint_avg_prob+mate_singlepoint_prob))) {
				new_genome=(mom->gnome)->mate_multipoint_avg(dad->gnome,count,mom->orig_fitness,dad->orig_fitness);
				//cout<<"mate multipoint_avg baby: "<<new_genome<<endl;
			}
			else {
				new_genome=(mom->gnome)->mate_singlepoint(dad->gnome,count);
			}
			mate_baby=true;
			//Determine whether to mutate the baby's Genome
			//This is done randomly or if the mom and dad are the same organism
			if ((randfloat()>mate_only_prob)||
				((dad->gnome)->genome_id==(mom->gnome)->genome_id)||
				(((dad->gnome)->compatibility(mom->gnome))==0.0)) {
				//Do the mutation depending on probabilities of 
				//various mutations
				if (randfloat()<mutate_add_node_prob) {
					new_genome->mutate_add_node(innovations,cur_node_id,cur_innov_num);
					//cout<<"mutate_add_node: "<<new_genome<<endl;
					mut_struct_baby=true;
				}
				else if (randfloat()<mutate_add_link_prob) {
					net_analogue=net_analogue->genesis(generation,new_genome);
					new_genome->mutate_add_link(innovations,cur_innov_num,newlink_tries);
					delete net_analogue;
					//cout<<"mutate_add_link: "<<new_genome<<endl;
					mut_struct_baby=true;
				}
				else {
					//Only do other mutations when not doing sturctural mutations
					if (randfloat()<mutate_random_trait_prob) {
						new_genome->mutate_random_trait();
						//cout<<"..mutate random trait: "<<new_genome<<endl;
					}
					if (randfloat()<mutate_link_trait_prob) {
						new_genome->mutate_link_trait(1);
						//cout<<"..mutate link trait: "<<new_genome<<endl;
					}
					if (randfloat()<mutate_node_trait_prob) {
						new_genome->mutate_node_trait(1);
						//cout<<"mutate_node_trait: "<<new_genome<<endl;
					}
					if (randfloat()<mutate_link_weights_prob) {
						new_genome->mutate_link_weights(mut_power,1.0,GAUSSIAN);
						//cout<<"mutate_link_weights: "<<new_genome<<endl;
					}
					if (randfloat()<mutate_toggle_enable_prob) {
						new_genome->mutate_toggle_enable(1);
						//cout<<"mutate_toggle_enable: "<<new_genome<<endl;
					}
					if (randfloat()<mutate_gene_reenable_prob) {
						new_genome->mutate_gene_reenable(); 
						//cout<<"mutate_gene_reenable: "<<new_genome<<endl;
					}
				}
				//Create the baby
				baby=new Organism(0.0,new_genome,generation);
			}
			else {
				//Create the baby without mutating first
				baby=new Organism(0.0,new_genome,generation);
			}
		}
		//Add the baby to its proper Species
		//If it doesn't fit a Species, create a new one
		baby->mut_struct_baby=mut_struct_baby;
		baby->mate_baby=mate_baby;
		curspecies=species.begin();
		if (curspecies==species.end()) {
			//Create the first species
			newspecies=new Species(++last_species,true);
			species.push_back(newspecies);
			newspecies->add_Organism(baby);  //Add the baby
			// baby->species=newspecies;  //Point the baby to its species ....tableau associatif.
		} 
		else {
			comporg=(*curspecies)->first();
			found=false;
			while((curspecies!=species.end())&&(!found)) {	
				if (comporg==0) {
					//Keep searching for a matching species
					++curspecies;
					if (curspecies!=species.end())
						comporg=(*curspecies)->first();
				}
				else if (((baby->gnome)->compatibility(comporg->gnome))<compat_threshold) {
					//Found compatible species, so add this organism to it
					(*curspecies)->add_Organism(baby);
					//baby->species=(*curspecies);  //Point organism to its species
					found=true;  //Note the search is over
				}
				else {
					//Keep searching for a matching species
					++curspecies;
					if (curspecies!=species.end()) 
						comporg=(*curspecies)->first();
				}
			}
			//If we didn't find a match, create a new species
			if (found==false) {
				newspecies=new Species(++last_species,true);
				species.push_back(newspecies);
				newspecies->add_Organism(baby);  //Add the baby
				//	baby->species=newspecies;  //Point baby to its species
			}
		} //end else 
	}
	return true;
}

//Take a snapshot of the current generation for visualization purpose
void Population::snapshot() {
	Generation_viz *thisgen=new Generation_viz;
	list<Species*>::iterator curspecies;
	Species_viz *curviz;
	//Add each Species in this generation to the snapshot
	for(curspecies=species.begin();curspecies!=species.end();++curspecies) {
		curviz=new Species_viz((*curspecies));
		(thisgen->species_list).push_back(curviz);
	}
	//Add the snapshot to the Population's entire list of snapshots
	generation_snapshots.push_back(thisgen);
}

/* faire ca en STL !!
//Visualize the Population's Speciation using a graphic depiction
//width and height denote the size of the window
void Population::visualize(int width,int height,int start_gen,int stop_gen) {
  list<Generation_viz*>::iterator curgen;
  list<Species_viz*>::iterator curspec;
  int spaces; //Number of blank spaces in the current row
  int xpixels;
  int ypixels;
  double popsize=(double) organisms.size();
  double species_pixels; //Number pf pixels for current species
  int xpos; 
  int ypos;
  int space=2;
  PopulationWindow *window;
  double fitness_total=0.0; //For averaging
  double total_species=0.0;
  double variance_total=0.0;  //Intermediate part of variance calculation  
  bool y_for_gens;  //Marks which axis should be used for time

  //Don't allow illegal values
  if ((start_gen<stop_gen)&&
      (start_gen>0)&&
      (stop_gen<=(generation_snapshots.size()))) {

  //Make a decision about which axis to use for time (generations)
  if (width>height) y_for_gens=false;
  else y_for_gens=true;

  //First compute some statistics to help in visualization
  //Get the mean max_fitness of all species at all generations
  for(curgen=generation_snapshots.begin();curgen!=generation_snapshots.end();++curgen) {

    for(curspec=((*curgen)->species_list).begin();curspec!=((*curgen)->species_list).end();++curspec) {
      fitness_total+=(*curspec)->max_fitness;
      total_species+=1.0;
    }    
  }
  mean_fitness=fitness_total/total_species;

  cout<<"Got mean fitness: "<<mean_fitness<<endl;

  //Get the standard deviation
  for(curgen=generation_snapshots.begin();curgen!=generation_snapshots.end();++curgen) {

    for(curspec=((*curgen)->species_list).begin();curspec!=((*curgen)->species_list).end();++curspec) {
      variance_total+=pow((((*curspec)->max_fitness)-mean_fitness),2);
    }
  }
  variance=variance_total/(total_species-1.0);
  standard_deviation=sqrt(total_species*variance/(total_species-1.0));
  
  cout<<"Got mean standard deviation: "<<standard_deviation<<endl;

  //Go through each generation
  for(curgen=generation_snapshots.begin();curgen!=generation_snapshots.end();++curgen) {
    //Within each generation, compute the x values for the Species

    //First we see how many blank spaces (Size 3 pixels) we will have
    spaces=((*curgen)->species_list).size()-1;
    
    //There are 25 pixels given for printing a gen_num on left, space 3 on right
    xpixels=width-23-spaces*space;  //6 is the borders, pixels for each space
    ypixels=height-23-spaces*space;

    xpos=25;  //Starting xpos
    ypos=height-25;  //Starting ypos if we were orienting along y

    //Now go through the Species snapshots themselves
    for(curspec=((*curgen)->species_list).begin();curspec!=((*curgen)->species_list).end();++curspec) {
      //Compute the # of pixels this Species will comprise
      if (y_for_gens)
	species_pixels=(((double) (*curspec)->size)/popsize)*((double) xpixels);
      else species_pixels=(((double) (*curspec)->size)/popsize)*((double) ypixels);
      species_pixels=floor (species_pixels+0.5);
      
      if (y_for_gens) {
	(*curspec)->startx=xpos;
	xpos+=(int) species_pixels;
	(*curspec)->endx=xpos;
	xpos+=space;  //Put a potential space between each Species
      }
      else { 
	(*curspec)->starty=ypos;
	ypos-=(int) species_pixels;
	(*curspec)->endy=ypos;
	ypos-=space;  //Put a potential space between each Species
      }
    }
  }
  //Now pop up the window and draw the snapshot
  window = new PopulationWindow(this,width,height,start_gen,stop_gen);
  window->show();
  myapp->run();
  }
  else {
    cout<<"Sorry, the generations you asked for cannot be printed."<<endl;
    cout<<"Please check the values to make sure they are valid."<<endl;
  }
}
*/

//Print Population to a file specified by a string
bool Population::print_to_file(char *filename){
	list<Organism*>::iterator curorg;
	ofstream outFile(filename,ios::out);
	//Make sure it worked
	if (!outFile) {
		cerr<<"Can't open "<<filename<<" for output"<<endl;
		return false;
	}
	//Print all the Organisms' Genomes to the outFile
	for(curorg=organisms.begin();curorg!=organisms.end();++curorg) {
		((*curorg)->gnome)->print_to_file(outFile);
		//We can confirm by writing the genome #'s to the screen
		//cout<<((*curorg)->gnome)->genome_id<<endl;
	}
	outFile.close();
	return true;
}

bool Population::print_to_file_by_species(char *filename) {
	list<Species*>::iterator curspecies;
	ofstream outFile(filename,ios::out);
	//Make sure it worked
	if (!outFile) {
		cerr<<"Can't open "<<filename<<" for output"<<endl;
		return false;
	}
	//Step through the Species and print them to the file
	for(curspecies=species.begin();curspecies!=species.end();++curspecies) {
		(*curspecies)->print_to_file(outFile);
	}
	outFile.close();
	return true;
}

//Debug Population
///Note: This checks each genome by verifying each one
//       Only useful for debugging
bool Population::verify() {
	list<Organism*>::iterator curorg;
	bool verification;
	for(curorg=organisms.begin();curorg!=organisms.end();++curorg) {
		verification=((*curorg)->gnome)->verify();
	}
	return verification;
} 
