#ifndef _GeneticSpecies
#define _GeneticSpecies

#include <iostream>
#include <fstream>
#include <list>
#include "GeneticOrganism.h"
#include "GeneticGenome.h"
#include "Neat.h"

using namespace std;

/* ---------------------------------------------  */
/* SPECIES CLASS:
   A Species is a group of similar Organisms      
   Reproduction takes place mostly within a
   single species, so that compatible organisms
   can mate.                                      */
/* ---------------------------------------------  */

class Species : public NEAT {

public:

	int id;
	int age;  //The age of the Species 
	double ave_fitness;  //The average fitness of the Species
	double max_fitness;  //Max fitness of the Species
	double max_fitness_ever;  //The max it ever had
	int expected_offspring;
	bool novel;
	bool checked;
	list<Organism*> organisms; //The organisms in the Species
	int age_of_last_improvement;  //If this is too long ago, the Species will goes extinct

	bool add_Organism(Organism *o);
	Organism *first();

	//Change the fitness of all the organisms in the species 
	//to possibly depend slightly on the age of the species
	//and then divide it by the size of the species so that the
	//organisms in the species "share" the fitness
	void adjust_fitness();
	double compute_average_fitness(); 
	double compute_max_fitness();

	//Counts the number of offspring expected from all its members
	//skim is for keeping track of remaining fractional parts of offspring
	//and distributing them among species
	double count_offspring(double skim);

	//Compute generations since last improvement
	int last_improved() {
		return age-age_of_last_improvement;
	}

	//Remove an organism from Species
	bool remove_org(Organism *org);

	double size() {
		return organisms.size();
	}

	Organism *get_champ();

	Species(int i) {
		id=i;
		age=1;
		ave_fitness=0.0;
		expected_offspring=0;
		novel=false;
		age_of_last_improvement=0;
		max_fitness=0;
		max_fitness_ever=0;
	}

	//Allows the creation of a Species that won't age (a novel one)
	//This protects new Species from aging inside their first generation
	Species(int i,bool n) {
		id=i;
		age=1;
		ave_fitness=0.0;
		expected_offspring=0;
		novel=n;
		age_of_last_improvement=0;
		max_fitness=0;
		max_fitness_ever=0;
	}

	~Species() {
		list<Organism*>::iterator curorg;
		for(curorg=organisms.begin();curorg!=organisms.end();++curorg) {
			delete (*curorg);
		}
	}

	bool print_to_file(ofstream &outFile);
	bool operator<(Species *x);
};

/* Special Structures for Speciation visualization */
class Species_viz : public NEAT {
 
public:
  
	int id;
	int size;
	double fitness;   //The average fitness
	double max_fitness;  //Max fitness for Species
	int startx;   //Starting x position on window
	int endx;     //Ending x position on window
	//The program can orient species along the x or y axis
	int starty;
	int endy;
	bool firstgen;  //Is it the Species' first generation?

	//A visualization of a Species is constructed off of that Species s
	Species_viz(Species *s) {
		id=s->id;
		size=(s->organisms).size();
		fitness=s->ave_fitness;
		max_fitness=s->max_fitness;
		startx=0;  //These positions will be computed later
		endx=0;
		if (s->age==1) firstgen=true; 
		else firstgen=false;
	}
};

/* The visualization of a generation is a list of all the Species visualizations
	of that generation */
struct Generation_viz {
	int gen_num;
	list<Species_viz*> species_list;
};

#endif

