#ifndef _Utiles
#define _Utiles

#include <iostream>
#include <fstream>
#include <cstdlib>
#include <cmath>
#include <list>
#include <vector>

using namespace std;

typedef int boolean;
typedef int nodetype;
typedef int nodeplace;
typedef int mutator;
typedef int functype;
typedef int innovtype;

class Utiles {

public:

	/* These are NNode types */
	static const int NEURON;
	static const int SENSOR;

	/* These have to do with genetic usage */
	static const int HIDDEN;
	static const int INPUT;
	static const int OUTPUT;
	static const int BIAS;

	/* Mutators are variables that specify a kind of mutation */
	static const mutator GAUSSIAN;  //This adds Gaussian noise to the weights
	static const mutator COLDGAUSSIAN;  //This sets weights to numbers chosen from a Gaussian distribution

	/* Activation Types */
	static const int SIGMOID;

	//Typedef used in the Innovation class
	static const int NEWNODE;
	static const int NEWLINK;
	
	static const int TRUE;
	static const int FALSE;

	Utiles () {}

	/* Inline Random Functions */
	static bool randbit() {
		return rand()%2;
	}
	static int randposneg() {
		if (randbit()) return 1; 
		else return -1;
	}
	static int randint(int x,int y) {
		return rand()%(y-x+1)+x;
	}
	static double randfloat() {
		return (rand())/(RAND_MAX+1.0);
	}

	//Returns a normally distributed deviate with 0 mean and unit variance
	//Algorithm is from Numerical Recipes in C, Second Edition
	static double gaussrand() {
		static int iset=0;
		static double gset;
		double fac,rsq,v1,v2;

		if (iset==0) {
			do {
				v1=2.0*((rand())/(RAND_MAX+1.0))-1.0;
				v2=2.0*((rand())/(RAND_MAX+1.0))-1.0;
				rsq=v1*v1+v2*v2;
			} 
			while (rsq>=1.0 || rsq==0.0);
			fac=sqrt(-2.0*log(rsq)/rsq);
			gset=v1*fac;
			iset=1;
			return v2*fac;
		}
		else {
			iset=0;
			return gset;
		}
	}

	//This is an incorrect gassian distribution...but it is faster than gaussrand (maybe it's good enough?)
	static double gaussrand_wrong() {
		return (randposneg())*(sqrt(-log((rand()*1.0)/RAND_MAX)));
	}

	/* Sigmoid activation function- helper for NNode */
	// SIGMOID FUNCTION ********************************
	//This is a signmoidal activation function, which is an S-shaped squashing function
	//It smoothly limits the amplitude of the output of a neuron to between 0 and 1
	//It is a helper to the neural-activation function get_active_out
	//It is made inline so it can execute quickly since it is at every non-sensor 
	// node in a network.
	//NOTE:  In order to make node insertion in the middle of a link possible,
	// the signmoid can be shifted to the right and more steeply sloped:
	// slope=4.924273
	// constant= 2.4621365
	// These parameters optimize mean squared error between the old output,
	// and an output of a node inserted in the middle of a link between
	// the old output and some other node. 
	// When not right-shifted, the steepened slope is closest to a linear
	// ascent as possible between -0.5 and 0.5

	static double fsigmoid(double activesum,double slope,double constant) {

		//RIGHT SHIFTED ---------------------------------------------------------
		//return (1/(1+(exp(-(slope*activesum-constant))))); //ave 3213 clean on 40 runs of p2m and 3468 on another 40 
		//41394 with 1 failure on 8 runs

		//LEFT SHIFTED ----------------------------------------------------------
		//return (1/(1+(exp(-(slope*activesum+constant))))); //original setting ave 3423 on 40 runs of p2m, 3729 and 1 failure also

		//PLAIN SIGMOID ---------------------------------------------------------
		//return (1/(1+(exp(-activesum)))); //3511 and 1 failure

		//LEFT SHIFTED NON-STEEPENED---------------------------------------------
		//return (1/(1+(exp(-activesum-constant)))); //simple left shifted

		//NON-SHIFTED STEEPENED
		return (1/(1+(exp(-(slope*activesum))))); //Compressed
	}
	
	~Utiles () {}

};

#endif
