#include "Dirty.h"

Dirtytable *dirty_open(int size) {
	Dirtytable *dt = (Dirtytable *)malloc(sizeof(Dirtytable));
	if(!dt) return NULL;

	dt->size = size;
	dt->rects = (SDL_Rect *)calloc(size, sizeof(SDL_Rect));
	if(!dt->rects) {
		free(dt);
		return NULL;
	}

	dt->count = 0;
	dt->best = 0;
	return dt;
}

void dirty_close(Dirtytable *dt) {
	free(dt->rects);
	free(dt);
}

void mergerect(SDL_Rect *from, SDL_Rect *to) {
	int x1 = from->x;
	int y1 = from->y;
	int x2 = from->x + from->w;
	int y2 = from->y + from->h;
	if(to->x < x1) x1 = to->x;
	if(to->y < y1) y1 = to->y;
	if(to->x + to->w > x2) x2 = to->x + to->w;
	if(to->y + to->h > y2) y2 = to->y + to->h;
	to->x = x1;
	to->y = y1;
	to->w = x2 - x1;
	to->h = y2 - y1;
}

void intersectrect(SDL_Rect *from, SDL_Rect *to) {
	int Amin, Amax, Bmin, Bmax;
	Amin = to->x;
	Amax = Amin + to->w;
	Bmin = from->x;
	Bmax = Bmin + from->w;
	if(Bmin > Amin) Amin = Bmin;
	to->x = Amin;
	if(Bmax < Amax) Amax = Bmax;
	to->w = Amax - Amin > 0 ? Amax - Amin : 0;

	Amin = to->y;
	Amax = Amin + to->h;
	Bmin = from->y;
	Bmax = Bmin + from->h;
	if(Bmin > Amin) Amin = Bmin;
	to->y = Amin;
	if(Bmax < Amax) Amax = Bmax;
	to->h = Amax - Amin > 0 ? Amax - Amin : 0;
}

void dirty_add(Dirtytable *dt, SDL_Rect *dr) {
	int i = 0, j, best_i, best_loss;
	/*
	 * Look for merger candidates.
	 *
	 * We start right before the best match we
	 * had the last time around. This can give
	 * us large numbers of direct or quick hits
	 * when dealing with old/new rects for moving
	 * objects and the like.
	 */
	best_i = -1;
	best_loss = 100000000;
	if(dt->count) i = (dt->best + dt->count - 1) % dt->count;
	for(j = 0; j < dt->count; ++j) {
		int a1, a2, am, ratio, loss;
		SDL_Rect testr;

		a1 = dr->w * dr->h;

		testr = dt->rects[i];
		a2 = testr.w * testr.h;

		mergerect(dr, &testr);
		am = testr.w * testr.h;

		/* Perfect or Instant Pick? */
		ratio = 100 * am / (a1 > a2 ? a1 : a2);
		if(ratio < PIG_INSTANT_MERGE) {
			/* Ok, this is good enough! Stop searching. */
			mergerect(dr, &dt->rects[i]);
			dt->best = i;
			return;
		}

		loss = am - a1 - a2;
		if(loss < best_loss) {
			best_i = i;
			best_loss = loss;
			dt->best = i;
		}

		++i;
		i %= dt->count;
	}
	/* ...and if the best result is good enough, merge! */
	if((best_i >= 0) && (best_loss < PIG_WORST_MERGE)) {
		mergerect(dr, &dt->rects[best_i]);
		return;
	}

	/* Try to add to table... */
	if(dt->count < dt->size) {
		dt->rects[dt->count++] = *dr;
		return;
	}

	/* Emergency: Table full! Grab best candidate... */
	mergerect(dr, &dt->rects[best_i]);
}

void dirty_merge(Dirtytable *dt, Dirtytable *from) {
	int i;
	for(i = 0; i < from->count; ++i)
		dirty_add(dt, from->rects + i);
}
