#include "DrawNetwork.h"

//network graphic methods
void graph(const Organism* org,bool randomDrawing) {

	//used surfaces
	SDL_Surface* screen;
	SDL_Surface* screen_backup;

	//the system video is initialized 
	if (SDL_Init(SDL_INIT_VIDEO) <0) {
		//SDL_GetError() returns a description of the error
		printf("SDL Init Error: %s\n", SDL_GetError());
		return;
	}

	//when exit, execute SDL_Quit to restore everything
	atexit(SDL_Quit);
	
	//a 32 bit integer used to store screen's flags 
	Uint32 flags = SDL_HWSURFACE|SDL_DOUBLEBUF;

	//SDL_SetVideoMode is used to have screen associated to the monitor 
	if (!(screen = SDL_SetVideoMode(DIM_H, DIM_V, 0,flags))) {
		printf("Screen Init Error : %s\n", SDL_GetError());
		return;
	}

	//white background
	fill_square(screen, NULL, white);

	//screen_backup is a surface of the same dimensions of screen used  to save the image when the user
	//changes its form (window to full screen and vice versa). 
	screen_backup = SDL_CreateRGBSurface(flags,screen->w,screen->h,screen->format->BitsPerPixel,0,0,0,0);
	
	//***********************************
	//here we call the drawing function
	//**********************************
	draw_net(org,screen,randomDrawing);
	
	//SDL_Event struct used to manage events
	SDL_Event event;

	//variables used to control the game loop 
	bool done = false;
	bool paused = false;

	//game or main loop (as you prefer)
	while (!done) {
		//SDL_PollEvent manages user's events 
		while (SDL_PollEvent(&event)) {
			
			//the program quits if someone press the x symbol of the window with the mouse 
			if (event.type == SDL_QUIT)  
				done = true;  

			if (event.type == SDL_KEYDOWN) {
				//it quits also if someone press ESC 
				if (event.key.keysym.sym == SDLK_ESCAPE) 
					done = true; 
				else if (event.key.keysym.sym == SDLK_p) 
					paused = !paused ;

				//visualization form is changed if someone press SPACE 
				//(from window to full screen and vice versa)
				if (event.key.keysym.sym == SDLK_SPACE) {
					//the actual screen is saved in screen_backup
					SDL_BlitSurface(screen,NULL,screen_backup,NULL);
					//visualization form is changed
					flags^=SDL_FULLSCREEN;
					screen = SDL_SetVideoMode(DIM_H, DIM_V, 0,flags);
					//the original image is reBlitted
					SDL_BlitSurface(screen_backup,NULL,screen,NULL);
				}
			}
		}

		//the screen is refreshed
		SDL_Flip(screen);

		//a minimum delay 
		SDL_Delay(1);
	}
}

// this function does the real job inside the window just created
void draw_net(const Organism* org,SDL_Surface* surface,bool randomDrawing) {
	// first reorganize the network in layers:
	list<list<Neuron*> > layers;
	double max_weight_link=org->arrange_topology(layers);

	// initialize the widths of each neuron's row, and order nodes:
	list<int> widths;
	list< list<Neuron*> >::iterator layers_iter;
	for (layers_iter=layers.begin(); layers_iter!=layers.end(); layers_iter++) {
		widths.push_back((*layers_iter).size());
		sortList(*layers_iter);
	}
	
	// some constants initialisations:
	int height=layers.size(); // height (>=2, because inputs and outputs, at least)
	int cell_side=21; // size of a square representing a node 
	int gapH; // horizontal distance between two nodes (will depend on the layer)
	int gapV=(DIM_V/height)-cell_side; // vertical distance between two nodes
	SDL_Rect cell = {0,0,cell_side,cell_side}; // the basic cell to be drawn (a node)
	
	// then we calculate the coordinates before drawing
	int curAbs,curOrd=max((gapV-cell_side)/2,0); // ord initialized with upper offset
	int counter=0;
	list<int>::iterator wid_it=widths.begin();
	list<Neuron*>::iterator curnode_;
	for (layers_iter=layers.begin(); layers_iter!=layers.end(); layers_iter++) {
		gapH=(DIM_H/(*wid_it))-cell_side;
		if (randomDrawing) curAbs=0;
		else {
			int left_offset=max((gapH-cell_side)/2,0),shift_pixels=left_offset/height;
			curAbs=left_offset+(int)(0.5*counter*shift_pixels); // abs initialized with shifted left offset
		}
		for (curnode_=(*layers_iter).begin(); curnode_!=(*layers_iter).end(); curnode_++) {
			if (randomDrawing) (*curnode_)->x=(int)floor(randfloat()*DIM_H);
			else {
				(*curnode_)->x=curAbs;
				curAbs+=(cell_side+gapH);
			}
			(*curnode_)->y=curOrd;
		}
		curOrd+=(cell_side+gapV);
		wid_it++;
		counter++;
	}

	// finally: the drawing
	point s,g;
	int shifts; // to draw wide lines for strong connections
	list<IncomingLink*>::iterator curlink_;
	for (layers_iter=layers.begin(); layers_iter!=layers.end(); layers_iter++) {
		for (curnode_=(*layers_iter).begin(); curnode_!=(*layers_iter).end(); curnode_++) {
			cell.x=(*curnode_)->x;
			cell.y=(*curnode_)->y;
			s.y=cell.y+cell_side/2;

			if ((*curnode_)->label=="input") draw_square(surface, &cell, black, blue);
			else if ((*curnode_)->label=="output") draw_square(surface, &cell, black, red);
			else draw_square(surface, &cell, black, black);

			for (curlink_=(*curnode_)->incoming.begin(); curlink_!=(*curnode_)->incoming.end(); curlink_++) {
				g.y=(*curlink_)->node->y+cell_side/2;
				if (!(*curlink_)->enable) { // just a little light line for disable links
					s.x=cell.x+cell_side/2;
					g.x=(*curlink_)->node->x+cell_side/2;
					draw_line(surface,s,g,green);
					continue;
				}
				shifts=(int)(cell_side*(fabs((*curlink_)->weight))/(2.0*max_weight_link));
				while (shifts>=0) {
					s.x=cell.x+cell_side/3+shifts;
					g.x=(*curlink_)->node->x+cell_side/3+shifts;
					if ((*curlink_)->weight>0.0) draw_line(surface,s,g,black);
					else draw_line(surface,s,g,purple);
					shifts--;
				}
			}
		}
	}
}
