#ifndef _FightInputs
#define _FightInputs

#include <list>
#include <SDL/SDL.h>
#include "Neuron.h"
#include "Organism.h"

using namespace std;

// type for controls:
typedef int Control;

// and actions:
typedef int Action;
	
// structure with two actions and two rectangles representing (old) positions;
// and two bools to tell if a player went through a stair.
struct ActionsAndPositions {
	ActionsAndPositions () {}
	Action act1;
	Action act2;
	SDL_Rect r1;
	SDL_Rect r2;
	bool lvC1;
	bool lvC2;
};

// storage class, and handle inputs to get actions

class FightInputs {

public:

	static const int SCREEN_W=1024,SCREEN_H=768; // screen size
	static const double MIN_LEARNING=0.1; // minimal parameter for learning
	
	static const int GROUND=340; // coordinate of ground, to test returns to floor
	static const int STEP=20; // width of a fighter's step
	static const int FULL_JUMP=300; // height of a jump
	static const int FTH_JUMP=50;
	static const int STEP_DOWN=120;
	
	static const double D_PUNCH1_DAMAGE=0.5;
	static const double D_PUNCH2_DAMAGE=1.5;
	static const double PUNCH_DAMAGE=1.0;
	static const double A_PUNCH_DAMAGE=1.5;
	
	static const double D_KICK1_DAMAGE=0.5;
	static const double D_KICK2_DAMAGE=1.5;
	static const double KICK_DAMAGE=1.0;
	static const double A_KICK_DAMAGE=1.5;
	static const double REV_KICK_DAMAGE=1.0;
	static const double KNEE_HIT_DAMAGE=2.0;
	
	static const int EPSILON=10; // distance of hit
	static const double ACTIVATION_THRESHOLD=0.1; // minimal threshold to get active output
	
	// images' widths and heights:
	static int* tabWidthActs_down;
	static int* tabWidthActs;
	static int* tabWidthActs_air;
	static int* tabHeightActs_down;
	static int* tabHeightActs;
	static int* tabHeightActs_air;
	
	// characteristic distances of hits:
	static int* distX;

	static const int waitingW=142,waitingH=380;
	static const int guardW=158,guardH=397;
	static const int downW=196,downH=251;
	static const int downGuardW=178,downGuardH=274;
	static const int kneeW=230,kneeH=404;
	static const int punch1W=286,punch1H=358;
	static const int punch2W=245,punch2H=376;
	static const int downPunch1W=275,downPunch1H=281;
	static const int downPunch2W=182,downPunch2H=427;
	static const int airPunch1W=272,airPunch1H=222;
	static const int airPunch2W=261,airPunch2H=291;
	static const int kick1W=287,kick1H=407;
	static const int kick2W=284,kick2H=367;
	static const int downKick1W=271,downKick1H=231;
	static const int downKick2W=295,downKick2H=312;
	static const int airKickW=309,airKickH=320;
	static const int reverseKick1W=310,reverseKick1H=163;
	static const int reverseKick2W=249,reverseKick2H=395;
	static const int jumpW=118,jumpH=391;
	
	// controls:
	static const int NOTHING=-1;
	static const int LEFT=0;
	static const int RIGHT=1;
	static const int UP=2;
	static const int DOWN=3;
	static const int ZERO=4;
	static const int ONE=5;
	static const int FIVE=6;
	static const int TWO=7;
	static const int SIX=8;

	// actions:
	static const int UNDEFINED=0;
	static const int DIR_LEFT=1;
	static const int DIR_RIGHT=2;
	static const int DIR_UP=3;
	static const int DIR_UP_LEFT=4;
	static const int DIR_UP_RIGHT=5;
	static const int DIR_DOWN=6;
	static const int DIR_DOWN_LEFT=7;
	static const int DIR_DOWN_RIGHT=8;
	static const int GUARD=9;
	static const int PUNCH1=10;
	static const int PUNCH2=11;
	static const int KICK1=12;
	static const int KICK2=13;
	static const int REVERSE_KICK1=14;
	static const int REVERSE_KICK2=15;
	static const int KNEE_HIT=16;

	// must be called before inputs treatment:
	static void init_inputs();
	
	// get networks most active outputs:
	static void get_controls(Uint8* keys,Organism* org1,Organism* org2,int diffPosX_2_1,int diffPosY_2_1,
			Control& p1ct1,Control& p1ct2,Control& p2ct1,Control& p2ct2);
	
	static bool FALSE;
	static vector<Action> emptyVector;
	// just a function to transform inputs into actions:
	static ActionsAndPositions inputs_handler(Organism* org1,Organism* org2,double& life1,SDL_Rect& position1,double& life2,
		SDL_Rect& position2,bool& downPressed1=FALSE,bool& downPressed2=FALSE,vector<Action>& humanActions=emptyVector);
};

#define MIN_LEARNING FightInputs::MIN_LEARNING
#define GROUND FightInputs::GROUND
#define STEP_DOWN FightInputs::STEP_DOWN
#define init_inputs FightInputs::init_inputs
#define get_controls FightInputs::get_controls
#define inputs_handler FightInputs::inputs_handler
#define tabWidthActs_down FightInputs::tabWidthActs_down
#define tabWidthActs FightInputs::tabWidthActs
#define tabWidthActs_air FightInputs::tabWidthActs_air
#define tabHeightActs_down FightInputs::tabHeightActs_down
#define tabHeightActs FightInputs::tabHeightActs
#define tabHeightActs_air FightInputs::tabHeightActs_air
#define distX FightInputs::distX
#define FALSE FightInputs::FALSE

#endif
