#ifndef _Functions
#define _Functions

#include <list>
#include <cstdlib>
#include <cmath>
#include <ctime>
#include <iostream>

using namespace std;

class Functions {

public:

	Functions () {
		srand(time(NULL));
	}

	// sort generic non-empty list by insertion in place (not optimal, but easy for a first implementation)
	template<class Tp>
	static void sortList(list<Tp*>& lt) {
		typename list<Tp*>::iterator iter1=lt.begin(),iter2=lt.begin(),aux;
		iter2++;
		while (iter2!=lt.end()) {
			if (*(*iter2) < *(*iter1)) {
				// go back in the list till we find an element >= *iter2,
				// or the beginning of the list.
				aux=iter1;
				while (aux!=lt.begin() && *(*iter2) < *(*aux)) aux--;
				if (aux!=lt.begin()) aux++;
				lt.insert(aux,*iter2);
				lt.erase(iter2);
				iter2=iter1;
				iter2++;
			}
			else {
				iter1++;
				iter2++;
			}
		}
		return;
	}
	
	static int maxm(int a,int b) {
		if (a>b) return a;
		else return b;
	}
	static float maxf(float a,float b) {
		if (a>b) return a;
		else return b;
	}
	static int minm(int a,int b) {
		if (a<b) return a;
		else return b;
	}
	static float minf(float a,float b) {
		if (a<b) return a;
		else return b;
	}
	
	/* Inline Random Functions */
	static bool randbit() {
		return rand()%2;
	}
	static int randposneg() {
		if (rand()%2) return 1; 
		else return -1;
	}
	static int randint(int x,int y) {
		return rand()%(y-x+1)+x;
	}
	static double randfloat() {
		return (double)rand()/RAND_MAX;
	}

	//Returns a normally distributed deviate with 0 mean and unit variance
	//Algorithm is from Numerical Recipes in C, Second Edition
	static double gaussrand() {
		static int iset=0;
		static double gset;
		double fac,rsq,v1,v2;

		if (iset==0) {
			do {
				v1=2.0*(double)rand()/RAND_MAX-1.0;
				v2=2.0*(double)rand()/RAND_MAX-1.0;
				rsq=v1*v1+v2*v2;
			}
			while (rsq>=1.0 || rsq<=0.0);
			fac=sqrt(-2.0*log(rsq)/rsq);
			gset=v1*fac;
			iset=1;
			return v2*fac;
		}
		else {
			iset=0;
			return gset;
		}
	}

	//This is an incorrect pseudo-gaussian distribution, 
	// faster than gaussrand and seems adapted to the reproduction phase
	static double p_gaussrand() {
		return (randposneg())*(sqrt(-log((double)rand()/RAND_MAX)));
	}

	// SIGMOID FUNCTION ********************************
	//This is a sigmoidal activation function, which is an S-shaped squashing function
	//It smoothly limits the amplitude of the output of a neuron to between 0 and 1
	//It is made inline so it can execute quickly since it is at every non-sensor 
	// node in a network.
	//NOTE:  In order to make node insertion in the middle of a link,
	// the sigmoid can be shifted to the right and more steeply sloped:
	// slope=4.924273
	// constant=-2.4621365
	// These parameters optimize mean squared error between the old output,
	// and an output of a node inserted in the middle of a link between
	// the old output and some other node. 
	// When not right-shifted, the steepened slope is closest to a linear
	// ascent as possible between 0 and 1

	~Functions () {}

};

#define sortList Functions::sortList
#define maxm Functions::maxm
#define maxf Functions::maxf
#define minm Functions::minm
#define minf Functions::minf
#define randbit Functions::randbit
#define randposneg Functions::randposneg
#define randint Functions::randint
#define randfloat Functions::randfloat
#define gaussrand Functions::gaussrand
#define p_gaussrand Functions::p_gaussrand

#endif
