#include "Mate.h"

/* This method mates this Organism with another Organism g.
     For every point in each Organism, where each Organism shares
     the innovation number, the Gene is chosen randomly from
     either parent.  If one parent has an innovation absent in
     the other, the baby may inherit the innovation
     if it is from the more fit parent.*/
Organism* mate_multipoint(const Organism* org1,const Organism* org2) {
	Organism* baby=new Organism(); // the new organism
				// we have to initialize his hidden neurons and innovations
				// fitness will be set elsewhere
	vector<int> innovLink,innovNode;
	list<Neuron*>::const_iterator curnode,curnode1,curnode2;
	list<IncomingLink*>::const_iterator curlink;
	// tells if genes from one organism should be chosen:
	bool org1_isBetter=false,org2_isBetter=false,org1_isSmaller=true,org2_isSmaller=true;
	
	//preprocessing: array of new neurons indexed by id's of the parents
	// highest ID's are typically find in the hidden layers;
	// if no hidden layer, we go through outputs neurons
	int sizeID=0;
	bool hdn=false;
	if (!org1->hidden_nodes.empty()) {
		hdn=true;
		for (curnode=org1->hidden_nodes.begin(); curnode!=org1->hidden_nodes.end(); curnode++)
			if ((*curnode)->id>sizeID) sizeID=(*curnode)->id;
	}
	if (!org2->hidden_nodes.empty()) {
		hdn=true;
		for (curnode=org2->hidden_nodes.begin(); curnode!=org2->hidden_nodes.end(); curnode++)
			if ((*curnode)->id>sizeID) sizeID=(*curnode)->id;
	}
	if (!hdn) sizeID=org1->inputs.size()+org1->outputs.size()-1;

	Neuron** tabNodes=new Neuron*[sizeID+1];
	bool* dupHidNodes=(bool*)calloc(sizeID+1,sizeof(bool)); // mark the nodes encountered twice in the hidden layers
	bool* nodes=(bool*)calloc(sizeID+1,sizeof(bool)); // mark all the nodes inside the organism;
														// to avoid "ghost" incoming nodes
	// duplication phase
	for (curnode=org1->inputs.begin(); curnode!=org1->inputs.end(); curnode++) {
		tabNodes[(*curnode)->id]=new Neuron(*curnode);
		baby->inputs.push_back(tabNodes[(*curnode)->id]);
		nodes[(*curnode)->id]=true;
	}
	for (curnode=org1->outputs.begin(); curnode!=org1->outputs.end(); curnode++) {
		tabNodes[(*curnode)->id]=new Neuron(*curnode);
		nodes[(*curnode)->id]=true;
	}
	for (curnode=org1->hidden_nodes.begin(); curnode!=org1->hidden_nodes.end(); curnode++) {
		tabNodes[(*curnode)->id]=new Neuron(*curnode);
		dupHidNodes[(*curnode)->id]=true;
	}
	for (curnode=org2->hidden_nodes.begin(); curnode!=org2->hidden_nodes.end(); curnode++) {
		if (!dupHidNodes[(*curnode)->id]) tabNodes[(*curnode)->id]=new Neuron(*curnode);
	}
	free(dupHidNodes);

	//Figure out which genome is better
	//The worse genome should not be allowed to add extra structural baggage
	//If they are the same, use the smaller one's disjoint and excess genes only
	// (principle of Occam's razor)
	if (org1->orig_fitness>org2->orig_fitness) org1_isBetter=true;
	else if (org1->orig_fitness==org2->orig_fitness) {
		if (org1->hidden_nodes.size() < org2->hidden_nodes.size()) org2_isSmaller=false;
		else if (org1->hidden_nodes.size() > org2->hidden_nodes.size()) org1_isSmaller=false;
	}
	else org2_isBetter=true;

	//Now move through the Genes of each parent until both genomes end;
	//neurons and incoming links are order by increasing id, so it's relatively easy
	curnode1=org1->hidden_nodes.begin();
	curnode2=org2->hidden_nodes.begin();
	while (curnode1!=org1->hidden_nodes.end() || curnode2!=org2->hidden_nodes.end()) {
		if (curnode1==org1->hidden_nodes.end()) {
			if (org1_isBetter) break;  //Skip excess from the worse genome
			if (org2_isBetter || org2_isSmaller) {
				if (global_node_innovation[(*curnode2)->id]!=0) innovNode.push_back((*curnode2)->id);
				for (curlink=(*curnode2)->incoming.begin(); curlink!=(*curnode2)->incoming.end(); curlink++) {
					// if a link is disabled, it has a little chance to be enabled for the child
					if (!(*curlink)->enable && randfloat()<0.25)
						tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,true);
					else tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
					if ((*curlink)->innovation_num>=0) innovLink.push_back((*curlink)->innovation_num);
				}
				baby->hidden_nodes.push_back(tabNodes[(*curnode2)->id]);
				baby->nb_links+=(*curnode2)->incoming.size();
				nodes[(*curnode2)->id]=true;
			}
			curnode2++;
		}
		else if (curnode2==org2->hidden_nodes.end()) {
			if (org2_isBetter) break;  //Skip excess from the worse genome
			if (org1_isBetter || org1_isSmaller) {
				if (global_node_innovation[(*curnode1)->id]!=0) innovNode.push_back((*curnode1)->id);
				for (curlink=(*curnode1)->incoming.begin(); curlink!=(*curnode1)->incoming.end(); curlink++) {
					// if a link is disabled, it has a little chance to be enabled for the child
					if (!(*curlink)->enable && randfloat()<0.25)
						tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,true);
					else tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
					if ((*curlink)->innovation_num>=0) innovLink.push_back((*curlink)->innovation_num);
				}
				baby->hidden_nodes.push_back(tabNodes[(*curnode1)->id]);
				baby->nb_links+=(*curnode1)->incoming.size();
				nodes[(*curnode1)->id]=true;
			}
			curnode1++;
		}
		else {
			//two cases: features in both organisms, or just in one
			if ((*curnode1)->id==(*curnode2)->id) {
				// trial to add the best:
				if (org1_isBetter) {
					if (global_node_innovation[(*curnode1)->id]!=0) innovNode.push_back((*curnode1)->id);
					for (curlink=(*curnode1)->incoming.begin(); curlink!=(*curnode1)->incoming.end(); curlink++) {
						// if a link is disabled, it has a little chance to be enabled for the child
						if (!(*curlink)->enable && randfloat()<0.25)
							tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,true);
						else tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
						if ((*curlink)->innovation_num>=0) innovLink.push_back((*curlink)->innovation_num);
					}
					baby->hidden_nodes.push_back(tabNodes[(*curnode1)->id]);
					baby->nb_links+=(*curnode1)->incoming.size();
					nodes[(*curnode1)->id]=true;
				}
				else if (org2_isBetter) {
					if (global_node_innovation[(*curnode2)->id]!=0) innovNode.push_back((*curnode2)->id);
					for (curlink=(*curnode2)->incoming.begin(); curlink!=(*curnode2)->incoming.end(); curlink++) {
						// if a link is disabled, it has a little chance to be enabled for the child
						if (!(*curlink)->enable && randfloat()<0.25)
							tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,true);
						else tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
						if ((*curlink)->innovation_num>=0) innovLink.push_back((*curlink)->innovation_num);
					}
					baby->hidden_nodes.push_back(tabNodes[(*curnode2)->id]);
					baby->nb_links+=(*curnode2)->incoming.size();
					nodes[(*curnode2)->id]=true;
				}
				else {
					// choose at random
					if (randbit()) {
						if (global_node_innovation[(*curnode1)->id]!=0) innovNode.push_back((*curnode1)->id);
						for (curlink=(*curnode1)->incoming.begin(); curlink!=(*curnode1)->incoming.end(); curlink++) {
							// if a link is disabled, it has a little chance to be enabled for the child
							if (!(*curlink)->enable && randfloat()<0.25)
								tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,true);
							else tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
							if ((*curlink)->innovation_num>=0) innovLink.push_back((*curlink)->innovation_num);
						}
						baby->hidden_nodes.push_back(tabNodes[(*curnode1)->id]);
						baby->nb_links+=(*curnode1)->incoming.size();
						nodes[(*curnode1)->id]=true;
					}
					else {
						if (global_node_innovation[(*curnode2)->id]!=0) innovNode.push_back((*curnode2)->id);
						for (curlink=(*curnode2)->incoming.begin(); curlink!=(*curnode2)->incoming.end(); curlink++) {
							// if a link is disabled, it has a little chance to be enabled for the child
							if (!(*curlink)->enable && randfloat()<0.25)
								tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,true);
							else tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
							if ((*curlink)->innovation_num>=0) innovLink.push_back((*curlink)->innovation_num);
						}
						baby->hidden_nodes.push_back(tabNodes[(*curnode2)->id]);
						baby->nb_links+=(*curnode2)->incoming.size();
						nodes[(*curnode2)->id]=true;
					}
				}
				curnode1++;
				curnode2++;
			}
			else if ((*curnode1)->id<(*curnode2)->id) {
				if (org1_isBetter || org1_isSmaller) {
					if (global_node_innovation[(*curnode1)->id]!=0) innovNode.push_back((*curnode1)->id);
					for (curlink=(*curnode1)->incoming.begin(); curlink!=(*curnode1)->incoming.end(); curlink++) {
						// if a link is disabled, it has a little chance to be enabled for the child
						if (!(*curlink)->enable && randfloat()<0.25)
							tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,true);
						else tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
						if ((*curlink)->innovation_num>=0) innovLink.push_back((*curlink)->innovation_num);
					}
					baby->hidden_nodes.push_back(tabNodes[(*curnode1)->id]);
					baby->nb_links+=(*curnode1)->incoming.size();
					nodes[(*curnode1)->id]=true;
				}
				curnode1++;
			}
			else {
				if (org2_isBetter || org2_isSmaller) {
					if (global_node_innovation[(*curnode2)->id]!=0) innovNode.push_back((*curnode2)->id);
					for (curlink=(*curnode2)->incoming.begin(); curlink!=(*curnode2)->incoming.end(); curlink++) {
						// if a link is disabled, it has a little chance to be enabled for the child
						if (!(*curlink)->enable && randfloat()<0.25)
							tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,true);
						else tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
						if ((*curlink)->innovation_num>=0) innovLink.push_back((*curlink)->innovation_num);
					}
					baby->hidden_nodes.push_back(tabNodes[(*curnode2)->id]);
					baby->nb_links+=(*curnode2)->incoming.size();
					nodes[(*curnode2)->id]=true;
				}
				curnode2++;
			}
		}
	}
	// then look at the output nodes
	curnode1=org1->outputs.begin();
	curnode2=org2->outputs.begin();
	while (curnode1!=org1->outputs.end()) {
		// trial to add the best:
		if (org1_isBetter) {
			for (curlink=(*curnode1)->incoming.begin(); curlink!=(*curnode1)->incoming.end(); curlink++) {
				// if a link is disabled, it has a little chance to be enabled for the child
				if (!(*curlink)->enable && randfloat()<0.25)
					tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,true);
				else tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
				if ((*curlink)->innovation_num>=0) innovLink.push_back((*curlink)->innovation_num);
			}
			baby->outputs.push_back(tabNodes[(*curnode1)->id]);
			baby->nb_links+=(*curnode1)->incoming.size();
		}
		else if (org2_isBetter) {
			for (curlink=(*curnode2)->incoming.begin(); curlink!=(*curnode2)->incoming.end(); curlink++) {
				// if a link is disabled, it has a little chance to be enabled for the child
				if (!(*curlink)->enable && randfloat()<0.25)
					tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,true);
				else tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
				if ((*curlink)->innovation_num>=0) innovLink.push_back((*curlink)->innovation_num);
			}
			baby->outputs.push_back(tabNodes[(*curnode2)->id]);
			baby->nb_links+=(*curnode2)->incoming.size();
		}
		else {
			// choose at random
			if (randbit()) {
				for (curlink=(*curnode1)->incoming.begin(); curlink!=(*curnode1)->incoming.end(); curlink++) {
					// if a link is disabled, it has a little chance to be enabled for the child
					if (!(*curlink)->enable && randfloat()<0.25)
						tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,true);
					else tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
					if ((*curlink)->innovation_num>=0) innovLink.push_back((*curlink)->innovation_num);
				}
				baby->outputs.push_back(tabNodes[(*curnode1)->id]);
				baby->nb_links+=(*curnode1)->incoming.size();
			}
			else {
				for (curlink=(*curnode2)->incoming.begin(); curlink!=(*curnode2)->incoming.end(); curlink++) {
					// if a link is disabled, it has a little chance to be enabled for the child
					if (!(*curlink)->enable && randfloat()<0.25)
						tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,true);
					else tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
					if ((*curlink)->innovation_num>=0) innovLink.push_back((*curlink)->innovation_num);
				}
				baby->outputs.push_back(tabNodes[(*curnode2)->id]);
				baby->nb_links+=(*curnode2)->incoming.size();
			}
		}
		curnode1++;
		curnode2++;
	}

	list<Neuron*>::iterator curnode_;
	list<IncomingLink*>::iterator curlink_,deadlink;
	// we now have to erase the incomings to "ghost" nodes:
	for (curnode_=baby->hidden_nodes.begin(); curnode_!=baby->hidden_nodes.end(); curnode_++) {
		curlink_=(*curnode_)->incoming.begin();
		while (curlink_!=(*curnode_)->incoming.end()) {
			if (!nodes[(*curlink_)->node->id]) {
				deadlink=curlink_;
				curlink_++;
				(*curnode_)->incoming.erase(deadlink);
			}
			else curlink_++;
		}
	}
	for (curnode_=baby->outputs.begin(); curnode_!=baby->outputs.end(); curnode_++) {
		curlink_=(*curnode_)->incoming.begin();
		while (curlink_!=(*curnode_)->incoming.end()) {
			if (!nodes[(*curlink_)->node->id]) {
				deadlink=curlink_;
				curlink_++;
				(*curnode_)->incoming.erase(deadlink);
			}
			else curlink_++;
		}
	}

	// finally, look through all innovations, and add them to the lists
	sort(innovLink.begin(),innovLink.end());
	for (int i=innovLink.size()-1; i>=0; i--) {
		if (!nodes[global_link_innovation[innovLink[i]]->incnode->id]) continue;
		baby->link_innovations.push_front(global_link_innovation[innovLink[i]]);
	}
	free(nodes);
	sort(innovNode.begin(),innovNode.end());
	for (int i=innovNode.size()-1; i>=0; i--)
		baby->node_innovations.push_front(global_node_innovation[innovNode[i]]);

	return baby;
}

/* This is like mate_multipoint except it averages at each
   matching gene instead of choosing one or the other */
Organism* mate_multipoint_avg(const Organism* org1,const Organism* org2) {
	Organism* baby=new Organism(); // the new organism
				// we have to initialize his hidden neurons and innovations
				// fitness will be set elsewhere
	vector<int> innovLink,innovNode;
	list<Neuron*>::const_iterator curnode,curnode1,curnode2;
	list<IncomingLink*>::const_iterator curlink,curlink1,curlink2;
	// tells if genes from one organism should be chosen
	bool org1_isBetter=false,org2_isBetter=false,org1_isSmaller=true,org2_isSmaller=true;
	
	//preprocessing: array of new neurons indexed by id's of the parents
	// highest ID's are typically find in the hidden layers;
	// if no hidden layer, we go through outputs neurons
	int sizeID=0;
	bool hdn=false;
	if (!org1->hidden_nodes.empty()) {
		hdn=true;
		for (curnode=org1->hidden_nodes.begin(); curnode!=org1->hidden_nodes.end(); curnode++)
			if ((*curnode)->id>sizeID) sizeID=(*curnode)->id;
	}
	if (!org2->hidden_nodes.empty()) {
		hdn=true;
		for (curnode=org2->hidden_nodes.begin(); curnode!=org2->hidden_nodes.end(); curnode++)
			if ((*curnode)->id>sizeID) sizeID=(*curnode)->id;
	}
	if (!hdn) sizeID=org1->inputs.size()+org1->outputs.size()-1;

	Neuron** tabNodes=new Neuron*[sizeID+1];
	bool* dupHidNodes=(bool*)calloc(sizeID+1,sizeof(bool)); // mark the nodes encountered twice in the hidden layers
	bool* nodes=(bool*)calloc(sizeID+1,sizeof(bool)); // mark all the nodes inside the organism;
														// to avoid "ghost" incoming nodes
	// duplication phase
	for (curnode=org1->inputs.begin(); curnode!=org1->inputs.end(); curnode++) {
		tabNodes[(*curnode)->id]=new Neuron(*curnode);
		baby->inputs.push_back(tabNodes[(*curnode)->id]);
		nodes[(*curnode)->id]=true;
	}
	for (curnode=org1->outputs.begin(); curnode!=org1->outputs.end(); curnode++) {
		tabNodes[(*curnode)->id]=new Neuron(*curnode);
		nodes[(*curnode)->id]=true;
	}
	for (curnode=org1->hidden_nodes.begin(); curnode!=org1->hidden_nodes.end(); curnode++) {
		tabNodes[(*curnode)->id]=new Neuron(*curnode);
		dupHidNodes[(*curnode)->id]=true;
	}
	for (curnode=org2->hidden_nodes.begin(); curnode!=org2->hidden_nodes.end(); curnode++) {
		if (!dupHidNodes[(*curnode)->id]) tabNodes[(*curnode)->id]=new Neuron(*curnode);
	}
	free(dupHidNodes);

	//Figure out which genome is better
	//The worse genome should not be allowed to add extra structural baggage
	//If they are the same, use the smaller one's disjoint and excess genes only
	// (principle of Occam's razor)
	if (org1->orig_fitness>org2->orig_fitness) org1_isBetter=true;
	else if (org1->orig_fitness==org2->orig_fitness) {
		if (org1->hidden_nodes.size() < org2->hidden_nodes.size()) org2_isSmaller=false;
		else if (org1->hidden_nodes.size() > org2->hidden_nodes.size()) org1_isSmaller=false;
	}
	else org2_isBetter=true;

	// we still bias toward better organisms
	// NOTE: all those biases are not natural, and we could see what happens
	// if we let "natural biological process" happens..
	double thisCoeff=1.0,orgCoeff=1.0;
	if (org1_isBetter) thisCoeff+=(org1->orig_fitness-org2->orig_fitness);
	else if (org2_isBetter) orgCoeff+=(org2->orig_fitness-org1->orig_fitness);

	//Now move through the Genes of each parent until both genomes end;
	//neurons and incoming links are order by increasing id, so it's relatively easy
	curnode1=org1->hidden_nodes.begin();
	curnode2=org2->hidden_nodes.begin();
	while (curnode1!=org1->hidden_nodes.end() || curnode2!=org2->hidden_nodes.end()) {
		if (curnode1==org1->hidden_nodes.end()) {
			if (org1_isBetter) break; //Skip excess from the worse genome
			if (org2_isBetter || org2_isSmaller) {
				if (global_node_innovation[(*curnode2)->id]!=0) innovNode.push_back((*curnode2)->id);
				for (curlink=(*curnode2)->incoming.begin(); curlink!=(*curnode2)->incoming.end(); curlink++) {
					// if a link is disabled, it has a little chance to be enabled for the child
					if (!(*curlink)->enable && randfloat()<0.25)
						tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,true);
					else tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
					if ((*curlink)->innovation_num>=0) innovLink.push_back((*curlink)->innovation_num);
				}
				baby->hidden_nodes.push_back(tabNodes[(*curnode2)->id]);
				baby->nb_links+=(*curnode2)->incoming.size();
				nodes[(*curnode2)->id]=true;
			}
			curnode2++;
		}
		else if (curnode2==org2->hidden_nodes.end()) {
			if (org2_isBetter) break; //Skip excess from the worse genome
			if (org1_isBetter || org1_isSmaller) {
				if (global_node_innovation[(*curnode1)->id]!=0) innovNode.push_back((*curnode1)->id);
				for (curlink=(*curnode1)->incoming.begin(); curlink!=(*curnode1)->incoming.end(); curlink++) {
					// if a link is disabled, it has a little chance to be enabled for the child
					if (!(*curlink)->enable && randfloat()<0.25)
						tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,true);
					else tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
					if ((*curlink)->innovation_num>=0) innovLink.push_back((*curlink)->innovation_num);
				}
				baby->hidden_nodes.push_back(tabNodes[(*curnode1)->id]);
				baby->nb_links+=(*curnode1)->incoming.size();
				nodes[(*curnode1)->id]=true;
			}
			curnode1++;
		}
		else {
			//two cases: features in both organisms, or just in one
			if ((*curnode1)->id==(*curnode2)->id) {
				// average the features, so it's more subtile (the only difference);
				if (global_node_innovation[(*curnode1)->id]!=0) innovNode.push_back((*curnode1)->id);
				curlink1=(*curnode1)->incoming.begin();
				curlink2=(*curnode2)->incoming.begin();
				while (curlink1!=(*curnode1)->incoming.end() || curlink2!=(*curnode2)->incoming.end()) {
					if (curlink1==(*curnode1)->incoming.end()) {
						if (org1_isBetter) break; // skip the worse organism (Occam's razor)
						if (org2_isBetter || org2_isSmaller) {
							// if a link is disabled, it has a little chance to be enabled for the child
							if (!(*curlink2)->enable && randfloat()<0.25)
								tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink2)->node->id],(*curlink2)->weight,(*curlink2)->innovation_num,true);
							else tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink2)->node->id],(*curlink2)->weight,(*curlink2)->innovation_num,(*curlink2)->enable);
							baby->nb_links++;
							if ((*curlink2)->innovation_num>=0) innovLink.push_back((*curlink2)->innovation_num);
						}
						curlink2++;
					}
					else if (curlink2==(*curnode2)->incoming.end()) {
						if (org2_isBetter) break; // skip the worse organism (Occam's razor)
						if (org1_isBetter || org1_isSmaller) {
							// if a link is disabled, it has a little chance to be enabled for the child
							if (!(*curlink1)->enable && randfloat()<0.25)
								tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink1)->node->id],(*curlink1)->weight,(*curlink1)->innovation_num,true);
							else tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink1)->node->id],(*curlink1)->weight,(*curlink1)->innovation_num,(*curlink1)->enable);
							baby->nb_links++;
							if ((*curlink1)->innovation_num>=0) innovLink.push_back((*curlink1)->innovation_num);
						}
						curlink1++;
					}
					else {
						// two cases: the links share the same innovation number, or not
						if ((*curlink1)->innovation_num<(*curlink2)->innovation_num) {
							if (org1_isBetter || org1_isSmaller) {
								// if a link is disabled, it has a little chance to be enabled for the child
								if (!(*curlink1)->enable && randfloat()<0.25)
									tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink1)->node->id],(*curlink1)->weight,(*curlink1)->innovation_num,true);
								else tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink1)->node->id],(*curlink1)->weight,(*curlink1)->innovation_num,(*curlink1)->enable);
								baby->nb_links++;
								if ((*curlink1)->innovation_num>=0) innovLink.push_back((*curlink1)->innovation_num);
							}
							curlink1++;
						}
						else if ((*curlink1)->innovation_num>(*curlink2)->innovation_num) {
							if (org2_isBetter || org2_isSmaller) {
								// if a link is disabled, it has a little chance to be enabled for the child
								if (!(*curlink2)->enable && randfloat()<0.25)
									tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink2)->node->id],(*curlink2)->weight,(*curlink2)->innovation_num,true);
								else tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink2)->node->id],(*curlink2)->weight,(*curlink2)->innovation_num,(*curlink2)->enable);
								baby->nb_links++;
								if ((*curlink2)->innovation_num>=0) innovLink.push_back((*curlink2)->innovation_num);
							}
							curlink2++;
						}
						else {
							// if a link is disabled for both parents, it has a little chance to be enabled for the child
							// if present in only one parent, more chances to get the gene ..etc
							if ((!(*curlink1)->enable && !(*curlink2)->enable && randfloat()<0.25)
								|| ((!(*curlink1)->enable || !(*curlink2)->enable) && randfloat()<0.6))
								tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink1)->node->id],
								(thisCoeff*(*curlink1)->weight+orgCoeff*(*curlink2)->weight)/(thisCoeff+orgCoeff),(*curlink1)->innovation_num);
							else tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink1)->node->id],
								(thisCoeff*(*curlink1)->weight+orgCoeff*(*curlink2)->weight)/(thisCoeff+orgCoeff),(*curlink1)->innovation_num,(*curlink1)->enable);
							baby->nb_links++;
							if ((*curlink1)->innovation_num>=0) innovLink.push_back((*curlink1)->innovation_num);
							curlink1++;
							curlink2++;
						}
					}
				}
				// finally, avergage the thresholds:
				tabNodes[(*curnode1)->id]->threshold=(thisCoeff*((*curnode1)->threshold)+orgCoeff*((*curnode2)->threshold))/(thisCoeff+orgCoeff);
				baby->hidden_nodes.push_back(tabNodes[(*curnode1)->id]);
				nodes[(*curnode1)->id]=true;
				curnode1++;
				curnode2++;
			}
			else if ((*curnode1)->id<(*curnode2)->id) {
				if (org1_isBetter || org1_isSmaller) {
					if (global_node_innovation[(*curnode1)->id]!=0) innovNode.push_back((*curnode1)->id);
					for (curlink=(*curnode1)->incoming.begin(); curlink!=(*curnode1)->incoming.end(); curlink++) {
						// if a link is disabled, it has a little chance to be enabled for the child
						if (!(*curlink)->enable && randfloat()<0.25)
							tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,true);
						else tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
						if ((*curlink)->innovation_num>=0) innovLink.push_back((*curlink)->innovation_num);
					}
					baby->hidden_nodes.push_back(tabNodes[(*curnode1)->id]);
					baby->nb_links+=(*curnode1)->incoming.size();
					nodes[(*curnode1)->id]=true;
				}
				curnode1++;
			}
			else {
				if (org2_isBetter || org2_isSmaller) {
					if (global_node_innovation[(*curnode2)->id]!=0) innovNode.push_back((*curnode2)->id);
					for (curlink=(*curnode2)->incoming.begin(); curlink!=(*curnode2)->incoming.end(); curlink++) {
						// if a link is disabled, it has a little chance to be enabled for the child
						if (!(*curlink)->enable && randfloat()<0.25)
							tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,true);
						else tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
						if ((*curlink)->innovation_num>=0) innovLink.push_back((*curlink)->innovation_num);
					}
					baby->hidden_nodes.push_back(tabNodes[(*curnode2)->id]);
					baby->nb_links+=(*curnode2)->incoming.size();
					nodes[(*curnode2)->id]=true;
				}
				curnode2++;
			}
		}
	}
	// then look at the output nodes
	curnode1=org1->outputs.begin();
	curnode2=org2->outputs.begin();
	while (curnode1!=org1->outputs.end()) {
		// average the features, so it's more subtile (the only difference):
		curlink1=(*curnode1)->incoming.begin();
		curlink2=(*curnode2)->incoming.begin();
		while (curlink1!=(*curnode1)->incoming.end() || curlink2!=(*curnode2)->incoming.end()) {
			if (curlink1==(*curnode1)->incoming.end()) {
				if (org1_isBetter) break; // skip the worse organism (Occam's razor)
				if (org2_isBetter || org2_isSmaller) {
					// if a link is disabled, it has a little chance to be enabled for the child
					if (!(*curlink2)->enable && randfloat()<0.25)
						tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink2)->node->id],(*curlink2)->weight,(*curlink2)->innovation_num,true);
					else tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink2)->node->id],(*curlink2)->weight,(*curlink2)->innovation_num,(*curlink2)->enable);
					baby->nb_links++;
					if ((*curlink2)->innovation_num>=0) innovLink.push_back((*curlink2)->innovation_num);
				}
				curlink2++;
			}
			else if (curlink2==(*curnode2)->incoming.end()) {
				if (org2_isBetter) break; // skip the worse organism (Occam's razor)
				if (org1_isBetter || org1_isSmaller) {
					// if a link is disabled, it has a little chance to be enabled for the child
					if (!(*curlink1)->enable && randfloat()<0.25)
						tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink1)->node->id],(*curlink1)->weight,(*curlink1)->innovation_num,true);
					else tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink1)->node->id],(*curlink1)->weight,(*curlink1)->innovation_num,(*curlink1)->enable);
					baby->nb_links++;
					if ((*curlink1)->innovation_num>=0) innovLink.push_back((*curlink1)->innovation_num);
				}
				curlink1++;
			}
			else {
				// two cases: the links point the same node, or not;
				// here we can't just look at innovation numbers because two original links have the same (-1)
				if ((*curlink1)->node->id<(*curlink2)->node->id) {
					if (org1_isBetter || org1_isSmaller) {
						// if a link is disabled, it has a little chance to be enabled for the child
						if (!(*curlink1)->enable && randfloat()<0.25)
							tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink1)->node->id],(*curlink1)->weight,(*curlink1)->innovation_num,true);
						else tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink1)->node->id],(*curlink1)->weight,(*curlink1)->innovation_num,(*curlink1)->enable);
						baby->nb_links++;
						if ((*curlink1)->innovation_num>=0) innovLink.push_back((*curlink1)->innovation_num);
					}
					curlink1++;
				}
				else if ((*curlink1)->node->id>(*curlink2)->node->id) {
					if (org2_isBetter || org2_isSmaller) {
						// if a link is disabled, it has a little chance to be enabled for the child
						if (!(*curlink2)->enable && randfloat()<0.25)
							tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink2)->node->id],(*curlink2)->weight,(*curlink2)->innovation_num,true);
						else tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink2)->node->id],(*curlink2)->weight,(*curlink2)->innovation_num,(*curlink2)->enable);
						baby->nb_links++;
						if ((*curlink2)->innovation_num>=0) innovLink.push_back((*curlink2)->innovation_num);
					}
					curlink2++;
				}
				else {
					// if a link is disabled for both parents, it has a little chance to be enabled for the child
					// if present in only one parent, more chances to get the gene ..etc
					if ((!(*curlink1)->enable && !(*curlink2)->enable && randfloat()<0.25)
						|| ((!(*curlink1)->enable || !(*curlink2)->enable) && randfloat()<0.6))
						tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink1)->node->id],
						(thisCoeff*(*curlink1)->weight+orgCoeff*(*curlink2)->weight)/(thisCoeff+orgCoeff),(*curlink1)->innovation_num,true);
					else tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink1)->node->id],
						(thisCoeff*(*curlink1)->weight+orgCoeff*(*curlink2)->weight)/(thisCoeff+orgCoeff),(*curlink1)->innovation_num,(*curlink1)->enable);
					baby->nb_links++;
					if ((*curlink1)->innovation_num>=0) innovLink.push_back((*curlink1)->innovation_num);
					curlink1++;
					curlink2++;
				}
			}
		}
		// finally, avergage the thresholds:
		tabNodes[(*curnode1)->id]->threshold=(thisCoeff*((*curnode1)->threshold)+orgCoeff*((*curnode2)->threshold))/(thisCoeff+orgCoeff);
		baby->outputs.push_back(tabNodes[(*curnode1)->id]);
		curnode1++;
		curnode2++;
	}

	list<Neuron*>::iterator curnode_;
	list<IncomingLink*>::iterator curlink_,deadlink;
	// we now have to erase the incomings to "ghost" nodes:
	for (curnode_=baby->hidden_nodes.begin(); curnode_!=baby->hidden_nodes.end(); curnode_++) {
		curlink_=(*curnode_)->incoming.begin();
		while (curlink_!=(*curnode_)->incoming.end()) {
			if (!nodes[(*curlink_)->node->id]) {
				deadlink=curlink_;
				curlink_++;
				(*curnode_)->incoming.erase(deadlink);
			}
			else curlink_++;
		}
	}
	for (curnode_=baby->outputs.begin(); curnode_!=baby->outputs.end(); curnode_++) {
		curlink_=(*curnode_)->incoming.begin();
		while (curlink_!=(*curnode_)->incoming.end()) {
			if (!nodes[(*curlink_)->node->id]) {
				deadlink=curlink_;
				curlink_++;
				(*curnode_)->incoming.erase(deadlink);
			}
			else curlink_++;
		}
	}

	// finally, look through all innovations, and add them to the lists
	sort(innovLink.begin(),innovLink.end());
	for (int i=innovLink.size()-1; i>=0; i--) {
		if (!nodes[global_link_innovation[innovLink[i]]->incnode->id]) continue;
		baby->link_innovations.push_front(global_link_innovation[innovLink[i]]);
	}
	free(nodes);
	sort(innovNode.begin(),innovNode.end());
	for (int i=innovNode.size()-1; i>=0; i--)
		baby->node_innovations.push_front(global_node_innovation[innovNode[i]]);

	return baby;
}

/* This method is similar to a standard single point crossover
     operator. Weights are averaged as in the previous mating
     method. A node id is chosen in the range of the organism's ids.
     We begin with the smaller organism, and when the node id is reached
	 we go on with the other organism */
Organism* mate_singlepoint(const Organism* org1,const Organism* org2) {
	Organism* baby=new Organism(); // the new organism
				// we have to initialize his hidden neurons and innovations;
				// fitness will be set elsewhere
	vector<int> innovLink,innovNode;
	list<Neuron*>::const_iterator curnode,curnode1,curnode2;
	list<IncomingLink*>::const_iterator curlink;
	// tells if genes from one organism should be chosen
	bool org1_isBetter=false,org2_isBetter=false,org1_isSmaller=true,org2_isSmaller=true;

	//preprocessing: array of new neurons indexed by id's of the parents
	// NOTE: maybe this could be improved with a global array..
	// highest ID's are typically find in the hidden layers;
	// if no hidden layer, we go through outputs neurons
	int sizeID1=0,sizeID2=0;
	if (!org1->hidden_nodes.empty()) {
		for (curnode=org1->hidden_nodes.begin(); curnode!=org1->hidden_nodes.end(); curnode++)
			if ((*curnode)->id>sizeID1) sizeID1=(*curnode)->id;
	}
	else sizeID1=org1->inputs.size()+org1->outputs.size()-1;
	if (!org2->hidden_nodes.empty()) {
		for (curnode=org2->hidden_nodes.begin(); curnode!=org2->hidden_nodes.end(); curnode++)
			if ((*curnode)->id>sizeID2) sizeID2=(*curnode)->id;
	}
	else sizeID2=org2->inputs.size()+org2->outputs.size()-1;
	
	int sizeID=maxm(sizeID1,sizeID2);
	Neuron** tabNodes=new Neuron*[sizeID+1];
	bool* dupHidNodes=(bool*)calloc(sizeID+1,sizeof(bool)); // mark the nodes encountered twice in the hidden layers
	bool* nodes=(bool*)calloc(sizeID+1,sizeof(bool)); // mark all the nodes inside the organism;
														// to avoid "ghost" incoming nodes
	// duplication phase
	for (curnode=org1->inputs.begin(); curnode!=org1->inputs.end(); curnode++) {
		tabNodes[(*curnode)->id]=new Neuron(*curnode);
		baby->inputs.push_back(tabNodes[(*curnode)->id]);
		nodes[(*curnode)->id]=true;
	}
	for (curnode=org1->outputs.begin(); curnode!=org1->outputs.end(); curnode++) {
		tabNodes[(*curnode)->id]=new Neuron(*curnode);
		nodes[(*curnode)->id]=true;
	}
	for (curnode=org1->hidden_nodes.begin(); curnode!=org1->hidden_nodes.end(); curnode++) {
		tabNodes[(*curnode)->id]=new Neuron(*curnode);
		dupHidNodes[(*curnode)->id]=true;
	}
	for (curnode=org2->hidden_nodes.begin(); curnode!=org2->hidden_nodes.end(); curnode++) {
		if (!dupHidNodes[(*curnode)->id]) tabNodes[(*curnode)->id]=new Neuron(*curnode);
	}
	free(dupHidNodes);

	//Figure out which genome is better
	//The worse genome should not be allowed to add extra structural baggage
	//If they are the same, use the smaller one's disjoint and excess genes only
	// (principle of Occam's razor)
	if (org1->orig_fitness>org2->orig_fitness) org1_isBetter=true;
	else if (org1->orig_fitness==org2->orig_fitness) {
		if (org1->hidden_nodes.size() < org2->hidden_nodes.size()) org2_isSmaller=false;
		else if (org1->hidden_nodes.size() > org2->hidden_nodes.size()) org1_isSmaller=false;
	}
	else org2_isBetter=true;

	// determine the crossover point
	int crossPoint;
	if (sizeID1<sizeID2) {
		if (org1_isBetter)
			crossPoint=randint(sizeID1/3,sizeID1);
		else if (org2_isBetter)
			crossPoint=randint(0,2*(sizeID1/3));
		else crossPoint=randint(0,sizeID1);
	}
	else {
		if (org1_isBetter)
			crossPoint=randint(0,2*(sizeID2/3));
		else if (org2_isBetter)
			crossPoint=randint(sizeID2/3,sizeID2);
		else crossPoint=randint(0,sizeID2);
	}

	//Now move through the Genes of each parent until both genomes end;
	//neurons and incoming links are order by increasing id, so it's relatively easy
	curnode1=org1->hidden_nodes.begin();
	curnode2=org2->hidden_nodes.begin();
	// prepare the beginning of the second half part (if needed):
	if (sizeID1<sizeID2) {
		while (curnode2!=org2->hidden_nodes.end() && (*curnode2)->id<crossPoint) curnode2++;
	}
	else {
		while (curnode1!=org1->hidden_nodes.end() && (*curnode1)->id<crossPoint) curnode1++;
	}
	while (curnode1!=org1->hidden_nodes.end() || curnode2!=org2->hidden_nodes.end()) {
		if (curnode1==org1->hidden_nodes.end()) {
			if (sizeID1<sizeID2 || (*curnode2)->id<crossPoint) {
				if (org1_isBetter) break; //Skip excess from the worse genome
				if (org2_isBetter || org2_isSmaller) {
					if (global_node_innovation[(*curnode2)->id]!=0) innovNode.push_back((*curnode2)->id);
					for (curlink=(*curnode2)->incoming.begin(); curlink!=(*curnode2)->incoming.end(); curlink++) {
						// if a link is disabled, it has a little chance to be enabled for the child
						if (!(*curlink)->enable && randfloat()<0.25)
							tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,true);
						else tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
						if ((*curlink)->innovation_num>=0) innovLink.push_back((*curlink)->innovation_num);
					}
					baby->hidden_nodes.push_back(tabNodes[(*curnode2)->id]);
					baby->nb_links+=(*curnode2)->incoming.size();
					nodes[(*curnode2)->id]=true;
				}
				curnode2++;
			}
			else break;
		}
		else if (curnode2==org2->hidden_nodes.end()) {
			if (sizeID2<=sizeID1 || (*curnode1)->id<crossPoint) {
				if (org2_isBetter) break;  //Skip excess from the worse genome
				if (org1_isBetter || org1_isSmaller) {
					if (global_node_innovation[(*curnode1)->id]!=0) innovNode.push_back((*curnode1)->id);
					for (curlink=(*curnode1)->incoming.begin(); curlink!=(*curnode1)->incoming.end(); curlink++) {
						// if a link is disabled, it has a little chance to be enabled for the child
						if (!(*curlink)->enable && randfloat()<0.25)
							tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,true);
						else tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
						if ((*curlink)->innovation_num>=0) innovLink.push_back((*curlink)->innovation_num);
					}
					baby->hidden_nodes.push_back(tabNodes[(*curnode1)->id]);
					baby->nb_links+=(*curnode1)->incoming.size();
					nodes[(*curnode1)->id]=true;
				}
				curnode1++;
			}
			else break;
		}
		else {
			//we just look where we are relatively to the cutting point
			if (sizeID1<sizeID2) {
				if ((*curnode1)->id<crossPoint) {
					if (global_node_innovation[(*curnode1)->id]!=0) innovNode.push_back((*curnode1)->id);
					for (curlink=(*curnode1)->incoming.begin(); curlink!=(*curnode1)->incoming.end(); curlink++) {
						// if a link is disabled, it has a little chance to be enabled for the child
						if (!(*curlink)->enable && randfloat()<0.25)
							tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,true);
						else tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
						if ((*curlink)->innovation_num>=0) innovLink.push_back((*curlink)->innovation_num);
					}
					baby->hidden_nodes.push_back(tabNodes[(*curnode1)->id]);
					baby->nb_links+=(*curnode1)->incoming.size();
					nodes[(*curnode1)->id]=true;
					curnode1++;
				}
				else {
					if (global_node_innovation[(*curnode2)->id]!=0) innovNode.push_back((*curnode2)->id);
					for (curlink=(*curnode2)->incoming.begin(); curlink!=(*curnode2)->incoming.end(); curlink++) {
						// if a link is disabled, it has a little chance to be enabled for the child
						if (!(*curlink)->enable && randfloat()<0.25)
							tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,true);
						else tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
						if ((*curlink)->innovation_num>=0) innovLink.push_back((*curlink)->innovation_num);
					}
					baby->hidden_nodes.push_back(tabNodes[(*curnode2)->id]);
					baby->nb_links+=(*curnode2)->incoming.size();
					nodes[(*curnode2)->id]=true;
					curnode2++;
				}
			}
			else {
				if ((*curnode2)->id<crossPoint) {
					if (global_node_innovation[(*curnode2)->id]!=0) innovNode.push_back((*curnode2)->id);
					for (curlink=(*curnode2)->incoming.begin(); curlink!=(*curnode2)->incoming.end(); curlink++) {
						// if a link is disabled, it has a little chance to be enabled for the child
						if (!(*curlink)->enable && randfloat()<0.25)
							tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,true);
						else tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
						if ((*curlink)->innovation_num>=0) innovLink.push_back((*curlink)->innovation_num);
					}
					baby->hidden_nodes.push_back(tabNodes[(*curnode2)->id]);
					baby->nb_links+=(*curnode2)->incoming.size();
					nodes[(*curnode2)->id]=true;
					curnode2++;
				}
				else {
					if (global_node_innovation[(*curnode1)->id]!=0) innovNode.push_back((*curnode1)->id);
					for (curlink=(*curnode1)->incoming.begin(); curlink!=(*curnode1)->incoming.end(); curlink++) {
						// if a link is disabled, it has a little chance to be enabled for the child
						if (!(*curlink)->enable && randfloat()<0.25)
							tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,true);
						else tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
						if ((*curlink)->innovation_num>=0) innovLink.push_back((*curlink)->innovation_num);
					}
					baby->hidden_nodes.push_back(tabNodes[(*curnode1)->id]);
					baby->nb_links+=(*curnode1)->incoming.size();
					nodes[(*curnode1)->id]=true;
					curnode1++;
				}
			}
		}
	}
	// then look at the output nodes (of course we add all the outputs!)
	// outputs are always in the same order, for their final interpretation
	curnode1=org1->outputs.begin();
	curnode2=org2->outputs.begin();
	while (curnode1!=org1->outputs.end()) {
		if (sizeID1<sizeID2) {
			if ((*curnode1)->id<crossPoint) {
				for (curlink=(*curnode1)->incoming.begin(); curlink!=(*curnode1)->incoming.end(); curlink++) {
					// if a link is disabled, it has a little chance to be enabled for the child
					if (!(*curlink)->enable && randfloat()<0.25)
						tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,true);
					else tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
					if ((*curlink)->innovation_num>=0) innovLink.push_back((*curlink)->innovation_num);
				}
				baby->outputs.push_back(tabNodes[(*curnode1)->id]);
				baby->nb_links+=(*curnode1)->incoming.size();
			}
			else {
				for (curlink=(*curnode2)->incoming.begin(); curlink!=(*curnode2)->incoming.end(); curlink++) {
					// if a link is disabled, it has a little chance to be enabled for the child
					if (!(*curlink)->enable && randfloat()<0.25)
						tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,true);
					else tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
					if ((*curlink)->innovation_num>=0) innovLink.push_back((*curlink)->innovation_num);
				}
				baby->outputs.push_back(tabNodes[(*curnode2)->id]);
				baby->nb_links+=(*curnode2)->incoming.size();
			}
		}
		else {
			if ((*curnode2)->id<crossPoint) {
				for (curlink=(*curnode2)->incoming.begin(); curlink!=(*curnode2)->incoming.end(); curlink++) {
					// if a link is disabled, it has a little chance to be enabled for the child
					if (!(*curlink)->enable && randfloat()<0.25)
						tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,true);
					else tabNodes[(*curnode2)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
					if ((*curlink)->innovation_num>=0) innovLink.push_back((*curlink)->innovation_num);
				}
				baby->outputs.push_back(tabNodes[(*curnode2)->id]);
				baby->nb_links+=(*curnode2)->incoming.size();
			}
			else {
				for (curlink=(*curnode1)->incoming.begin(); curlink!=(*curnode1)->incoming.end(); curlink++) {
					// if a link is disabled, it has a little chance to be enabled for the child
					if (!(*curlink)->enable && randfloat()<0.25)
						tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,true);
					else tabNodes[(*curnode1)->id]->add_incoming(tabNodes[(*curlink)->node->id],(*curlink)->weight,(*curlink)->innovation_num,(*curlink)->enable);
					if ((*curlink)->innovation_num>=0) innovLink.push_back((*curlink)->innovation_num);
				}
				baby->outputs.push_back(tabNodes[(*curnode1)->id]);
				baby->nb_links+=(*curnode1)->incoming.size();
			}
		}
		curnode1++;
		curnode2++;
	}

	list<Neuron*>::iterator curnode_;
	list<IncomingLink*>::iterator curlink_,deadlink; // for removal of ghost links
	// we now have to erase the incomings to "ghost" nodes:
	for (curnode_=baby->hidden_nodes.begin(); curnode_!=baby->hidden_nodes.end(); curnode_++) {
		curlink_=(*curnode_)->incoming.begin();
		while (curlink_!=(*curnode_)->incoming.end()) {
			if (!nodes[(*curlink_)->node->id]) {
				deadlink=curlink_;
				curlink_++;
				(*curnode_)->incoming.erase(deadlink);
			}
			else curlink_++;
		}
	}
	for (curnode_=baby->outputs.begin(); curnode_!=baby->outputs.end(); curnode_++) {
		curlink_=(*curnode_)->incoming.begin();
		while (curlink_!=(*curnode_)->incoming.end()) {
			if (!nodes[(*curlink_)->node->id]) {
				deadlink=curlink_;
				curlink_++;
				(*curnode_)->incoming.erase(deadlink);
			}
			else curlink_++;
		}
	}

	// finally, look through all innovations, and add them to the lists
	sort(innovLink.begin(),innovLink.end());
	for (int i=innovLink.size()-1; i>=0; i--) {
		if (!nodes[global_link_innovation[innovLink[i]]->incnode->id]) continue;
		baby->link_innovations.push_front(global_link_innovation[innovLink[i]]);
	}
	free(nodes);
	sort(innovNode.begin(),innovNode.end());
	for (int i=innovNode.size()-1; i>=0; i--)
		baby->node_innovations.push_front(global_node_innovation[innovNode[i]]);

	return baby;
}
