#ifndef _Neat
#define _Neat

#include <iostream>
#include <fstream>
#include <string>

using namespace std;

class NEAT { 

public:

	/* The power of a linkweight mutation: */
	static double weight_mut_power;
	/* the power of a threshold mutation: */
	static double thresh_mut_power;
  
	/* These 3 global coefficients are used to determine the formula for
		computing the compatibility between 2 genomes.  The formula is:
		disjoint_coeff*pdg+excess_coeff*peg+weight_diff_coeff*mdmg.
		See the compatibility method in the Genome class for more info
		They can be thought of as the importance of disjoint Genes,
		excess Genes, and parametric difference between Genes of the
		same function, respectively. */
	static double disjoint_coeff;
	static double excess_coeff;
	static double weight_diff_coeff;
  
	/* compatibility threshold under which two Genomes are considered the same species */
	static double compat_threshold;
	/* Globals involved in the epoch cycle - mating, reproduction, etc.. */
	static double age_significance; /* How much does age matter? */
	static double survival_thresh; /* Percent of ave fitness for survival */
	static double mutate_link_weights_prob;
	static double gausspoint_thresh;
	static double coldgausspoint_thresh;
	static double mutate_toggle_enable_prob;
	static double mutate_gene_reenable_prob;
	static double mutate_threshold_prob;
	static double mutate_add_link_prob;
	static double mutate_add_node_prob;
	static double mate_multipoint_prob;     
	static double mate_multipoint_avg_prob;
	static double mate_singlepoint_prob;
	
	static double frac_mut_node;
	static double frac_mut_link;
	static double frac_toggle_link;
	static double frac_mutate_thresh;
	
	static int pop_size; /* size of the initial population */
	static int generations; /* max number of generations */
	static int young_limit; /* age under which species are considered young */
	static int dropoff_age; /* Age where Species starts to be penalized */
	static int print_every; /* Tells to print population to file every n generations */
	static int num_runs; /* The number of runs to average over in an experiment */
	static double search_time; /* time allowed for the experiment */
  
	//Construct NEAT object off of a NE param (.ne) file
	static void load_neat_params(char *filename) {
		ifstream paramFile(filename);
		if (paramFile==0) {
			cerr << "error can't open file " << filename << " for input" << endl;
			exit(1);
		}
		char curword[20];
		//**********LOAD IN PARAMETERS***************//
		cout << "NEAT READING IN " << filename << endl;

		paramFile >> curword;
		paramFile >> weight_mut_power;
		paramFile >> curword;
		paramFile >> thresh_mut_power;

		paramFile >> curword;
		paramFile >> disjoint_coeff;
		paramFile >> curword;
		paramFile >> excess_coeff;
		paramFile >> curword;
		paramFile >> weight_diff_coeff;

		paramFile >> curword;
		paramFile >> compat_threshold;

		paramFile >> curword;
		paramFile >> age_significance;
		paramFile >> curword;
		paramFile >> survival_thresh;
		paramFile >> curword;
		paramFile >> mutate_link_weights_prob;
		paramFile >> curword;
		paramFile >> gausspoint_thresh;
		paramFile >> curword;
		paramFile >> coldgausspoint_thresh;
		paramFile >> curword;
		paramFile >> mutate_toggle_enable_prob;
		paramFile >> curword;
		paramFile >> mutate_gene_reenable_prob;
		paramFile >> curword;
		paramFile >> mutate_threshold_prob;
		paramFile >> curword;
		paramFile >> mutate_add_link_prob;
		paramFile >> curword;
		paramFile >> mutate_add_node_prob;
		paramFile >> curword;
		paramFile >> mate_multipoint_prob;
		paramFile >> curword;
		paramFile >> mate_multipoint_avg_prob;
		paramFile >> curword;
		paramFile >> mate_singlepoint_prob;

		paramFile >> curword;
		paramFile >> frac_mut_node;
		paramFile >> curword;
		paramFile >> frac_mut_link;
		paramFile >> curword;
		paramFile >> frac_toggle_link;
		paramFile >> curword;
		paramFile >> frac_mutate_thresh;

		paramFile >> curword;
		paramFile >> pop_size;
		paramFile >> curword;
		paramFile >> generations;
		paramFile >> curword;
		paramFile >> young_limit;
		paramFile >> curword;
		paramFile >> dropoff_age;
		paramFile >> curword;
		paramFile >> print_every;
		paramFile >> curword;
		paramFile >> num_runs;
		paramFile >> curword;
		paramFile >> search_time;

		paramFile.close();

		cout << "weight_mut_power=" << weight_mut_power << endl;
		cout << "thresh_mut_power=" << thresh_mut_power << endl;

		cout << "disjoint_coeff=" << disjoint_coeff << endl;
		cout << "excess_coeff=" << excess_coeff << endl;
		cout << "weight_diff_coeff=" << weight_diff_coeff << endl;

		cout<< "compat_threshold=" << compat_threshold << endl;

		cout << "age_significance=" << age_significance << endl;
		cout << "survival_thresh=" << survival_thresh << endl;
		cout << "mutate_link_weights_prob=" << mutate_link_weights_prob << endl;
		cout << "gausspoint_thresh=" << gausspoint_thresh << endl;
		cout << "coldgausspoint_thresh=" << coldgausspoint_thresh << endl;
		cout << "mutate_toggle_enable_prob=" << mutate_toggle_enable_prob << endl;
		cout << "mutate_gene_reenable_prob=" << mutate_gene_reenable_prob << endl;
		cout << "mutate_threshold_prob=" << mutate_threshold_prob << endl;
		cout << "mutate_add_link_prob=" << mutate_add_link_prob << endl;
		cout << "mutate_add_node_prob=" << mutate_add_node_prob << endl;
		cout << "mate_multipoint_prob=" << mate_multipoint_prob << endl;
		cout << "mate_multipoint_avg_prob=" << mate_multipoint_avg_prob << endl;
		cout << "mate_singlepoint_prob=" << mate_singlepoint_prob << endl;

		cout << "frac_mut_link=" << frac_mut_link << endl;
		cout << "frac_mut_node=" << frac_mut_node << endl;
		cout << "frac_toggle_link=" << frac_toggle_link << endl;
		cout << "frac_mutate_thresh=" << frac_mutate_thresh << endl;

		cout << "pop_size=" << pop_size << endl;
		cout << "generations=" << generations << endl;
		cout << "young_limit=" << young_limit << endl;
		cout << "dropoff_age=" << dropoff_age << endl;
		cout << "print_every=" << print_every << endl;
		cout << "num_runs=" << num_runs << endl;
		cout << "search_time=" << search_time << endl;
	}

	NEAT () {}

	NEAT (char* filename) {
		load_neat_params(filename);
	}

};

#endif
