#ifndef _Neuron
#define _Neuron

#include <list>
#include <string>
#include <fstream>

using namespace std;

class Neuron;

/* A LINK/gene is a connection from one node to another with an associated weight,
 * along with an "innovation number" which tells when
 * in the history of a population the gene first arose. This allows
 * the system to track innovations and use those to determine which
 * organisms are compatible. (i.e. in the same species) */

class IncomingLink {

public:
	
	Neuron* node; // one predecessor in the net
	double weight; //Weight of connection
	int innovation_num;
	bool enable; //When this is off the Gene is disabled

	IncomingLink () {}
	IncomingLink (Neuron* nd,double w,int innov=-1,bool en=true) {
		node=nd;
		weight=w;
		innovation_num=innov;
		enable=en;
	}
	~IncomingLink () {}
};

/* a neuron receive one or several inputs, and 
 * get activated or not depending on the weights
 * of connections */

class Neuron {

public:
	
	/* Activation Types */
	static const int SIGMOID=0;
	static const int LINEAR=1;
	static const int nbActTypes=2;

	int functype; // type of the activation function
	int id; // useful for reproduction
	list<IncomingLink*> incoming; //predecessors of the neuron (empty if sensor)
	double activation,incoming_sum; //keeps track of which activation the node is currently in
	double threshold; // node's activation threshold under which nothing happens
					// (after transformation of inputs!); begins with 0, remains in (0,1).
	bool enable; // if an input is enable, useful for the pig game
	bool activeIncoming; // tells if the node receive some signal (often true, but..)
	
	// fields for drawing and learning:
	int x,y;
	bool visited;
	string label;
	int depth; // depth of the neuron in the arranged topology netwok
	
	Neuron () {}
	
	Neuron (int n_id,int ft=0,double thresh=0.0) {
		id=n_id;
		functype=ft;
		threshold=thresh;
		activation=0.0;
		enable=true;
	}

	// copy constructor: (without the incoming links! have to be added manually;
	// we do that because at his creation a neuron often doesn't know his predecessors)
	Neuron (const Neuron* node) {
		id=node->id;
		functype=node->functype;
		threshold=node->threshold;
		activation=node->activation;
		enable=node->enable;
	}

	~Neuron() {
		for (list<IncomingLink*>::iterator curlink=incoming.begin(); curlink!=incoming.end(); curlink++) 
			delete *curlink;
	}

	// insert an incoming link in innovation num's order
	void add_incoming(Neuron* node,double weight,int innovation_num=-1,bool enable=true) {
		IncomingLink* inc=new IncomingLink(node,weight,innovation_num,enable);
		list<IncomingLink*>::iterator curlink=incoming.begin();
		while (curlink!=incoming.end() && (*curlink)->innovation_num<inc->innovation_num) curlink++;
		incoming.insert(curlink,inc);
	}
	
	// comparison for ordering before drawing:
	bool operator<(const Neuron& node) const {
		return id < node.id;
	}

};

#define SIGMOID Neuron::SIGMOID
#define LINEAR Neuron::SIGMOID
#define nbActTypes Neuron::nbActTypes

#endif
