#include "Pig.h"

// the organism we are testing:
Organism* neural_pig;

void player_handler(PIG_object *po, const PIG_event *ev) {
	GAMESTATE *gs = (GAMESTATE *)po->owner->userdata;
	if (neural_pig!=0) {
		gs->keys[SDLK_LEFT]=0;
		gs->keys[SDLK_RIGHT]=0;
		gs->keys[SDLK_UP]=0;
		switch (neural_pig->getActiveOutput()) {
			case 0:
				gs->keys[SDLK_LEFT]=1;
				break;
			case 1:
				gs->keys[SDLK_RIGHT]=1;
				break;
			case 2:
				gs->keys[SDLK_UP]=1;
				break;
			default: break;
		}
	}

	switch(ev->type) {
		case PIG_PREFRAME:
			switch(po->state) {
				case WAITING:
					if(1 == po->age)
						message(gs, "Get ready!");
					else if(po->age > 50)
						po->state = FALLING;
					break;
				case WALKING:
					if(gs->keys[SDLK_LEFT]) {
						po->ax = -(20 + po->vx) * .4;
						po->target = 3 + po->age % 4 - 1;
						if(5 == po->target)
							po->target = 3;
					}
					else if(gs->keys[SDLK_RIGHT]) {
						po->ax = (20 - po->vx) * .4;
						po->target = 9 + po->age % 4 - 1;
						if(11 == po->target)
							po->target = 9;
					}
					else {
						po->ax = -po->vx * .8;
						if(po->target >= 6)
							po->target = (po->target + 1) %PIG_FRAMES;
						else if(po->target)
							--po->target;
					}
					break;
				case FALLING:
					if(gs->keys[SDLK_LEFT])
						po->ax = -(20 + po->vx) * .2;
					else if(gs->keys[SDLK_RIGHT])
						po->ax = (20 - po->vx) * .2;
					else
						po->ax = -po->vx * .2;
					po->target = (po->target + 1) % PIG_FRAMES;
					break;
			}
			po->timer[0] = 1;
			break;
		case PIG_TIMER0:
			if(po->x < 0)
				po->x = 0;
			else if(po->x > po->owner->view.w - 1)
				po->x = po->owner->view.w - 1;
			switch(po->state) {
				case WALKING:
					if(po->power)
						--po->power;
					po->image = po->target % PIG_FRAMES;
					if(!pig_test_map(gs->pe, (int)(po->x),(int)(po->y) + 1)) {
						po->state = FALLING;
						po->ay = GRAV_ACC;
					}
					if(gs->jump || gs->keys[SDLK_UP]) {
						po->ay = 0;
						po->vy = -JUMP_SPEED;
						po->state = FALLING;
						gs->jump = 0;
					}
					break;
				case FALLING:
					if(po->vy > 2)
						po->power = 3;
					po->ay = GRAV_ACC;
					po->image = po->target;
					break;
				case KNOCKED:
					po->power = 0;
					po->ay = GRAV_ACC;
					po->target = (po->target + 2) % PIG_FRAMES;
					po->image = po->target;
					po->ax = -po->vx * .2;
					break;
				case NEXT_LEVEL:
					po->vx = (SCREEN_W / 2 - po->x) * .1;
					po->target = (po->target + 1) % PIG_FRAMES;
					po->image = po->target;
					break;
				case DEAD:
					po->ax = po->ay = 0;
					po->vx = po->vy = 0;
					break;
			}
			if(gs->jump)
				--gs->jump;
			if(NEXT_LEVEL != po->state) {
				if(gs->enemycount <= 0) {
					message(gs, "Well Done!");
					po->state = NEXT_LEVEL;
					po->vy = 0;
					po->ay = -1;
					po->tilemask = 0;
					po->hitgroup = 0;
					po->timer[2] = 50;
				}
			}
			break;
		case PIG_TIMER1:
			/* Snap out of KNOCKED mode */
			po->state = FALLING;
			break;
		case PIG_TIMER2:
			switch(po->state) {
				case NEXT_LEVEL:
					add_life(gs);
					pig_object_close(po);
					load_level(gs,gs->level+1);
					new_player(gs);
					break;
				default:
					pig_object_close(po);
					load_map_level(gs,gs->level);
					new_player(gs);
					break;
			}
			break;
		case PIG_HIT_TILE:
			if(KNOCKED == po->state)
				break;
			if(ev->cinfo.sides & PIG_TOP) {
				po->y = ev->cinfo.y;
				po->vy = 0;
				po->ay = 0;
			}
			po->state = WALKING;
			break;
		case PIG_HIT_OBJECT:
			if(KNOCKED == po->state)
				break;
			switch(ev->obj->hitgroup) {
				case GROUP_ENEMY:
					if((po->power && ev->cinfo.sides & PIG_TOP) ||
						(po->vy - ev->obj->vy) >= 15) {
						/* Win: Stomp! */
						inc_score(gs, ev->obj->score);
						if (neural_pig!=0) {
							// disable inputs corresponding to this enemy, if neural player:
							list<Neuron*>::iterator curnode=neural_pig->inputs.begin();
							for (int i=6*(ev->obj->rank+1)-1; i>=0; i--) curnode++;
							for (int i=5; i>=0; i--) {
								(*curnode)->enable=false;
								curnode++;
							}
						}

						ev->obj->y = ev->cinfo.y + 10;
						if(po->vy > 0)
							ev->obj->vy = po->vy;
						else
							ev->obj->vy = 10;
						ev->obj->ay = GRAV_ACC;
						ev->obj->tilemask = 0;
						ev->obj->hitgroup = 0;
						if(gs->jump || gs->keys[SDLK_UP]) {
							/* Mega jump! */
							po->vy = -(JUMP_SPEED + 7);
							gs->jump = 0;
						}
						else {
							/* Bounce a little */
							po->vy = -15;
						}
						po->y = ev->cinfo.y;
						po->ay = 0;
						po->state = FALLING;
					}
					else {
						/* Lose: Knocked! */
						po->vy = -15;
						po->ay = GRAV_ACC;
						po->state = KNOCKED;
						po->timer[1] = 11;
						new_star(gs, (int)(po->x), (int)(po->y) - 20, -5, 3);
						new_star(gs, (int)(po->x), (int)(po->y) - 20, 2, -6);
						new_star(gs, (int)(po->x), (int)(po->y) - 20, 4, 4);
					}
					break;
			}
			break;
		case PIG_OFFSCREEN:
			/*
			* Dead pigs don't care about being off-screen.
			* A timer is used to remove them, and to continue
			* the game with a new life.
			*/
			if(DEAD == po->state)
				break;
			if(po->y < 0)	/* Above the playfield is ok. */
				break;
			if(gs->lives)
				message(gs, "Oiiiiiiink!!!");
			else
				message(gs, "Game Over!");
			po->state = DEAD;
			po->timer[2] = 50;
		default:
			break;
	}
}

PIG_object *new_player(GAMESTATE *gs) {
	PIG_object *po;
	if(!gs->lives)
		return NULL;

	po = pig_object_open(gs->pe, SCREEN_W / 2, -50, 1);
	if(!po)
		return NULL;

	remove_life(gs);
	po->ibase = gs->pigframes;
	po->handler = player_handler;
	po->hitmask = GROUP_ENEMY;
	po->type=2;
	return po;
}

int start_game(GAMESTATE *gs) {
	gs->score = 0;
	gs->lives = 5;

	gs->player = new_player(gs);
	if(!gs->player)
		return -1;

	return 0;
}

/*----------------------------------------------------------
	main()
----------------------------------------------------------*/

int pig_main(int argc,char** argv) {

	SDL_Surface *screen;
	GAMESTATE *gs;
	int i,testLevel=1;
	int bpp = 0;
	int flags = SDL_DOUBLEBUF | SDL_HWSURFACE;

	SDL_Init(SDL_INIT_VIDEO);
	atexit(SDL_Quit);
	
	// determines if we are playing as human, or testing the network:
	char* nameFileOrg=0;
	if (argc>2) {
		if (argv[2][0]>='0' && argv[2][0]<='9') {
			testLevel=atoi(argv[2]);
			if (argc>3) nameFileOrg=argv[3];
		}
		else nameFileOrg=argv[2];
	}
	if (nameFileOrg!=0) {
		std::ifstream fileOrg(nameFileOrg);
		if (!fileOrg) {
			fprintf(stderr,"error: can't read network file\n");
			return -1;
		}
		neural_pig=new Organism(fileOrg);
		fileOrg.close();
	}
	else neural_pig=0;

	for(i = 1; i < argc; ++i) {
		if(strncmp(argv[i], "-s", 2) == 0)
			flags &= ~SDL_DOUBLEBUF;
		else if(strncmp(argv[i], "-f", 2) == 0)
			flags |= SDL_FULLSCREEN;
		else
			bpp = atoi(&argv[i][1]);
	}

	screen = SDL_SetVideoMode(SCREEN_W, SCREEN_H, bpp, flags);
	if(!screen) {
		fprintf(stderr, "Failed to open screen!\n");
		return 1;
	}

	SDL_WM_SetCaption("Fixed Rate Pig", "Pig");
	SDL_ShowCursor(0);

	gs = init_all(screen);
	if(!gs)
		return 1;

	gs->keys = SDL_GetKeyState(&i);
	gs->level=0;

	pig_start(gs->pe, 0);
	gs->refresh_screen = gs->pe->pages;
	int j=0;
	while(gs->running) {
		SDL_Event ev;

		if (gs->level==0) {
			load_level(gs,testLevel);
			start_game(gs);
		}

		/* Handle input */
		while(SDL_PollEvent(&ev) > 0)
			handle_input(gs, &ev);

		/* Run the game logic */
		run_logic(gs->pe,neural_pig);

		/*
		 * Limit the dashboard frame rate to 15 fps.
		 */
		if(j%15==0) dashboard(gs);

		/* Update sprites */
		if(gs->refresh_screen) {
			--gs->refresh_screen;
			pig_refresh_all(gs->pe);
		}
		else
			pig_refresh(gs->pe);

		/* Make the new frame visible */
		pig_flip(gs->pe);

		/* make things visible for human */
		SDL_Delay(40);
		
		j++;
	}

	pig_close(gs->pe);
	return 0;
}
