#include "PigEval.h"

Organism* neural_pig_eval;
GAMESTATE *gs;
SDL_Surface *screen;

int countLoops; // global to "detect" loops in behavior
const int MAX_LOOPS=300;
int evalLevel; // level for evaluation

void player_handler_eval(PIG_object *po, const PIG_event *ev) {
	GAMESTATE *gs = (GAMESTATE *)po->owner->userdata;
	gs->keys[SDLK_LEFT]=0;
	gs->keys[SDLK_RIGHT]=0;
	gs->keys[SDLK_UP]=0;
	switch (neural_pig_eval->getActiveOutput()) {
		case 0:
			gs->keys[SDLK_LEFT]=1;
			break;
		case 1:
			gs->keys[SDLK_RIGHT]=1;
			break;
		case 2:
			gs->keys[SDLK_UP]=1;
			break;
		default: break;
	}

	switch(ev->type) {
		case PIG_PREFRAME:
			switch(po->state) {
				case WAITING:
					countLoops=MAX_LOOPS;
					if(po->age > 50)
						po->state = FALLING;
					break;
				case WALKING:
					if(gs->keys[SDLK_LEFT]) {
						po->ax = -(20 + po->vx) * .4;
						po->target = 3 + po->age % 4 - 1;
						if(5 == po->target)
							po->target = 3;
					}
					else if(gs->keys[SDLK_RIGHT]) {
						po->ax = (20 - po->vx) * .4;
						po->target = 9 + po->age % 4 - 1;
						if(11 == po->target)
							po->target = 9;
					}
					else {
						po->ax = -po->vx * .8;
						if(po->target >= 6)
							po->target = (po->target + 1) %PIG_FRAMES;
						else if(po->target)
							--po->target;
					}
					break;
				case FALLING:
					if(gs->keys[SDLK_LEFT])
						po->ax = -(20 + po->vx) * .2;
					else if(gs->keys[SDLK_RIGHT])
						po->ax = (20 - po->vx) * .2;
					else
						po->ax = -po->vx * .2;
					po->target = (po->target + 1) % PIG_FRAMES;
					break;
			}
			po->timer[0] = 1;
			break;
		case PIG_TIMER0:
			if(po->x < 0)
				po->x = 0;
			else if(po->x > po->owner->view.w - 1)
				po->x = po->owner->view.w - 1;
			switch(po->state) {
				case WALKING:
					if(po->power)
						--po->power;
					po->image = po->target % PIG_FRAMES;
					if(!pig_test_map_eval(gs->pe, (int)(po->x),(int)(po->y) + 1)) {
						po->state = FALLING;
						po->ay = GRAV_ACC;
					}
					if(gs->jump || gs->keys[SDLK_UP]) {
						po->ay = 0;
						po->vy = -JUMP_SPEED;
						po->state = FALLING;
						gs->jump = 0;
					}
					break;
				case FALLING:
					if(po->vy > 2)
						po->power = 3;
					po->ay = GRAV_ACC;
					po->image = po->target;
					break;
				case KNOCKED:
					po->power = 0;
					po->ay = GRAV_ACC;
					po->target = (po->target + 2) % PIG_FRAMES;
					po->image = po->target;
					po->ax = -po->vx * .2;
					break;
				case NEXT_LEVEL:
					po->vx = (SCREEN_W / 2 - po->x) * .1;
					po->target = (po->target + 1) % PIG_FRAMES;
					po->image = po->target;
					break;
				case DEAD:
					po->ax = po->ay = 0;
					po->vx = po->vy = 0;
					break;
			}
			if(gs->jump)
				--gs->jump;
			if(NEXT_LEVEL != po->state) {
				if(gs->enemycount <= 0) {
					po->state = NEXT_LEVEL;
					po->vy = 0;
					po->ay = -1;
					po->tilemask = 0;
					po->hitgroup = 0;
					po->timer[2] = 50;
				}
			}
			break;
		case PIG_TIMER1:
			/* Snap out of KNOCKED mode */
			po->state = FALLING;
			break;
		case PIG_TIMER2:
			switch(po->state) {
				case NEXT_LEVEL:
					add_life_eval(gs);
					pig_object_close_eval(po);
					load_level_eval(gs,gs->level+1);
					new_player_eval(gs);
					break;
				default:
					pig_object_close_eval(po);
					load_map_level_eval(gs,gs->level);
					new_player_eval(gs);
					break;
			}
			break;
		case PIG_HIT_TILE:
			if(KNOCKED == po->state)
				break;
			if(ev->cinfo.sides & PIG_TOP) {
				po->y = ev->cinfo.y;
				po->vy = 0;
				po->ay = 0;
			}
			po->state = WALKING;
			break;
		case PIG_HIT_OBJECT:
			if(KNOCKED == po->state)
				break;
			switch(ev->obj->hitgroup) {
				case GROUP_ENEMY:
					if((po->power && ev->cinfo.sides & PIG_TOP) ||
						(po->vy - ev->obj->vy) >= 15) {
						/* Win: Stomp! */
						inc_score_eval(gs, ev->obj->score);
						countLoops=MAX_LOOPS;
						// disable inputs corresponding to this enemy:
						list<Neuron*>::iterator curnode=neural_pig_eval->inputs.begin();
						for (int i=6*(ev->obj->rank+1)-1; i>=0; i--) curnode++;
						for (int i=5; i>=0; i--) {
							(*curnode)->enable=false;
							curnode++;
						}

						ev->obj->y = ev->cinfo.y + 10;
						if(po->vy > 0)
							ev->obj->vy = po->vy;
						else
							ev->obj->vy = 10;
						ev->obj->ay = GRAV_ACC;
						ev->obj->tilemask = 0;
						ev->obj->hitgroup = 0;
						if(gs->jump || gs->keys[SDLK_UP]) {
							/* Mega jump! */
							po->vy = -(JUMP_SPEED + 7);
							gs->jump = 0;
						}
						else {
							/* Bounce a little */
							po->vy = -15;
						}
						po->y = ev->cinfo.y;
						po->ay = 0;
						po->state = FALLING;
					}
					else {
						/* Lose: Knocked! */
						po->vy = -15;
						po->ay = GRAV_ACC;
						po->state = KNOCKED;
						po->timer[1] = 11;
						new_star_eval(gs, (int)(po->x), (int)(po->y) - 20, -5, 3);
						new_star_eval(gs, (int)(po->x), (int)(po->y) - 20, 2, -6);
						new_star_eval(gs, (int)(po->x), (int)(po->y) - 20, 4, 4);
					}
					break;
			}
			break;
		case PIG_OFFSCREEN:
			/*
			* Dead pigs don't care about being off-screen.
			* A timer is used to remove them, and to continue
			* the game with a new life.
			*/
			if(DEAD == po->state)
				break;
			if(po->y < 0)	/* Above the playfield is ok. */
				break;
			po->state = DEAD;
			po->timer[2] = 50;
		default:
			break;
	}
}

PIG_object *new_player_eval(GAMESTATE *gs) {
	PIG_object *po;
	if(!gs->lives)
		return NULL;

	po = pig_object_open_eval(gs->pe, SCREEN_W / 2, -50, 1);
	if(!po)
		return NULL;

	remove_life_eval(gs);
	po->ibase = gs->pigframes;
	po->handler = player_handler_eval;
	po->hitmask = GROUP_ENEMY;
	po->type=2;
	return po;
}

int start_game_eval(GAMESTATE *gs) {
	gs->score = 0;
	gs->lives = 5;

	gs->player = new_player_eval(gs);
	if(!gs->player)
		return -1;

	return 0;
}

// initialize screen and some others for the next function
void init_pig_eval(int level) {
	int flags = SDL_DOUBLEBUF | SDL_HWSURFACE;

	SDL_Init(SDL_INIT_VIDEO);
	atexit(SDL_Quit);

	screen = SDL_SetVideoMode(SCREEN_W, SCREEN_H, 0, flags);
	if(!screen) {
		fprintf(stderr, "Failed to open screen!\n");
		return;
	}

	gs = init_all_eval(screen);
	if(!gs)
		return;

	int i;
	gs->keys = SDL_GetKeyState(&i);
	
	pig_start_eval(gs->pe, 0);
	gs->refresh_screen = gs->pe->pages;

	evalLevel=level;
}

// here we play the pig game, but much faster because not printed to screen, to give a fitness to the organism;
void launch_test_pig(Organism* org) {
	neural_pig_eval=org; // assign the current organism to the global variable
	// flushing to permit determinism:
	neural_pig_eval->flush();
	// and reinit inputs
	neural_pig_eval->reinitInputs();

	gs->level=0;
	org->winner=false;
	load_level_eval(gs,evalLevel);
	start_game_eval(gs);
	
	countLoops=MAX_LOOPS;
	while(gs->lives && --countLoops >=0) {
		SDL_Event ev;
		// Handle input
		while(SDL_PollEvent(&ev) > 0)
			handle_input_eval(gs, &ev);

		// Run the game logic
		run_logic_eval(gs->pe,neural_pig_eval);

		if ((evalLevel==1 && gs->score==800)
			|| (evalLevel==2 && gs->score==1000)
			|| (evalLevel==3 && gs->score==1200)
			|| (evalLevel==4 && gs->score==2400)) {
			org->winner=true;
			break;
		}

		// Update sprites
		if(gs->refresh_screen) {
			--gs->refresh_screen;
			pig_refresh_all_eval(gs->pe);
		}
		else
			pig_refresh_eval(gs->pe);

		// Make the new frame visible
		pig_flip_eval(gs->pe);
	}

	org->fitness=gs->score+25*gs->lives;
}

// to make things more clean, call that after all evaluations:
void finish_pig_eval() {
	pig_close_eval(gs->pe);
}

/*
// trial with a metamodel:
bool *map1,*map2,*map3,*map4; // maps of levels (true==wall, false==empty)

// parameters to fasten the evaluation function:
double* inputs;
bool* visitedTiles;
// pig's parameters:
double* pigParam;
// enemies parameters:
double* enemyParam;
bool* enemyAlive;

// initialize boolean maps and storage vectors; must be called before next function.
// for the moment: only map 1
void init_pig_eval() {
	map1=(bool*)malloc(MAP_W*MAP_H*sizeof(bool));
	map2=(bool*)malloc(MAP_W*MAP_H*sizeof(bool));
	map3=(bool*)malloc(MAP_W*MAP_H*sizeof(bool));
	map4=(bool*)malloc(MAP_W*MAP_H*sizeof(bool));
	
	// init first map (1 for ground, 0 for air)
	for (int i=MAP_H-1; i>=0; i--) {
		if (i==13) {
			for (int j=MAP_W-1; j>=0; j--) map1[j+i*MAP_W]=true;
		}
		else if (i==10) {
			for (int j=3; j>=0; j--) map1[j+i*MAP_W]=true;
			for (int j=4; j<MAP_W-4; j++) map1[j+i*MAP_W]=false;
			for (int j=MAP_W-4; j<MAP_W; j++) map1[j+i*MAP_W]=true;
		}
		else if (i==7) {
			for (int j=6; j>=0; j--) map1[j+i*MAP_W]=false;
			for (int j=7; j<MAP_W-7; j++) map1[j+i*MAP_W]=true;
			for (int j=MAP_W-7; j<MAP_W; j++) map1[j+i*MAP_W]=false;
		}
		else {
			for (int j=MAP_W-1; j>=0; j--) map1[j+i*MAP_W]=false;
		}
	}

	// parameters to fasten the next function:
	inputs=(double*)malloc(31*sizeof(double));
	visitedTiles=(bool*)malloc(MAP_W*MAP_H*sizeof(bool));
	// pig's parameters:
	pigParam=(double*)malloc(6*sizeof(double));
	// enemies parameters:
	enemyParam=(double*)malloc(24*sizeof(double));
	enemyAlive=(bool*)malloc(4*sizeof(bool));
}

// here we play the pig game, but much faster because not printed to screen, to give a fitness to the organism;
// fitness calculation is simple:
// +1 for each enemy killed, -0.1 for each lost life.
void launch_test_pig(Organism* org) {
	// we will just calculate the game physic with a simplified metamodel (using tiles structure),
	// and deduce the approximate behavior of the pig.
	// later, learning during game could correct (to some extent) the drawbacks of the metamodel.
	
	int countTiles,killed,usedLives,nextAction;
	bool jumped,pigIsDead;
	int nbTiles;
	
	// parameters to store last actions for speed/acceleration:
	int lastLeft,lastRight;

	// flushing to permit determinism:
	org->flush();
	// and reinit inputs
	org->reinitInputs();
	
	//////////////////////////////////
	// initial parameters for map 1:
	//////////////////////////////////

	// pig's parameters:
	pigParam[0]=12.0;
	pigParam[1]=6.0;
	pigParam[2]=0.0;
	pigParam[3]=0.0;
	pigParam[4]=0.0;
	pigParam[5]=0.0;
	// enemies parameters:
	enemyParam[0]=2.0;
	enemyParam[1]=9.0;
	enemyParam[2]=5.0;
	enemyParam[3]=0.0;
	enemyParam[4]=0.0;
	enemyParam[5]=0.0;
	enemyParam[6]=22.0;
	enemyParam[7]=9.0;
	enemyParam[8]=-5.0;
	enemyParam[9]=0.0;
	enemyParam[10]=0.0;
	enemyParam[11]=0.0;
	enemyParam[12]=5.0;
	enemyParam[13]=12.0;
	enemyParam[14]=-5.0;
	enemyParam[15]=0.0;
	enemyParam[16]=0.0;
	enemyParam[17]=0.0;
	enemyParam[18]=19.0;
	enemyParam[19]=12.0;
	enemyParam[20]=5.0;
	enemyParam[21]=0.0;
	enemyParam[22]=0.0;
	enemyParam[23]=0.0;
	// and, finally, the boolean vector of the living enemies:
	for (int i=3; i>=0; i--) enemyAlive[i]=true;

	countTiles=0;
	killed=0;
	usedLives=0;
	lastLeft=0;
	lastRight=0;
	// reinit the space visited:
	for (int i=MAP_W*MAP_H-1; i>=0; i--) visitedTiles[i]=false;
	nbTiles=0;

	while(countTiles++ < 100) { // stopping criteria: if moved throughout n tiles and nothing happened,
								// or win or death.
		// calculate current inputs
		for (int i=29; i>=0; i--) {
			if (i>=6) inputs[i]=enemyParam[i-6];
			else inputs[i]=pigParam[i];
		}
		inputs[30]=1.0;

		// get the network's most active output;
		// here we don't flush, because it's the evolution of a single organism
		org->load_sensors(inputs);

		for (int i=org->hidden_nodes.size(); i>=0; i--) org->activate();
		nextAction=org->getActiveOutput();

		/////////////////////////////////////////////
		// re-calculate pig and enemies parameters:
		/////////////////////////////////////////////

		jumped=false;
		switch (nextAction) {
			case 0:
				lastRight=0;
				if (lastLeft==0) lastLeft=7;
				else lastLeft++;
				if (pigParam[5]==0.0) {
					if (lastLeft>=20) {
						pigParam[4]=0.0;
						pigParam[2]=-20.0;
						pigParam[0]=maxf(pigParam[0]-2.0,0.0);
					}
					else {
						pigParam[4]=-(20-lastLeft)*0.4;
						pigParam[2]=-lastLeft;
						pigParam[0]=maxf(pigParam[0]-0.1*lastLeft,0.0);
					}
				}
				else {
					if (lastLeft>=20) {
						pigParam[4]=0.0;
						pigParam[2]=-10.0;
						pigParam[0]=maxf(pigParam[0]-1.5,0.0);
					}
					else {
						pigParam[4]=-(20-lastLeft)*0.2;
						pigParam[2]=-lastLeft*0.5;
						pigParam[0]=maxf(pigParam[0]-0.15*lastLeft,0.0);
					}
				}
				break;
			case 1:
				lastLeft=0;
				if (lastRight==0) lastRight=7;
				else lastRight++;
				if (pigParam[5]==0.0) {
					if (lastRight>=20) {
						pigParam[4]=0.0;
						pigParam[2]=20.0;
						pigParam[0]=minf(pigParam[0]+2.0,25.0);
					}
					else {
						pigParam[4]=(20-lastRight)*0.4;
						pigParam[2]=lastRight;
						pigParam[0]=minf(pigParam[0]+0.1*lastRight,25.0);
					}
				}
				else {
					if (lastRight>=20) {
						pigParam[4]=0.0;
						pigParam[2]=10.0;
						pigParam[0]=minf(pigParam[0]+1.5,25.0);
					}
					else {
						pigParam[4]=(20-lastRight)*0.2;
						pigParam[2]=lastRight*0.5;
						pigParam[0]=minf(pigParam[0]+0.15*lastRight,25.0);
					}
				}
				break;
			case 2:
				if (pigParam[5]==0.0) {
					pigParam[1]-=3.5;
					pigParam[3]=0.0;
					pigParam[5]=GRAV_ACC;
					jumped=true;
				}
				if (lastLeft>0) lastLeft--;
				else if (lastRight>0) lastRight--;
				break;
			default: break;
		}

		// we can't control 'y' movement after a jump, so if falling, update:
		if (pigParam[5]>0.0 && !jumped) {
			// we look what is just under the pig:
			if (map1[(int)floor(minf(pigParam[0],24.0)+MAP_W*minf(pigParam[1],16.0)+MAP_W)]) {
				// arrived on the ground:
				pigParam[3]=0.0;
				pigParam[5]=0.0;
			}
			else {
				//still flying: change vertical position according to speed (gravity approximation..)
				if (pigParam[3]<=2.0*JUMP_SPEED/7.0) pigParam[1]+=0.5;
				else pigParam[1]+=1.0;
				pigParam[3]+=JUMP_SPEED/7.0;
			}
		}

		// update enemies' parameters
		//no slime's update for the moment, still at level 1....
		for (int i=3; i>=0; i--) {
			if (!enemyAlive[i]) continue; // useless to update dead enemies
			if (i==0) {
				if (enemyParam[i*6]<=0.0) {
					enemyParam[i*6]=1.0;
					enemyParam[i*6+2]=5.0;
				}
				else if (enemyParam[i*6]>=4.0) {
					enemyParam[i*6]=3.0;
					enemyParam[i*6+2]=-5.0;
				}
				else if (enemyParam[i*6+2]>0.0) enemyParam[i*6]=minf(enemyParam[i*6]+1.0,4.0);
				else enemyParam[i*6]=maxf(enemyParam[i*6]-1.0,0.0);
			}
			else if (i==1) {
				if (enemyParam[i*6]<=21.0) {
					enemyParam[i*6]=22.0;
					enemyParam[i*6+2]=5.0;
				}
				else if (enemyParam[i*6]>=25.0) {
					enemyParam[i*6]=24.0;
					enemyParam[i*6+2]=-5.0;
				}
				else if (enemyParam[i*6+2]>0.0) enemyParam[i*6]=minf(enemyParam[i*6]+1.0,25.0);
				else enemyParam[i*6]=maxf(enemyParam[i*6]-1.0,21.0);
			}
			else {
				if (enemyParam[i*6]<=0.0) {
					enemyParam[i*6]=1.4;
					enemyParam[i*6+2]=7.0;
				}
				else if (enemyParam[i*6]>=25.0) {
					enemyParam[i*6]=23.6;
					enemyParam[i*6+2]=-7.0;
				}
				else if (enemyParam[i*6+2]>0.0) enemyParam[i*6]=minf(enemyParam[i*6]+1.4,25.0);
				else enemyParam[i*6]=maxf(enemyParam[i*6]-1.4,0.0);
			}
		}

		// update status counters if the pig doesn't fall to die:
		pigIsDead=false;
		if (pigParam[1]>=16.0) pigIsDead=true;
		else {
			for (int i=3; i>=0; i--) {
				if (enemyAlive[i] && fabs(pigParam[0]-enemyParam[i*6])<1.0 && fabs(pigParam[1]-enemyParam[i*6+1]+1.0)<1.0) {
					if (pigParam[5]>0.0 && !jumped) {
						// he is dead, and the pig fall on the floor
						// NOTE: later we could take into account mega-jump after killing..
						killed++;
						if (killed>=4) break;
						enemyAlive[i]=false;
						pigParam[1]=enemyParam[i*6+1];
						pigParam[3]=0.0;
						pigParam[5]=0.0;
						enemyParam[i*6]=0.0;
						enemyParam[i*6+1]=0.0;
						enemyParam[i*6+2]=0.0;
						enemyParam[i*6+3]=0.0;
					}
					else {
						// if the pig meet an enemy while he isn't falling, make it falling one more step:
						if (pigParam[1]+2.0>=16.0) pigIsDead=true;
						else {
							// it's OK, the pig just moves through the floor, and falls
							pigParam[1]+=2.0;
							pigParam[2]=0.0;
							pigParam[3]=0.0;
							pigParam[4]=0.0;
							pigParam[5]=GRAV_ACC;
						}
					}
				}
			}
		}

		if (pigIsDead) {
			// if dead, do what is necessary:
			usedLives++;
			if (usedLives>=5) break;
			lastLeft=0;
			lastRight=0;
			pigParam[0]=12.0;
			pigParam[1]=6.0;
			pigParam[2]=0.0;
			pigParam[3]=0.0;
			pigParam[4]=0.0;
			pigParam[5]=0.0;
		}
		else {
			if (pigParam[5]==0.0 && nextAction<2) {
				// just moved left or right: check where we arrive, maybe we will fall !
				if (!map1[(int)floor(minf(pigParam[0],24.0)+MAP_W*minf(pigParam[1],16.0)+MAP_W)]) {
					pigParam[3]=0.0;
					pigParam[5]=GRAV_ACC;
				}
			}
		}
	}

	if (usedLives>=5 || countTiles>100) { 
		org->fitness=killed-0.1*usedLives;
		return;
	}

	org->winner=(killed==4);
	org->fitness=killed-0.1*usedLives;
	if (org->winner) org->fitness+=1.0;
}
*/
