#include "IDCounters.h"
#include "Functions.h"
#include "Innovation.h"
#include "Organism.h"
#include "Neat.h"
#include "Population.h"
#include "Experiments.h"
#include "DrawNetwork.h"
#include "Pig.h"
#include "Fighter.h"
#include <cstdlib>

int main(int argc, char** argv) {

	Functions funcs; // initialize the random generator
	if (argc==1) {
		cerr << "error, NEAT.exe takes at least 1 argument" << endl;
		return -1;
	}

	if (!strcmp(argv[1],"play-pig")) {
		// means that we want to play the pig game,
		// either with network or human player
		return pig_main(argc,argv);
	}

	else if (!strcmp(argv[1],"play-fight")) {
		// means that we want to play the fight game, against network
		init_fight();
		return fight_main(argc,argv,false);
	}
	else if (!strcmp(argv[1],"draw")) {
		// just draw a network's graph:
		ifstream fileOrg(argv[2]);
		graph(new Organism(fileOrg),argv[3][0]!='0');
		return 0;
	}

	// if not play, means that we want to launch some experiment
	IDCounters id_counts; // initialize the counters
	Innovation innov; // initialize the innovations
	new NEAT(argv[1]); // load the experiment's parameters
	int indexL=0,indexR;
	while (argv[1][indexL]!='/') indexL++;
	indexR=++indexL;
	while (argv[1][indexR]!='_') indexR++;
	char* nameExp=(char*)malloc((indexR-indexL+1)*sizeof(char));
	for (int i=indexL; i<indexR; i++) nameExp[i-indexL]=argv[1][i];
	nameExp[indexR-indexL]=0;
	Organism** tabOrg=0;
	// launch the experiment:
	if (strcmp(nameExp,"pig")==0) tabOrg=test(nameExp,atoi(argv[2]));
	else if (strcmp(nameExp,"fight")==0) {
		init_fight();
		if (argv[2][0]!='0') fight_main(argc,argv,true);
		tabOrg=test(nameExp,atoi(argv[2]));
	}
	else tabOrg=test(nameExp);

	if (tabOrg[0]!=0) {
		cout << endl << "Organism " << tabOrg[0]->id << " is the winner, with fitness " << tabOrg[0]->fitness << endl;
		graph(tabOrg[0],false);
		tabOrg[0]->print_to_file("orgs/the_best.txt");
	}
	else if (tabOrg[1]!=0) {
		cout << endl << "Organism " << tabOrg[1]->id << " is the best candidate, with fitness " << tabOrg[1]->fitness << endl;
		graph(tabOrg[1],false);
		tabOrg[1]->print_to_file("orgs/best_fit.txt");
	}

	return 0;
}
